// File           : ControlLocalizer.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2006
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.IO;
using System.Resources;
using System.Collections;
using System.ComponentModel;
using System.Text.RegularExpressions;
using Environment;
using System.Globalization;

namespace ASPNETControlLocalizer
{
    /// <summary>
    /// ControlLocalizer is a utility class for localizing ASP.NET controls using resources in the .NET Framework Language Packs
    /// </summary>
    public class ControlLocalizer
    {
        private static Hashtable resourceAssemblies = new Hashtable();

        /// <summary>
        /// UpdateResources updates the localized resources in the given resource file for a given ASPX file for the given culture
        /// </summary>
        /// <param name="aspxFilename">The ASPX file containing the controls to be localized</param>
        /// <param name="resourceFilename">The resx resource file to be updated with the localized resources</param>
        /// <param name="cultureName">The culture to use to localize the resources for</param>
        /// <remarks>If the target resource filename does not exist it is created.</remarks>
        public static void UpdateResources(string aspxFilename, string resourceFilename, string cultureName)
        {
            AspNetControl[] aspNetControls = GetControls(aspxFilename);
            foreach (AspNetControl aspNetControl in aspNetControls)
            {
                UpdateResources(resourceFilename, cultureName, aspNetControl);
            }
        }
        /// <summary>
        /// UpdateResources updates the resources in the given resource file for the given culture for the given ASP.NET control
        /// </summary>
        /// <param name="resourceFilename">The resx resource file to be updated with the localized resources</param>
        /// <param name="cultureName">The culture to use to localize the resources for</param>
        /// <param name="aspNetControl">The AspNetcontrol to update resources for</param>
        /// <remarks>If the target resource filename does not exist it is created.</remarks>
        public static void UpdateResources(string resourceFilename, string cultureName, AspNetControl aspNetControl)
        {
            string controlName = aspNetControl.ControlType;
            string metaResourceKey = aspNetControl.MetaResourceKey;
            bool changeMade = false;
            LocalizableProperties localizableProperties = GetLocalizableProperties(controlName);

            Hashtable resourceStrings = GetResources(cultureName, localizableProperties);
            Hashtable oldResourceStrings = GetResourceEntries(resourceFilename);
            Hashtable newResourceStrings = (Hashtable)oldResourceStrings.Clone();
            foreach (DictionaryEntry resourceEntry in resourceStrings)
            {
                // if resourceEntry.Key is "Login_FailureText" then propertyName is "FailureText"
                int localizablePropertyIndex = 
                    localizableProperties.IndexOfResourceEntryName(resourceEntry.Key.ToString());
                if (localizablePropertyIndex > -1)
                {
                    LocalizableProperty localizableProperty = localizableProperties[localizablePropertyIndex];
                    if (localizableProperty.DefaultValue == null || 
                        localizableProperty.DefaultValue.ToString() != String.Empty)
                    {
                        string propertyName = localizableProperty.PropertyName;
                        // look for the key in the resource set

                        // key is something like "Login1Resource1.FailureText"
                        string key = metaResourceKey + "." + propertyName;
                        string translatedString = resourceEntry.Value.ToString();
                        if (oldResourceStrings.ContainsKey(key))
                        {
                            string oldResourceString = oldResourceStrings[key].ToString();
                            // this entry already exists in the existing resource entries but it should only be updated
                            // if it is not the same as the .NET Framework Language Pack value
                            if (String.Compare(oldResourceString, translatedString, StringComparison.Ordinal) != 0)
                            {
                                newResourceStrings[key] = translatedString;
                                changeMade = true;
                            }
                        }
                        else
                        {
                            // this entry is missing from the existing resource entries
                            newResourceStrings.Add(key, translatedString);
                            changeMade = true;
                        }
                    }
                }
            }

            if (changeMade)
            {
                using (ResXResourceWriter writer = new ResXResourceWriter(resourceFilename))
                {
                    IDictionaryEnumerator enumerator = newResourceStrings.GetEnumerator();
                    while (enumerator.MoveNext())
                    {
                        DictionaryEntry dictionaryEntry = (DictionaryEntry)enumerator.Current;
                        writer.AddResource(dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
                    }
                    writer.Generate();
                    writer.Close();
                }
            }
        }
        private static LocalizableProperties GetLocalizableProperties(string controlName)
        {
            // TODO: Verify that it is safe to assume that the control is in "System.Web.UI.WebControls"
            Type controlType = Type.GetType(String.Format(
                "System.Web.UI.WebControls.{0}, System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", controlName));
            if (controlType == null)
                return null;
            LocalizableProperties localizableProperties = new LocalizableProperties();
            foreach (PropertyInfo propertyInfo in controlType.GetProperties(BindingFlags.Instance | BindingFlags.Public))
            {
                object[] localizableAttributes = propertyInfo.GetCustomAttributes(typeof(LocalizableAttribute), true);
                if (localizableAttributes.GetLength(0) > 0)
                {
                    // this property is localizable
                    object defaultValue = null;
                    object[] defaultValueAttributes = propertyInfo.GetCustomAttributes(typeof(DefaultValueAttribute), true);
                    if (defaultValueAttributes.GetLength(0) > 0)
                    {
                        object value = ((DefaultValueAttribute)defaultValueAttributes[0]).Value;
                        if (value != null && value.GetType().Name == "String")
                            defaultValue = value;
                    }
                    // (WebSysDefaultValueAttribute is internal to System.Web.dll so it can't be referred to by its type)
                    object[] webSysDefaultValueAttributes = GetCustomAttributes(propertyInfo, "WebSysDefaultValueAttribute");
                    if (webSysDefaultValueAttributes.GetLength(0) > 0)
                        localizableProperties.Add(new LocalizableProperty(propertyInfo.Name, GetWebSysDefaultValue(webSysDefaultValueAttributes[0]), defaultValue));
                    else
                        localizableProperties.Add(new LocalizableProperty(propertyInfo.Name, controlName + "_" + propertyInfo.Name, defaultValue));
                }
            }
            return localizableProperties;
        }
        private static string GetWebSysDefaultValue(object webSysDefaultValueAttribute)
        {
            // Get the private "value" field of the internal DefaultValueAttribute object.
            // WebSysDefaultValueAttribute inherits from DefaultValueAttribute.
            // DefaultValueAttribute includes a property called Value but this property *MUST NOT*
            // be accessed. The Value property reads the private value field and loads the corresponding
            // resource string and assigns that value back to the private value field. Consequently
            // this is a one way trip because you can't ever get back to the name of the string
            // that identifies the resource entry. Instead you must access the private value field
            // *BEFORE* the public Value property is ever accessed.
            FieldInfo fieldInfo = webSysDefaultValueAttribute.GetType().BaseType.GetField("value",
                BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.FlattenHierarchy);
            return fieldInfo.GetValue(webSysDefaultValueAttribute).ToString();
        }
        private static object[] GetCustomAttributes(PropertyInfo propertyInfo, string className)
        {
            object[] allAttributes = propertyInfo.GetCustomAttributes(true);
            if (allAttributes.GetLength(0) == 0)
                return new object[] { };

            List<object> specificAttributes = new List<object>();
            foreach (object attribute in allAttributes)
            {
                if (attribute.GetType().Name == className)
                    specificAttributes.Add(attribute);
            }
            if (specificAttributes.Count == 0)
                return new object[] { };

            return (object[])specificAttributes.ToArray();
        }
        /// <summary>
        /// GetResourceEntries gets a Hashtable of resource entries for the given resource file
        /// </summary>
        /// <param name="resourceFilename">The resource file to get the resource entries for</param>
        /// <returns>A Hashtable of resource entries for the given resource file</returns>
        protected static Hashtable GetResourceEntries(string resourceFilename)
        {
            if (File.Exists(resourceFilename))
            {
                ResourceSet resourceSet;
                using (ResXResourceReader reader = new ResXResourceReader(resourceFilename))
                {
                    resourceSet = new ResourceSet(reader);
                    reader.Close();
                }
                return GetResourceSetTable(resourceSet);
            }
            else
                return new Hashtable();
        }
        /// <summary>
        /// GetResources gets a Hashtable of localized resource entries for the given culture for the given control type
        /// </summary>
        /// <param name="cultureName">The culture name to get the localized resource entries for (e.g. "es")</param>
        /// <param name="controlName">The control type to get the localized resource entries for (e.g. "Login")</param>
        /// <returns>A Hashtable of localized resource entries for the given culture for the given control type</returns>
        /// <remarks>GetResources looks through the relevant .NET Framework Language Pack for entries such as this one:-
        /// Login_FailureText: Texto que se va a mostrar cuando falle el intento de inicio de sesin.
        /// </remarks>
        protected static Hashtable GetResources(string cultureName, string controlName)
        {
            string languagePackCultureName;
            CultureInfo cultureInfo = new CultureInfo(cultureName);
            if (cultureInfo.IsNeutralCulture)
                // the culture is a neutral culture (e.g. "fr")
                // but language packs are always specific cultures
                // so we need to check for the specific culture (e.g. "fr-FR")
                languagePackCultureName = CultureInfo.CreateSpecificCulture(cultureName).Name;
            else
                languagePackCultureName = cultureName;

            if (! FrameworkHelper.IsLanguagePackInstalled(languagePackCultureName))
                throw new LanguagePackNotInstalledException(
                    ControlLocalizerResources.LanguagePackNotInstalledException, cultureName);

            if (resourceAssemblies[cultureName] == null)
            {
                string versionNumber = System.Environment.Version.Major + "." + System.Environment.Version.MajorRevision + "." + System.Environment.Version.Build;
                string assemblyName = FrameworkHelper.GetFrameworkPath() + "v" + versionNumber + @"\" + cultureName + @"\System.Web.resources.dll";
                resourceAssemblies[cultureName] = Assembly.LoadFrom(assemblyName);
            }

            Assembly resourceAssembly = (Assembly) resourceAssemblies[cultureName];
            Stream resourceStream = resourceAssembly.GetManifestResourceStream("System.Web." + cultureName + ".resources");
            Hashtable resourceStrings = new Hashtable();
            try
            {
                ResourceSet resourceSet = new ResourceSet(resourceStream);

                IDictionaryEnumerator dictionaryEnumerator = resourceSet.GetEnumerator();
                while (dictionaryEnumerator.MoveNext())
                {
                    string key = dictionaryEnumerator.Key.ToString();
                    if (key.StartsWith(controlName + "_"))
                    {
                        object resourceValue = resourceSet.GetObject(key, true);
                        resourceStrings.Add(key, resourceValue);
                    }
                }
            }
            finally
            {
                resourceStream.Close();
            }
            return resourceStrings;
        }
        /// <summary>
        /// GetResources gets a Hashtable of localized resource entries for the given culture for the given localizable properties
        /// </summary>
        /// <param name="cultureName">The culture name to get the localized resource entries for (e.g. "es")</param>
        /// <param name="localizableProperties">The LocalizableProperties list of localizable properties to get the localized resource entries for</param>
        /// <returns>A Hashtable of localized resource entries for the given culture for the given localizable properties</returns>
        /// <remarks>GetResources looks through the relevant .NET Framework Language Pack for entries such as this one:-
        /// Login_FailureText: Texto que se va a mostrar cuando falle el intento de inicio de sesin.
        /// </remarks>
        protected static Hashtable GetResources(string cultureName, LocalizableProperties localizableProperties)
        {
            string languagePackCultureName;
            CultureInfo cultureInfo = new CultureInfo(cultureName);
            if (cultureInfo.IsNeutralCulture)
                // the culture is a neutral culture (e.g. "fr")
                // but language packs are always specific cultures
                // so we need to check for the specific culture (e.g. "fr-FR")
                languagePackCultureName = CultureInfo.CreateSpecificCulture(cultureName).Name;
            else
                languagePackCultureName = cultureName;

            if (!FrameworkHelper.IsLanguagePackInstalled(languagePackCultureName))
                throw new LanguagePackNotInstalledException(
                    ControlLocalizerResources.LanguagePackNotInstalledException, cultureName);

            if (resourceAssemblies[cultureName] == null)
            {
                string versionNumber = System.Environment.Version.Major + "." + System.Environment.Version.MajorRevision + "." + System.Environment.Version.Build;
                string assemblyName = FrameworkHelper.GetFrameworkPath() + "v" + versionNumber + @"\" + cultureName + @"\System.Web.resources.dll";
                resourceAssemblies[cultureName] = Assembly.LoadFrom(assemblyName);
            }

            Assembly resourceAssembly = (Assembly)resourceAssemblies[cultureName];
            Stream resourceStream = resourceAssembly.GetManifestResourceStream("System.Web." + cultureName + ".resources");
            Hashtable resourceStrings = new Hashtable();
            try
            {
                ResourceSet resourceSet = new ResourceSet(resourceStream);

                IDictionaryEnumerator dictionaryEnumerator = resourceSet.GetEnumerator();
                while (dictionaryEnumerator.MoveNext())
                {
                    string key = dictionaryEnumerator.Key.ToString();
                    if (localizableProperties.IndexOfResourceEntryName(key) > -1)
                    {
                        object resourceValue = resourceSet.GetObject(key, true);
                        resourceStrings.Add(key, resourceValue);
                    }
                }
            }
            finally
            {
                resourceStream.Close();
            }
            return resourceStrings;
        }
        /// <summary>
        /// GetResourceSetTable gets the Hashtable which contains the resource entries in the ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to get the Hashtable from</param>
        /// <returns>The Hashtable which contains the resource entries in the ResourceSet</returns>
        protected static Hashtable GetResourceSetTable(ResourceSet resourceSet)
        {
            FieldInfo fieldInfo =
                resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
            if (fieldInfo != null)
            {
                object tableObject = fieldInfo.GetValue(resourceSet);
                if (tableObject != null)
                    return (Hashtable)tableObject;
            }
            return null;
        }

        /// <summary>
        /// aspNetcontrols is a temporary generic list of AspNetControl objects
        /// </summary>
        protected static List<AspNetControl> aspNetControls;

        /// <summary>
        /// GetControls gets an array of AspNetControl objects from an ASPX file
        /// </summary>
        /// <param name="aspxFilename">The ASPX file to get a list of localizable controls from</param>
        /// <returns>An array of AspNetControls containing all of the localizable controls in the given ASPX file</returns>
        /// <remarks>GetControls reads an ASPX file looking for controls with "meta:resourcekey" entries such as the following:-
        /// asp:Login ID="Login1" runat="server" meta:resourcekey="Login1Resource1"
        /// From this entry it adds an AspNetControl with a control type of "Login" and a meta resource key of "Login1Resource1"
        /// </remarks>
        protected static AspNetControl[] GetControls(string aspxFilename)
        {
            //TODO: Refactor the GetControls method so that it Regex.Replace returns a MatchCollection
            string aspxFile = ReadFileToString(aspxFilename);
            aspNetControls = new List<AspNetControl>();
            string aspStartControlPattern = @"[" + Regex.Escape("<") + @"][a][s][p][:]\w*";
            string aspInnerControlPattern = @"[\w\s:" + Regex.Escape("=") + Regex.Escape(@"""") + @"]*";
            string aspEndControlPattern   = @"[" + Regex.Escape(">") + @"]";
            string aspControlPattern = aspStartControlPattern + aspInnerControlPattern + aspEndControlPattern;
            Regex regex = new Regex(aspControlPattern, RegexOptions.IgnoreCase);
            regex.Replace(aspxFile, new MatchEvaluator(GetControlsMatchEvaluator));
            return (AspNetControl[]) aspNetControls.ToArray();
        }
        /// <summary>
        /// GetControlsMatchEvaluator is a match evaluator used by GetControls
        /// </summary>
        /// <param name="match"></param>
        /// <returns></returns>
        protected static String GetControlsMatchEvaluator(Match match)
        {
            int metaResourceKeyIndex = match.Value.IndexOf("meta:resourcekey=");
            if (metaResourceKeyIndex > -1)
            {
                // get the metaResourceKey (e.g. "Login1Resource1") from the control
                string metaResourceKey = match.Value.Substring(metaResourceKeyIndex + 18);
                int metaResourceKeySpace = metaResourceKey.IndexOf(@"""");
                metaResourceKey = metaResourceKey.Substring(0, metaResourceKeySpace);

                // get the control type (e.g. "Login") from the control
                string controlType = match.Value.Substring(5);
                int controlTypeSpace = controlType.IndexOf(" ");
                controlType = controlType.Substring(0, controlTypeSpace);

                aspNetControls.Add(new AspNetControl(controlType, metaResourceKey));
            }
            return match.Value;
        }
        /// <summary>
        /// ReadFileToString reads the contents of a file into a string
        /// </summary>
        /// <param name="filename">The name of the file to read</param>
        /// <returns>The contents of a file as a string</returns>
        protected static string ReadFileToString(string filename)
        {
            using (StreamReader streamReader = new StreamReader(filename))
            {
                return streamReader.ReadToEnd();
            }
        }
    }

    /// <summary>
    /// AspNetControl represents an ASP.NET control in an ASPX file
    /// </summary>
    public class AspNetControl
    {
        private string controlType;

        private string metaResourceKey;

        /// <summary>
        /// Constructs an AspNetControl object
        /// </summary>
        /// <param name="controlType">The unqualified type of the control</param>
        /// <param name="metaResourceKey">The meta:resourcekey used by the control</param>
        public AspNetControl(string controlType, string metaResourceKey)
        {
            this.controlType = controlType;
            this.metaResourceKey = metaResourceKey;
        }
        /// <summary>
        /// The unqualified type of the control
        /// </summary>
        public string ControlType
        {
            get { return controlType; }
            set { controlType = value; }
        }
        /// <summary>
        /// The meta:resourcekey used by the control
        /// </summary>
        public string MetaResourceKey
        {
            get { return metaResourceKey; }
            set { metaResourceKey = value; }
        }
    }
    /// <summary>
    /// LocalizableProperty represents a property that is localizable
    /// </summary>
    public class LocalizableProperty
    {
        private string propertyName;
        private string resourceEntryName;
        private object defaultValue;

        /// <summary>
        /// Constructs a LocalizableProperty object
        /// </summary>
        /// <param name="propertyName">The name of the localizable property</param>
        /// <param name="resourceEntryName">The name of the properties' corresponding resource entry</param>
        /// <param name="defaultValue">The default value (if any) of the property</param>
        public LocalizableProperty(string propertyName, string resourceEntryName, object defaultValue)
        {
            this.propertyName = propertyName;
            this.resourceEntryName = resourceEntryName;
            this.defaultValue = defaultValue;
        }
        /// <summary>
        /// DefaultValue is the default value (if any) of the property
        /// </summary>
        public object DefaultValue
        {
            get { return defaultValue; }
            set { defaultValue = value; }
        }
        /// <summary>
        /// PropertyName is the name of the localizable property
        /// </summary>
        public string PropertyName
        {
            get { return propertyName; }
            set { propertyName = value; }
        }
        /// <summary>
        /// ResourceEntryName is the name of the properties' corresponding resource entry
        /// </summary>
        public string ResourceEntryName
        {
            get { return resourceEntryName; }
            set { resourceEntryName = value; }
        }
    }

    /// <summary>
    /// LocalizableProperties is a list of LocalizableProperty objects
    /// </summary>
    public class LocalizableProperties : List<LocalizableProperty>
    {
        /// <summary>
        ///  Constructs a LocalizableProperties object
        /// </summary>
        public LocalizableProperties()
            : base()
        {
        }
        /// <summary>
        /// Get the index position of the LocalizableProperty corresponding to the given resource entry name
        /// </summary>
        /// <param name="resourceEntryName">The name of the resource entry</param>
        /// <returns>The index position of the LocalizableProperty corresponding to the given resource entry name</returns>
        public int IndexOfResourceEntryName(string resourceEntryName)
        {
            for (int index = 0; index < Count; index++)
            {
                if (String.CompareOrdinal(this[index].ResourceEntryName, resourceEntryName) == 0)
                    return index;
            }
            return -1;
        }
    }

    /// <summary>
    /// LanguagePackNotInstalledException is an exception that is thrown when a required .NET Framework Language Pack is not installed
    /// </summary>
    public class LanguagePackNotInstalledException : Exception
    {
        private string cultureName;
        /// <summary>
        /// Constructs a LanguagePackNotInstalledException object
        /// </summary>
        /// <param name="message">The message of the exception</param>
        /// <param name="cultureName">The name of the culture of the .NET Framework Language Pack</param>
        public LanguagePackNotInstalledException(string message, string cultureName)
            : base(message)
        {
            this.cultureName = cultureName;
        }
        /// <summary>
        /// The name of the culture of the .NET Framework Language Pack
        /// </summary>
        public string CultureName
        {
            get { return cultureName; }
            set { cultureName = value; }
        }
    }
}
