// File           : ConvertHTMLControlPlugIn.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : April 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using DevExpress.CodeRush.Core;
using DevExpress.CodeRush.PlugInCore;
using DevExpress.CodeRush.StructuralParser;
using System.Text;

namespace I18NRefactorings
{
    /// <summary>
    /// ConvertHTMLControlPlugIn is a refactoring for the Visual Studio HTML editor to convert HTML controls
    /// into ASP.NET server controls
    /// </summary>
    public partial class ConvertHTMLControlPlugIn : StandardPlugIn
    {
        private AttributeTranslation[] attributeTranslations;

        public override void InitializePlugIn()
        {
            // initialize the list of attribute translations (to convert HTML attributes to ASP.NET properties)
            attributeTranslations = new AttributeTranslation[] {
                    new AttributeTranslation("value", "Text", new string[] {"OPTION"}, false),
                    new AttributeTranslation("class", "CssClass"),
                    new AttributeTranslation("alt", "AlternateText"),
                    new AttributeTranslation("src", "ImageUrl")
                };

            base.InitializePlugIn();
        }
        public override void FinalizePlugIn()
        {
            base.FinalizePlugIn();
        }
        private void refactoringProvider1_LanguageSupported(LanguageSupportedEventArgs ea)
        {
            ea.Handled = (ea.LanguageID == "HTML");
        }
        private void refactoringProvider1_CheckAvailability(object sender, CheckContentAvailabilityEventArgs ea)
        {
            ea.Available = (ea.ActiveLanguage != null && ea.ActiveLanguage == "HTML" && 
                ea.Element != null && (ea.Element.ElementType == LanguageElementType.HtmlText ||
                (ea.Element.ElementType == LanguageElementType.HtmlElement &&
                CanRefactor((HtmlElement)ea.Element))));
        }
        private void refactoringProvider1_Apply(object sender, ApplyContentEventArgs ea)
        {
            if (ea.ActiveLanguage == null || ea.ActiveLanguage != "HTML" || ea.Element == null)
            {
                // do nothing
            }
            else if (ea.Element.ElementType == LanguageElementType.HtmlText)
            {
                // this is free form static text
                HtmlText htmlText = (HtmlText)ea.Element;
                ServerControlElement serverControlElement = new ServerControlElement();
                serverControlElement.Name = "asp:Label";
                serverControlElement.AddDetailNode(NewHtmlAttribute("runat", "server"));
                serverControlElement.AddNode(htmlText);
                CodeRush.Documents.ActiveTextDocument.Replace(
                    htmlText.Range,
                    CodeRush.Language.GenerateElement(serverControlElement).Trim(),
                    ConvertHTMLControlPlugInResources.ReplaceOperationMessage);
            }
            else if (ea.Element.ElementType == LanguageElementType.HtmlElement &&
                CanRefactor((HtmlElement)ea.Element))
            {
                // this is an HTML element that can be refactored
                HtmlElement htmlElement = (HtmlElement)ea.Element;
                ServerControlElement serverControlElement = new ServerControlElement();
                serverControlElement.Name = GetServerControlType(htmlElement);
                serverControlElement.AddDetailNode(NewHtmlAttribute("runat", "server"));
                HtmlAttribute additionalHtmlAttribute = GetTypeAdditionalAttribute(htmlElement);
                if (additionalHtmlAttribute != null)
                    // there is some additional attribute that needs to be set to get this type right
                    serverControlElement.AddDetailNode(additionalHtmlAttribute);

                bool idAttributeAdded = false;
                // process (and potentially convert) all of the HTML attributes into equivalent ASP.NET attributes
                foreach (HtmlAttribute htmlAttribute in htmlElement.Attributes)
                {
                    if (String.Compare(htmlAttribute.Name, "type", StringComparison.OrdinalIgnoreCase) == 0)
                    {
                        // we've already handled this attribute so we can ignore it
                    }
                    else if (String.Compare(htmlAttribute.Name, "onclick", StringComparison.OrdinalIgnoreCase) == 0)
                        if (IsSubmitButton(htmlElement))
                            // change "onclick" attribute name to "OnClientClick"
                            // (do not add "return false;" to the JavaScript because this is a submit button)
                            serverControlElement.AddDetailNode(NewHtmlAttribute(
                                "OnClientClick", htmlAttribute.Value));
                        else
                            // change "onclick" attribute name to "OnClientClick" and potentially add "return false;"
                            serverControlElement.AddDetailNode(NewHtmlAttribute(
                                "OnClientClick", AddReturnFalse(htmlAttribute.Value)));
                    else
                    {
                        AttributeTranslation attributeTranslation =
                            GetAttributeTranslation(attributeTranslations, htmlAttribute.Name, htmlElement);
                        if (attributeTranslation != null)
                            // this is an attribute that needs to be translated
                            serverControlElement.AddDetailNode(
                                NewHtmlAttribute(attributeTranslation.AspnetAttribute, htmlAttribute.Value));
                        else if (String.Compare(htmlAttribute.Name, "selected", StringComparison.OrdinalIgnoreCase) == 0)
                        {
                            // The "selected" HTML attribute value needs to be converted from "selected" to "True"
                            if (String.Compare(htmlAttribute.Value.ToString(), "selected", StringComparison.OrdinalIgnoreCase) == 0)
                                serverControlElement.AddDetailNode(NewHtmlAttribute("Selected", "True"));
                            else
                                // if this happens then there is something wrong with the "selected" attribute
                                // so just pass the value straight through and let the developer deal with it
                                serverControlElement.AddDetailNode(htmlAttribute);
                        }
                        else
                        {
                            // add the attribute as-is
                            serverControlElement.AddDetailNode(htmlAttribute);
                            if (String.Compare(htmlAttribute.Name, "id", StringComparison.OrdinalIgnoreCase) == 0)
                                // we've found the ID attribute so no need to generate a new one
                                idAttributeAdded = true;
                        }
                    }
                }
                if (!idAttributeAdded)
                {
                    serverControlElement.AddDetailNode(NewHtmlAttribute("ID", GetNewID(serverControlElement)));
                }
                if (htmlElement.Nodes.Count > 0)
                {
                    // add all of the child nodes "as-is" (i.e. without converting them)
                    foreach (object node in htmlElement.Nodes)
                    {
                        serverControlElement.AddNode((LanguageElement) node);
                    }
                }
                else
                    serverControlElement.IsEmptyTag = true;

                CodeRush.Documents.ActiveTextDocument.Replace(
                    htmlElement.Range,
                    CodeRush.Language.GenerateElement(serverControlElement).Trim(),
                    ConvertHTMLControlPlugInResources.ReplaceOperationMessage);
            }
        }
        private string AddReturnFalse(string javascript)
        {
            // Add "return false;" to the javascript string. This is done to ensure that the new ASP.NET button
            // behaves in the same way as the original HTML INPUT button. The original HTML INPUT button doesn't
            // perform a postback but the ASP.NET button will so "return false;" is added to prevent the postback
            // and ensure that the behaviour is the same.
            string javascriptLower = javascript.TrimEnd().ToLower();
            if (javascriptLower.EndsWith(";"))
                // strip the trailing semi-colon
                javascriptLower = javascriptLower.Substring(0, javascriptLower.Length - 1);

            if (!(javascriptLower.EndsWith("return false") || javascriptLower.EndsWith("return true")))
            {
                // the script does not end with "return false" or "return true" so add a "return" statement
                if (javascript.TrimEnd().EndsWith(";"))
                    javascript += "return false;";
                else
                    javascript += ";return false;";
            }
            return javascript;
        }
        private AttributeTranslation GetAttributeTranslation(AttributeTranslation[] translations, string htmlAttributeName, HtmlElement htmlElement)
        {
            foreach (AttributeTranslation translation in translations)
            {
                if (String.Compare(htmlAttributeName, translation.HtmlAttribute, StringComparison.OrdinalIgnoreCase) == 0 &&
                    ! IsHtmlElementException(translation, htmlElement))
                    return translation;
            }
            return null;
        }
        private bool IsHtmlElementException(AttributeTranslation translation, HtmlElement htmlElement)
        {
            if (! translation.IncludeHidden)
            {
                // exclude hidden INPUT controls
                if (IsHtmlElementHidden(htmlElement))
                    // it is a hidden INPUT control so it is an exception to the rule
                    return true;
            }
            if (translation.HtmlElementExceptions == null)
                return false;

            foreach (string htmlElementException in translation.HtmlElementExceptions)
            {
                if (String.Compare(htmlElementException, htmlElement.Name, StringComparison.OrdinalIgnoreCase) == 0)
                    return true;
            }

            return false;
        }
        private LanguageElement NewHtmlText(string text)
        {
            HtmlText htmlText = new HtmlText();
            htmlText.Text = text;
            return htmlText;
        }
        // This mechanism is a bit crude and tragic. It takes a poor guess that there are no existing
        // controls that have the same name as their control type (e.g. "Label") and are suffixed with
        // a numeric value of more than 1000 (e.g. "Label1001").
        private int controlCount = 1000;
        private string GetNewID(ServerControlElement serverControlElement)
        {
            return serverControlElement.Name.Substring(4) + (controlCount + 1).ToString();
        }
        private bool IsSubmitButton(HtmlElement htmlElement)
        {
            if (String.Compare(htmlElement.Name, "INPUT", StringComparison.OrdinalIgnoreCase) == 0)
            {
                foreach (HtmlAttribute htmlAttribute in htmlElement.Attributes)
                {
                    if (String.Compare(htmlAttribute.Name, "type", StringComparison.OrdinalIgnoreCase) == 0)
                        // if the type attribute is SUBMIT then this is a submit button
                        return (String.Compare(htmlAttribute.Value, "SUBMIT", StringComparison.OrdinalIgnoreCase) == 0);
                }
            }
            return false;
        }
        private string GetServerControlType(HtmlElement htmlElement)
        {
            string htmlElementName = htmlElement.Name.ToUpper();
            if (htmlElementName == "INPUT")
            {
                // the server control type is dictated by the "type" HTML attribute
                foreach (HtmlAttribute htmlAttribute in htmlElement.Attributes)
                {
                    if (String.Compare(htmlAttribute.Name, "type", StringComparison.OrdinalIgnoreCase) == 0)
                    {
                        // convert the HTML type attribute to an ASP.NET server control type
                        return HtmlTypeAttributeToServerControlType(htmlAttribute.Value);
                    }
                }
            }
            else if (htmlElementName == "A")
                return "asp:HyperLink";
            else if (htmlElementName == "LABEL")
                return "asp:Label";
            else if (htmlElementName == "IMG")
                return "asp:Image";
            else if (htmlElementName == "TEXTAREA")
                return "asp:TextBox";
            else if (htmlElementName == "BUTTON")
                return "asp:Button";
            else if (htmlElementName == "OPTION")
                return "asp:ListItem";
            else if (htmlElementName == "SELECT")
            {
                // If the control has a size attribute then this should translate to a ListBox
                foreach (HtmlAttribute htmlAttribute in htmlElement.Attributes)
                {
                    if (String.Compare(htmlAttribute.Name, "size", StringComparison.OrdinalIgnoreCase) == 0)
                        return "asp:ListBox";
                }
                // there is no size attribute so it is a DropDownList
                return "asp:DropDownList";
            }
            return "asp:TextBox";
        }
        private HtmlAttribute GetTypeAdditionalAttribute(HtmlElement htmlElement)
        {
            // only INPUT and TEXTAREA controls have additional attributes
            string htmlElementName = htmlElement.Name.ToUpper();
            if (htmlElementName == "INPUT")
            {
                foreach (HtmlAttribute htmlAttribute in htmlElement.Attributes)
                {
                    if (String.Compare(htmlAttribute.Name, "type", StringComparison.OrdinalIgnoreCase) == 0)
                    {
                        string htmlTypeAttribute = htmlAttribute.Value.ToUpper();
                        if (htmlTypeAttribute == "PASSWORD")
                            return NewHtmlAttribute("TextMode", "Password");

                        return null;
                    }
                }
            }
            else if (htmlElementName == "TEXTAREA")
                return NewHtmlAttribute("TextMode", "MultiLine");

            return null;
        }
        private bool CanRefactor(HtmlElement htmlElement)
        {
            string htmlElementName = htmlElement.Name.ToUpper();
            if (htmlElementName == "A")
                return true;
            if (htmlElementName == "INPUT" && ! IsHtmlElementReset(htmlElement))
                return true;
            if (htmlElementName == "LABEL")
                return true;
            if (htmlElementName == "IMG")
                return true;
            if (htmlElementName == "TEXTAREA")
                return true;
            if (htmlElementName == "BUTTON")
                return true;
            if (htmlElementName == "OPTION")
                return true;
            if (htmlElementName == "SELECT")
                return true;
            else
                return false;
        }
        private bool IsHtmlElementHidden(HtmlElement htmlElement)
        {
            return IsHtmlElementType(htmlElement, "hidden");
        }
        private bool IsHtmlElementReset(HtmlElement htmlElement)
        {
            return IsHtmlElementType(htmlElement, "reset");
        }
        private bool IsHtmlElementType(HtmlElement htmlElement, string typeValue)
        {
            foreach (HtmlAttribute htmlAttribute in htmlElement.Attributes)
            {
                if (String.Compare(htmlAttribute.Name, "type", StringComparison.OrdinalIgnoreCase) == 0)
                {
                    return String.Compare(htmlAttribute.Value, typeValue, StringComparison.OrdinalIgnoreCase) == 0;
                }
            }
            return false;
        }
        private string HtmlTypeAttributeToServerControlType(string htmlTypeAttribute)
        {
            htmlTypeAttribute = htmlTypeAttribute.ToUpper();
            if (htmlTypeAttribute == "TEXT")
                return "asp:TextBox";
            else if (htmlTypeAttribute == "BUTTON")
                return "asp:Button";
            else if (htmlTypeAttribute == "CHECKBOX")
                return "asp:CheckBox";
            else if (htmlTypeAttribute == "IMAGE")
                return "asp:ImageButton";
            else if (htmlTypeAttribute == "PASSWORD")
                return "asp:TextBox";
            else if (htmlTypeAttribute == "RADIO")
                return "asp:RadioButton";
            else if (htmlTypeAttribute == "SUBMIT")
                return "asp:Button";
            else if (htmlTypeAttribute == "HIDDEN")
                return "asp:HiddenField";

            return "asp:TextBox";
        }
        private HtmlAttribute NewHtmlAttribute(string name, string value)
        {
            HtmlAttribute htmlAttribute = new HtmlAttribute();
            htmlAttribute.Name = name;
            if (value.Length > 1)
            {
                // see if we need to add quotes to the value
                if (value[0] == '"' && value[value.Length] == '"')
                {
                    // do nothing
                }
                else if (value[0] == '\'' && value[value.Length] == '\'')
                {
                    // do nothing
                }
                else
                    value = @"""" + value + @"""";
            }
            htmlAttribute.Value = value;
            return htmlAttribute;
        }
    }
    public class AttributeTranslation
    {
        private string htmlAttribute;

        public string HtmlAttribute
        {
            get { return htmlAttribute; }
            set { htmlAttribute = value; }
        }
        private string aspnetAttribute;

        public string AspnetAttribute
        {
            get { return aspnetAttribute; }
            set { aspnetAttribute = value; }
        }

        private string[] htmlElementExceptions;

        public string[] HtmlElementExceptions
        {
            get { return htmlElementExceptions; }
            set { htmlElementExceptions = value; }
        }

        private bool includeHidden = true;

        public bool IncludeHidden
        {
            get { return includeHidden; }
            set { includeHidden = value; }
        }

        public AttributeTranslation(string htmlAttribute, string aspnetAttribute)
        {
            this.htmlAttribute = htmlAttribute;
            this.aspnetAttribute = aspnetAttribute;
        }
        public AttributeTranslation(string htmlAttribute, string aspnetAttribute, string[] htmlElementExceptions)
        {
            this.htmlAttribute = htmlAttribute;
            this.aspnetAttribute = aspnetAttribute;
            this.htmlElementExceptions = htmlElementExceptions;
        }
        public AttributeTranslation(
            string htmlAttribute, string aspnetAttribute, string[] htmlElementExceptions, bool includeHidden)
        {
            this.htmlAttribute = htmlAttribute;
            this.aspnetAttribute = aspnetAttribute;
            this.htmlElementExceptions = htmlElementExceptions;
            this.includeHidden = includeHidden;
        }
    }
}