// File           : HotKeyAssignmentBase.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Globalization;

namespace Internationalization
{
    /// <summary>
    /// HotKeyFormat specifies the format used to represent the hotkey to the user
    /// </summary>
    public enum HotKeyFormat {
        /// <summary>
        /// The actual HotKeyFormat is determined by the CurrentUICulture
        /// </summary>
        Culture,
        /// <summary>
        /// The hotkey is embedded within the text (the first letter of each word takes precedence)
        /// </summary>
        Embedded,
        /// <summary>
        /// The hotkey is embedded within the text (the first letter of each word does not take precedence)
        /// </summary>
        SimpleEmbedded,
        /// <summary>
        /// The hotkey is placed in brackets after the text (the hotkey is identified using an ampersand)
        /// </summary>
        BracketedSuffix,
        /// <summary>
        /// The hotkey is placed in brackets before the text (the hotkey is identified using an ampersand)
        /// </summary>
        BracketedPrefix,
        /// <summary>
        /// The hotkey is placed in brackets after the text (the hotkey is not identified using an ampersand)
        /// </summary>
        BracketedSuffixEmbedded,
        /// <summary>
        /// The hotkey is placed in brackets before the text (the hotkey is not identified using an ampersand)
        /// </summary>
        BracketedPrefixEmbedded
    };

    /// <summary>
	/// HotKeyAssignmentBase is a base class for HotKeyAssignment classes
	/// </summary>
	public class HotKeyAssignmentBase
	{
        /// <summary>
        /// NullChar is a null character constant
        /// </summary>
        protected const char NullChar = (Char) 0;
        private static char[] validHotKeys = 
            new Char[] {
                           'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
                           'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
                       };
        private static HotKeyFormat format = HotKeyFormat.Culture;

        private static StandardHotKey[] standardHotKeys = new StandardHotKey[] {};

        /// <summary>
        /// StandardHotKeys is an arrays of StandardHotKey objects which are used for standard hot key assignments
        /// </summary>
        public static StandardHotKey[] StandardHotKeys
        {
            get {return standardHotKeys;}
            set {standardHotKeys = value;}
        }
        /// <summary>
        /// AddStandardHotKey adds a hot key to the list of standard hot keys
        /// </summary>
        /// <param name="hotKey">The hot key character to add</param>
        /// <param name="text">The text for which the hot key is used</param>
        public static void AddStandardHotKey(char hotKey, string text)
        {
            ArrayList tempStandardHotKeys = new ArrayList(standardHotKeys);
            tempStandardHotKeys.Add(new StandardHotKey(hotKey, text));
            standardHotKeys = (StandardHotKey[]) tempStandardHotKeys.ToArray(typeof(StandardHotKey));
        }
        /// <summary>
        /// AddStandardHotKey adds a hot key to the list of standard hot keys
        /// </summary>
        /// <param name="hotKey">The hot key character to add</param>
        public static void AddStandardHotKey(char hotKey)
        {
            ArrayList tempStandardHotKeys = new ArrayList(standardHotKeys);
            tempStandardHotKeys.Add(new StandardHotKey(hotKey));
            standardHotKeys = (StandardHotKey[]) tempStandardHotKeys.ToArray(typeof(StandardHotKey));
        }
        /// <summary>
        /// AddStandardHotKey adds a hot key to the list of standard hot keys
        /// </summary>
        /// <param name="standardHotKey">The StandardHotKey to add to the list of standard hot keys</param>
        public static void AddStandardHotKey(StandardHotKey standardHotKey)
        {
            ArrayList tempStandardHotKeys = new ArrayList(standardHotKeys);
            tempStandardHotKeys.Add(standardHotKey);
            standardHotKeys = (StandardHotKey[]) tempStandardHotKeys.ToArray(typeof(StandardHotKey));
        }
        /// <summary>
        /// AddStandardHotKey adds a semi-colon delimited list of hot keys to the list of standard hot keys
        /// </summary>
        /// <param name="standardHotKeysList">A string containing a semi-colon delimited list of standard hot keys (e.g. "O:OK;X;Y:;C:Cancel")</param>
        public static void AddStandardHotKeys(string standardHotKeysList)
        {
            string[] standardHotKeys = standardHotKeysList.Split(';');
            foreach(string standardHotKey in standardHotKeys)
            {
                char hotKey;
                string text = null;
                int colonIndex = standardHotKey.IndexOf(":");
                if (colonIndex == -1)
                {
                    if (standardHotKey.Length == 1)
                        hotKey = standardHotKey[0];
                    else
                        throw new ArgumentException();
                }
                else
                {
                    if (colonIndex == 1)
                    {
                        hotKey = standardHotKey[0];
                        if (standardHotKey.Length > 2)
                            text = standardHotKey.Substring(2);
                    }
                    else
                        throw new ArgumentException();
                }
                if (text == null)
                    AddStandardHotKey(hotKey);
                else
                    AddStandardHotKey(hotKey, text);
            }        
        }
        /// <summary>
        /// SelectStandardHotKey identifies the character which is used as a hot key for the given text
        /// </summary>
        /// <param name="text">The text to find the hot key for</param>
        /// <returns>The character which is used as a hot key for the given text or NullChar if no character is used</returns>
        protected static char SelectStandardHotKey(string text)
        {
            foreach(StandardHotKey standardHotKey in standardHotKeys)
            {
                if (standardHotKey.Text != null && standardHotKey.Text != String.Empty &&
                    standardHotKey.Text.ToUpper(CultureInfo.InvariantCulture) == text.ToUpper(CultureInfo.InvariantCulture))
                    // this text matches the text for a standard hot key
                    return standardHotKey.HotKey;
            }
            return NullChar;
        }
        /// <summary>
        /// StandardHotKeyIndexOf returns the ordinal position of the StandardHotKey which uses the given hot key character
        /// </summary>
        /// <param name="hotKey">The hot key character to search for</param>
        /// <returns>The ordinal position of the StandardHotKey which uses the given hot key character or -1 if not found</returns>
        protected static int StandardHotKeyIndexOf(char hotKey)
        {
            int index = 0;
            foreach(StandardHotKey standardHotKey in standardHotKeys)
            {
                if (standardHotKey.HotKey == hotKey)
                    return index;
                index++;
            }
            return -1;
        }
        /// <summary>
        /// ValidHotKeys is an array of characters which are acceptable hot keys
        /// </summary>
        public static char[] ValidHotKeys
        {
            get {return validHotKeys;}
            set {validHotKeys = value;}
        }
        /// <summary>
        /// Format is the HotKeyFormat which should be used when assigning hot keys
        /// </summary>
        public static HotKeyFormat Format
        {
            get {return format;}
            set {format = value;}
        }
        /// <summary>
        /// IsValidHotKey returns true if the given character is a valid hot key (as determined by the list of hot keys)
        /// </summary>
        /// <param name="hotKey"></param>
        /// <returns></returns>
        protected static bool IsValidHotKey(char hotKey)
        {
            hotKey = Char.ToUpper(hotKey, CultureInfo.InvariantCulture);
            return Array.IndexOf(validHotKeys, hotKey) > -1;
        }
        /// <summary>
        /// SelectHotKey returns a suitable hot key character for a given string taking into account other hot key characters which have already been assigned
        /// </summary>
        /// <param name="text">The string to get the hot key character for</param>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <returns>A suitable hot key character for the given string</returns>
        protected static char SelectHotKey(string text, ArrayList assignedCharacters)
        {
            if (text.Trim().Length == 0)
                return NullChar;

            // start by trying the first letter of each word
            string[] words = text.Split(' ', ',', '.', '/', ':', ';');
            foreach(string word in words)
            {
                char upperChr = Char.ToUpper(word[0], CultureInfo.InvariantCulture);
                if (IsValidHotKey(upperChr) && assignedCharacters.IndexOf(upperChr) == -1 && StandardHotKeyIndexOf(upperChr) == -1)
                    return upperChr;
            }

            // the first letter of each word hasn't worked so work through the whole
            // string looking for a letter
            foreach(char chr in text)
            {
                char upperChr = Char.ToUpper(chr, CultureInfo.InvariantCulture);
                if (IsValidHotKey(upperChr) && assignedCharacters.IndexOf(upperChr) == -1 && StandardHotKeyIndexOf(upperChr) == -1)
                    return upperChr;
            }
            return GetNextUnusedHotKey(assignedCharacters);
        }
        /// <summary>
        /// GetNextUnusedHotKey gets the next valid hot key which has not been used
        /// </summary>
        /// <param name="assignedCharacters">An ArrayList of characters which have already been assigned</param>
        /// <returns>The next valid hot key which has not been used</returns>
        protected static char GetNextUnusedHotKey(ArrayList assignedCharacters)
        {
            foreach(char validHotKey in ValidHotKeys)
            {
                if (assignedCharacters.IndexOf(validHotKey) == -1 && StandardHotKeyIndexOf(validHotKey) == -1)
                    return validHotKey;
            }
            return NullChar;
        }
        /// <summary>
        /// ApplyHotKey returns a string which contains the given hot key which has been embedded according to the given HotKeyFormat
        /// (existing hot keys are removed from the string first)
        /// </summary>
        /// <param name="text">The string to embed the hot key in</param>
        /// <param name="hotKey">The hot key character to embed in the string</param>
        /// <param name="format">The HotKeyFormat which determines how the hot key should be embedded in the string</param>
        /// <returns>A string which contains the given hot key which has been embedded according to the given HotKeyFormat</returns>
        /// <remarks>If the given string already contains a hot key assignment it is removed</remarks>
        protected static string ApplyHotKey(string text, char hotKey, HotKeyFormat format)
        {
            // take out the existing hotkey (if any)
            text = RemoveHotKey(text, format);
            return AddHotKey(text, hotKey, format);
        }
        /// <summary>
        /// RemoveHotKey removes an existing hot key embedded in a string according to the HotKeyFormat
        /// </summary>
        /// <param name="text">The string to remove the hot key assignment from</param>
        /// <param name="format">The HotKeyFormat which identifies the notation used to embed the hot key</param>
        /// <returns>A string without a hot key assignment</returns>
        protected static string RemoveHotKey(string text, HotKeyFormat format)
        {
            switch(format)
            {
                case HotKeyFormat.BracketedPrefix:
                {
                    if (text.Length > 3 && text[0] == '(' && text[2] == ')')
                        // the string starts with "(?)" so assume that it is a
                        //bracketed prefix and remove it
                        text = text.Substring(3).TrimStart();

                    break;
                }
                case HotKeyFormat.BracketedPrefixEmbedded:
                {
                    if (text.Length > 4 && text[0] == '(' && text[1] == '&' && text[3] == ')')
                        // the string starts with "(&?)" so assume that it is a
                        //bracketed prefix and remove it
                        text = text.Substring(4).TrimStart();

                    break;
                }
                case HotKeyFormat.BracketedSuffix:
                {
                    if (text.Length > 3 && text[text.Length - 2] == '(' && text[text.Length] == ')')
                        // the string ends with "(?)" so assume that it is a
                        //bracketed suffix and remove it
                        text = text.Substring(0, text.Length - 3).TrimEnd();

                    break;
                }
                case HotKeyFormat.BracketedSuffixEmbedded:
                {
                    if (text.Length > 4 && text[text.Length - 3] == '('  && text[text.Length - 2] == '&' && text[text.Length] == ')')
                        // the string ends with "(&?)" so assume that it is a
                        //bracketed suffix and remove it
                        text = text.Substring(0, text.Length - 4).TrimEnd();

                    break;
                }
                case HotKeyFormat.SimpleEmbedded:
                case HotKeyFormat.Embedded:
                {
                    int startIndex = 0;
                    int index;
                    while ((index = text.IndexOf("&", startIndex)) > -1)
                    {
                        if (index < text.Length && text[index + 1] != '&' && (index == 0 || text[index - 1] != '&')
                            && text[index + 1] != ' ')
                            // remove the ampersand
                            return text.Substring(0, index) + text.Substring(index + 1);

                        startIndex = index + 1;
                    }
                    break;
                }
            }
            return text;
        }
        /// <summary>
        /// AddHotKey returns a string which contains the given hot key which has been embedded according to the given HotKeyFormat
        /// </summary>
        /// <param name="text">The string to embed the hot key in</param>
        /// <param name="hotKey">The hot key character to embed in the string</param>
        /// <param name="format">The HotKeyFormat which determines how the hot key should be embedded in the string</param>
        /// <returns>A string which contains the given hot key which has been embedded according to the given HotKeyFormat</returns>
        protected static string AddHotKey(string text, char hotKey, HotKeyFormat format)
        {
            switch (format)
            {
                case HotKeyFormat.SimpleEmbedded:
                case HotKeyFormat.Embedded:
                {
                    bool applied = false;
                    if (format == HotKeyFormat.Embedded)
                    {
                        // start by trying the first letter of each word
                        string[] words = text.Split(' ', ',', '.', '/', ':', ';');
                        foreach(string word in words)
                        {
                            char upperChr = Char.ToUpper(word[0], CultureInfo.InvariantCulture);
                            if (upperChr == hotKey)
                            {
                                int index = text.IndexOf(word);
                                text = text.Substring(0, index) + "&" + text.Substring(index);
                                applied = true;
                                break;
                            }
                        }
                    }

                    if (! applied)
                    {
                        int index = text.ToUpper(CultureInfo.InvariantCulture).IndexOf(hotKey);
                        if (index == -1)
                            // the hotkey isn't part of the string so default to bracketed suffix
                            text = text + " (&" + hotKey + ")";
                        else
                            text = text.Substring(0, index) + "&" + text.Substring(index);
                    }
                    break;
                }
                case HotKeyFormat.BracketedSuffix:
                {
                    text = text + " (" + hotKey + ")";
                    break;
                }
                case HotKeyFormat.BracketedSuffixEmbedded:
                {
                    text = text + " (&" + hotKey + ")";
                    break;
                }
                case HotKeyFormat.BracketedPrefix:
                {
                    text = "(" + hotKey + ") " + text;
                    break;
                }
                case HotKeyFormat.BracketedPrefixEmbedded:
                {
                    text = "(&" + hotKey + ") " + text;
                    break;
                }
            }
            return text;
        }
    }
    /// <summary>
    /// StandardHotKey is a definition of a standard hot key assignment with or without its associated text
    /// </summary>
    public class StandardHotKey
    {
        private char hotKey;
        private string text;
        /// <summary>
        /// Constructs a new StandardHotKey for the given hot key character and associated text
        /// </summary>
        /// <param name="hotKey">The hot key character</param>
        /// <param name="text">The text associated with the hot key character</param>
        public StandardHotKey(char hotKey, string text)
        {
            this.hotKey = hotKey;
            this.text = text;
        }
        /// <summary>
        /// Constructs a new StandardHotKey for the given hot key character
        /// </summary>
        /// <param name="hotKey">The hot key character</param>
        /// <remarks>The new StandardHotKey will have no associated text</remarks>
        public StandardHotKey(char hotKey)
        {
            this.hotKey = hotKey;
        }
        /// <summary>
        /// The hot key character
        /// </summary>
        public char HotKey
        {
            get {return hotKey;}
            set {hotKey = value;}
        }
        /// <summary>
        /// The text associated with the hot key character
        /// </summary>
        /// <remarks>If Text is null then the hot key character is simply 'reserved' and will not be used</remarks>
        public string Text
        {
            get {return text;}
            set {text = value;}
        }
    }
}
