﻿// File           : PseudoTranslationCustomCulture.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;

namespace Globalization
{
    /// <summary>
    /// PseudoTranslationCustomCulture is a custom culture helper object for the pseudo translation culture
    /// </summary>
	public class PseudoTranslationCustomCulture
	{
        /// <summary>
        /// Registers the pseudo translation culture
        /// </summary>
        public static void RegisterPseudoTranslationCulture()
		{
			CreatePseudoTranslationCultureAndRegionInfoBuilder().Register();
		}
        /// <summary>
        /// Creates a CultureAndRegionInfoBuilder object for the pseudo translation culture
        /// </summary>
        /// <returns>A CultureAndRegionInfoBuilder object for the pseudo translation culture</returns>
        public static CultureAndRegionInfoBuilder CreatePseudoTranslationCultureAndRegionInfoBuilder()
		{
			CultureAndRegionInfoBuilder builder = new CultureAndRegionInfoBuilder("pd-PD",
				CultureAndRegionModifiers.None);

			builder.Parent = CultureInfo.InvariantCulture;

			builder.CultureEnglishName = "PseudoLanguage (PseudoRegion)";
			builder.CultureNativeName = "[!!! ΡšĕǔđŏĽäŉğǔäğĕ (ΡšĕǔđŏŘĕğĭŏŉ) !!!]";
			builder.ThreeLetterISOLanguageName = "psd";
			builder.ThreeLetterWindowsLanguageName = "psd";
			builder.TwoLetterISOLanguageName = "pd";

			builder.RegionEnglishName = "PseudoRegion";
			builder.RegionNativeName = "[!!! ΡšĕǔđŏŘĕğĭŏŉ !!!]";
			builder.ThreeLetterISORegionName = "PSD";
			builder.ThreeLetterWindowsRegionName = "PSD";
			builder.TwoLetterISORegionName = "PD";

			builder.IetfLanguageTag = "pd-PD";

			builder.IsMetric = true;
			// US Keyboard Layout (probably not a wise idea to change the keyboard layout)
			builder.KeyboardLayoutId = 1033;
			// GeoId 0x200 is not currently allocated to any location
			builder.GeoId = 0x200;

			builder.GregorianDateTimeFormat = CreatePseudoDateTimeFormatInfo();

			builder.NumberFormat = CreatePseudoNumberFormatInfo();
			builder.CurrencyEnglishName = "Pseudo Dollars";
			builder.CurrencyNativeName = "[!!! Ρšĕǔđŏ Đŏľľäřš !!!]";
			builder.ISOCurrencySymbol = "PSD";

			builder.TextInfo = CultureInfo.InvariantCulture.TextInfo;
			builder.CompareInfo = CultureInfo.InvariantCulture.CompareInfo;

			return builder;
		}

		private static NumberFormatInfo CreatePseudoNumberFormatInfo()
		{
			NumberFormatInfo numberFormatInfo = new NumberFormatInfo();
			numberFormatInfo.CurrencyDecimalDigits = 4;
			numberFormatInfo.CurrencyDecimalSeparator = "@";
			numberFormatInfo.CurrencyGroupSeparator = "'";
			numberFormatInfo.CurrencyGroupSizes = new int[] { 1 };
			numberFormatInfo.CurrencyNegativePattern = 3;
			numberFormatInfo.CurrencyPositivePattern = 3;
			numberFormatInfo.CurrencySymbol = "~";
			numberFormatInfo.DigitSubstitution = DigitShapes.None;
			numberFormatInfo.NaNSymbol = "*NaN*";
			numberFormatInfo.NativeDigits = new string[] { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" };
			numberFormatInfo.NegativeInfinitySymbol = "*-Infinity*";
			numberFormatInfo.NegativeSign = "-";
			numberFormatInfo.NumberDecimalDigits = 2;
			numberFormatInfo.NumberDecimalSeparator = "@";
			numberFormatInfo.NumberGroupSeparator = "'";
			numberFormatInfo.NumberGroupSizes = new int[] { 1 };
			numberFormatInfo.NumberNegativePattern = 3;
			numberFormatInfo.PercentDecimalDigits = 2;
			numberFormatInfo.PercentDecimalSeparator = "@";
			numberFormatInfo.PercentGroupSeparator = "'";
			numberFormatInfo.PercentGroupSizes = new int[] { 1 };
			numberFormatInfo.PercentNegativePattern = 2;
			numberFormatInfo.PercentPositivePattern = 2;
			numberFormatInfo.PercentSymbol = "%";
			numberFormatInfo.PerMilleSymbol = "‰";
			numberFormatInfo.PositiveInfinitySymbol = "*Infinity*";
			numberFormatInfo.PositiveSign = "+";
			return numberFormatInfo;
		}
		private static DateTimeFormatInfo CreatePseudoDateTimeFormatInfo()
		{
			Calendar calendar = new GregorianCalendar(GregorianCalendarTypes.Localized);
			DateTimeFormatInfo dateTimeFormatInfo = new DateTimeFormatInfo();
			dateTimeFormatInfo.Calendar = calendar;

			dateTimeFormatInfo.AbbreviatedDayNames = new string[] { "*Sun*", "*Mon*", "*Tue*", "*Wed*", "*Thu*", "*Fri*", "*Sat*" };
			dateTimeFormatInfo.DayNames = new string[] { "*Sunday*", "*Monday*", "*Tuesday*", "*Wednesday*", "*Thursday*", "*Friday*", "*Saturday*" };
			dateTimeFormatInfo.ShortestDayNames = new string[] { "*Su*", "*Mo*", "*Tu*", "*We*", "*Th*", "*Fr*", "*Sa*" };

			dateTimeFormatInfo.AbbreviatedMonthNames = new string[] { 
			    "*Jan*", "*Feb*", "*Mar*", "*Apr*", "*May*", "*Jun*", "*Jul*", "*Aug*", "*Sep*", "*Oct*", "*Nov*", "*Dec*", "" };
			dateTimeFormatInfo.MonthNames = new string[] { 
			    "*January*", "*February*", "*March*", "*April*", "*May*", "*June*", "*July*", "*August*", "*September*", "*October*", "*November*", "*December*", "" };

			dateTimeFormatInfo.AbbreviatedMonthGenitiveNames = new string[] { 
			    "*Janth*", "*Febth*", "*Marth*", "*Aprth*", "*Mayth*", "*Junth*", "*Julth*", "*Augth*", "*Septh*", "*Octth*", "*Novth*", "*Decth*", "" };
			dateTimeFormatInfo.MonthGenitiveNames = new string[] { 
			    "*Januaryth*", "*Februaryth*", "*Marchth*", "*Aprilth*", "*Mayth*", "*Juneth*", "*Julyth*", "*Augustth*", "*Septemberth*", "*Octoberth*", "*Novemberth*", "*Decemberth*", "" };

			dateTimeFormatInfo.AMDesignator = "*AM*";
			dateTimeFormatInfo.CalendarWeekRule = CalendarWeekRule.FirstFullWeek;
			dateTimeFormatInfo.DateSeparator = "!";
			dateTimeFormatInfo.FirstDayOfWeek = DayOfWeek.Tuesday;
			dateTimeFormatInfo.FullDateTimePattern = "yyyy MMMM dd HH:mm:ss";
			dateTimeFormatInfo.LongDatePattern = "yyyy MMMM dd";
			dateTimeFormatInfo.LongTimePattern = "HH:mm:ss";
			dateTimeFormatInfo.MonthDayPattern = "MMMM dd";
			dateTimeFormatInfo.PMDesignator = "*PM*";
			dateTimeFormatInfo.ShortDatePattern = "yyyy-MM-dd";
			dateTimeFormatInfo.ShortTimePattern = "HH:mm";
			dateTimeFormatInfo.TimeSeparator = ":";
			dateTimeFormatInfo.YearMonthPattern = "yyyy, MMMM";

			return dateTimeFormatInfo;
		}
	}
}
