// File           : DbResourceManager.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Collections;
using System.Globalization;
using System.Data;
using System.Data.SqlClient;
using System.Reflection;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace Internationalization.Resources
{
	/// <summary>
	/// DbResourceManager is a resource manager for reading and writing resources in a database
	/// </summary>
    /// <remarks>Set the static ConnectionString property to the connection string used to connect to the database.
    /// The DbResourceManager works with SQL Server only in this implementation.</remarks>
	public class DbResourceManager: ComponentResourceManager
	{
		private string baseNameField;
		private static string connectionString = 
			"server=localhost;database=CustomResourceManagersExample;trusted_connection=true";

        /// <summary>
        /// ConnectionString is the connection string used to connect to the database
        /// </summary>
		public static string ConnectionString
		{
			get {return connectionString;}
			set {connectionString = value;}
		}
        /// <summary>
        /// Initialize initializes the new object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
		protected virtual void Initialize(string baseName, Assembly assembly)
		{
			this.baseNameField = baseName;
			ResourceSets = new Hashtable();
		}
        /// <summary>
        /// Constructs a DbResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">assembly is not used in this implementation</param>
        public DbResourceManager(string baseName, Assembly assembly)
		{
			Initialize(baseName, assembly);
		}
        /// <summary>
        /// Constructs a DbResourceManager object
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        public DbResourceManager(string baseName)
		{
			Initialize(baseName, null);
		}
        /// <summary>
        /// Constructs a DbResourceManager object
        /// </summary>
        /// <param name="resourceType">The Type for which resources should be read/written</param>
		public DbResourceManager(Type resourceType)
		{
			Initialize(resourceType.Name, resourceType.Assembly);
		}
        /// <summary>
        /// Provides the implementation for finding a ResourceSet
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to look for</param>
        /// <param name="createIfNotExists">If true and if the ResourceSet has not been loaded yet, load it</param>
        /// <param name="tryParents">If the ResourceSet cannot be loaded, try parent CultureInfo objects to see if they exist</param>
        /// <returns>The specified ResourceSet</returns>
		protected override ResourceSet InternalGetResourceSet(CultureInfo cultureInfo, bool createIfNotExists, bool tryParents)
		{
			if (ResourceSets.Contains(cultureInfo.Name))
				return ResourceSets[cultureInfo.Name] as ResourceSet;
			else
			{
				ResourceSet resourceSet = new DbResourceSet(baseNameField, cultureInfo);
				ResourceSets.Add(cultureInfo.Name, resourceSet);
				return resourceSet; 
			}
		}
	}

    /// <summary>
    /// DbResourceSet is a CustomResourceSet for reading and writing to/from databases
    /// </summary>
	public class DbResourceSet: CustomResourceSet
	{
		private string baseNameField;
		private CultureInfo cultureInfo;
        /// <summary>
        /// Constructs a DbResourceSet object
        /// </summary>
        /// <param name="baseNameField">The base name of the resource set</param>
        /// <param name="cultureInfo">The CultureInfo identifying the culture of the resources in the resource set</param>
		public DbResourceSet(string baseNameField, CultureInfo cultureInfo):
			base(new DbResourceReader(baseNameField, cultureInfo))
		{
			this.baseNameField = baseNameField;
			this.cultureInfo = cultureInfo;
		}
        /// <summary>
        /// GetDefaultReader gets the Type of the resource reader
        /// </summary>
        /// <returns>The Type of the resource reader</returns>
        /// <remarks>Use CreateDefaultReader instead because CreateDefaultReader passes the correct parameters
        /// to the resource reader constructor</remarks>
		public override Type GetDefaultReader()
		{
			return typeof(DbResourceReader);
		}
        /// <summary>
        /// GetDefaultWriter gets the Type of the resource writer
        /// </summary>
        /// <returns>The Type of the resource writer</returns>
        /// <remarks>Use CreateDefaultWriter instead because CreateDefaultWriter passes the correct parameters
        /// to the resource writer constructor</remarks>
        public override Type GetDefaultWriter()
		{
			return typeof(DbResourceWriter);
		}
        /// <summary>
        /// CreateDefaultReader creates an IResourceReader for reading resources
        /// </summary>
        /// <returns>An IResourceReader for reading resources</returns>
        /// <remarks>Use CreateDefaultReader instead of GetDefaultReader because CreateDefaultReader passes the
        /// correct parameters to the resource reader constructor</remarks>
        public override IResourceReader CreateDefaultReader()
		{
			return new DbResourceReader(baseNameField, cultureInfo);
		}
        /// <summary>
        /// CreateDefaultWriter creates an IResourceReader for writing resources
        /// </summary>
        /// <returns>An IResourceWriter for writing resources</returns>
        /// <remarks>Use CreateDefaultWriter instead of GetDefaultWriter because CreateDefaultWriter passes the
        /// correct parameters to the resource writer constructor</remarks>
        public override IResourceWriter CreateDefaultWriter()
		{
			return new DbResourceWriter(baseNameField, cultureInfo);
		}
	}

    /// <summary>
    /// DbResourceReader is an IResourceReader for reading resources from a database
    /// </summary>
	public class DbResourceReader: IResourceReader
	{
		private string baseNameField;
		private CultureInfo cultureInfo;

        /// <summary>
        /// Constructs a DbResourceReader object
        /// </summary>
        /// <param name="baseNameField">The base name of the resource reader</param>
        /// <param name="cultureInfo">The CultureInfo identifying the culture of the resources to be read</param>
        public DbResourceReader(string baseNameField, CultureInfo cultureInfo)
		{
			this.baseNameField = baseNameField;
			this.cultureInfo = cultureInfo;
		}
        /// <summary>
        /// Returns an IDictionaryEnumerator of the resources for this reader
        /// </summary>
        /// <returns>An IDictionaryEnumerator of the resources for this reader</returns>
		public System.Collections.IDictionaryEnumerator GetEnumerator()
		{
			Hashtable hashTable = new Hashtable();
			using(SqlConnection connection = new SqlConnection(DbResourceManager.ConnectionString))
			{
				connection.Open();
                using (SqlCommand command = GetSelectCommand(connection))
                {
                    SqlDataReader dataReader = command.ExecuteReader();
                    while (dataReader.Read())
                    {
                        object resourceValue = dataReader["ResourceValue"].ToString();
                        object resourceType = dataReader["ResourceType"];
                        if (resourceType != null && resourceType.ToString() != String.Empty && resourceType.ToString() != "System.String")
                            resourceValue = GetResourceValue((string)resourceValue, resourceType.ToString());

                        hashTable.Add(dataReader["ResourceName"].ToString(), resourceValue);
                    }
                    dataReader.Close();
                }
			}
			return hashTable.GetEnumerator();
		}
        /// <summary>
        /// GetResourceValue converts a resource value to a given type
        /// </summary>
        /// <param name="resourceValue">The string of the value to potentially convert</param>
        /// <param name="resourceTypeName">The fully qualified Type name</param>
        /// <returns>An object of the correct type for the value</returns>
		protected virtual object GetResourceValue(string resourceValue, string resourceTypeName)
		{
			// resourceTypeName is something like
			// System.Drawing.ContentAlignment, System.Drawing, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a
			// so className would be "System.Drawing.ContentAlignment"
			// and assemblyName would be "System.Drawing, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a".
			string className = resourceTypeName.Split(',')[0];
			string assemblyName = resourceTypeName.Substring(className.Length + 2);
			Assembly assembly = Assembly.Load(assemblyName);
			Type resourceType = assembly.GetType(className, true, true);

            TypeConverter typeConverter = TypeDescriptor.GetConverter(resourceType);
            if (typeConverter == null || !typeConverter.CanConvertFrom(typeof(string)))
                throw new ArgumentException(String.Format(DbResourceManagerResources.TypeConverterNotFound, new object[] { resourceValue }));

            return typeConverter.ConvertFromInvariantString(resourceValue);
        }
        /// <summary>
        /// GetSelectcommand gets an SqlCommand for SELECTing the resources for the base name and culture
        /// </summary>
        /// <param name="connection">The SqlConnection to use for the command</param>
        /// <returns>An SqlCommand for SELECTing the resources for the base name and culture</returns>
		protected virtual SqlCommand GetSelectCommand(SqlConnection connection)
		{
			SqlCommand command;
			if (cultureInfo.Equals(CultureInfo.InvariantCulture))
			{
				string commandText = 
					"SELECT ResourceName, ResourceValue, ResourceType FROM ResourceSets WHERE ResourceSetName=@resourceSetName" +
					" AND Culture IS NULL";

				command = new SqlCommand(commandText, connection);
				command.Parameters.Add("@resourceSetName", SqlDbType.NVarChar, 100).Value = baseNameField;
			}
			else
			{
				string commandText = 
					"SELECT ResourceName, ResourceValue, ResourceType FROM ResourceSets WHERE ResourceSetName=@resourceSetName" +
					" AND Culture=@culture";

				command = new SqlCommand(commandText, connection);
				command.Parameters.Add("@resourceSetName", SqlDbType.NVarChar, 100).Value = baseNameField;
				command.Parameters.Add("@culture", SqlDbType.NVarChar, 20).Value = cultureInfo.ToString();
			}
			return command;
		}
        /// <summary>
        /// Closes the resource reader after releasing any resources associated with it
        /// </summary>
		public void Close()
		{
		}
        /// <summary>
        /// Returns an IDictionaryEnumerator of the resources for this reader
        /// </summary>
        /// <returns>An IDictionaryEnumerator of the resources for this reader</returns>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
		{
			return this.GetEnumerator();
		}
        /// <summary>
        /// Releases all resources used by the object
        /// </summary>
		public void Dispose()
		{
		}
	}

    /// <summary>
    /// DbResourceWriter is an IResourceWriter for writing resources to a database
    /// </summary>
    public class DbResourceWriter : IResourceWriter
	{
		private string baseNameField;
		private CultureInfo cultureInfo;
		private SortedList resourceList;

        /// <summary>
        /// Constructs a DbResourceWriter object
        /// </summary>
        /// <param name="baseNameField">The base name of the resource writer</param>
        /// <param name="cultureInfo">The CultureInfo identifying the culture of the resources to be written</param>
        public DbResourceWriter(string baseNameField, CultureInfo cultureInfo)
		{
			this.baseNameField = baseNameField;
			this.cultureInfo = cultureInfo;
			resourceList = new SortedList();
		}
        /// <summary>
        /// Closes the resource writer after releasing any resources associated with it
        /// </summary>
        public void Close()
		{
			Dispose(true);
		}
        /// <summary>
        /// Releases all resources used by the object
        /// </summary>
        public void Dispose()
		{
			Dispose(true);
		}
        /// <summary>
        /// Releases all resources used by the object
        /// </summary>
        /// <param name="disposing">True if the object is being disposed</param>
		private void Dispose(bool disposing)
		{
			if (disposing && resourceList != null)
				Generate();
		}
        /// <summary>
        /// Adds a resource to the list of resources to be written to an output file or output stream
        /// </summary>
        /// <param name="name">The name of the resource</param>
        /// <param name="value">The value of the resource</param>
		public void AddResource(string name, object value)
		{
			if (name == null)
				throw new ArgumentNullException("name");
			if (resourceList == null)
				throw new InvalidOperationException(DbResourceManagerResources.InvalidOperationNoResources);

			resourceList.Add(name, value);
		}
        /// <summary>
        /// Adds a resource to the list of resources to be written to an output file or output stream
        /// </summary>
        /// <param name="name">The name of the resource</param>
        /// <param name="value">The value of the resource</param>
        public void AddResource(string name, string value)
		{
			AddResource(name, (Object) value);
		}
        /// <summary>
        /// Adds a resource to the list of resources to be written to an output file or output stream
        /// </summary>
        /// <param name="name">The name of the resource</param>
        /// <param name="value">The value of the resource</param>
        public void AddResource(string name, byte[] value)
		{
			AddResource(name, (Object) value);
		}
        /// <summary>
        /// Writes all the resources added by the AddResource method to the output file or stream
        /// </summary>
		public void Generate()
		{
			if (resourceList == null)
                throw new InvalidOperationException(DbResourceManagerResources.InvalidOperationNoResources);

			using (SqlConnection connection = new SqlConnection(DbResourceManager.ConnectionString))
			{
				connection.Open();
				SqlTransaction transaction = connection.BeginTransaction();
				try
				{
					DeleteExistingResource(transaction);
					foreach(DictionaryEntry dictionaryEntry in resourceList)
					{
						if (dictionaryEntry.Value != null)
							InsertResource(transaction, dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
					}
					transaction.Commit();
				}
				catch
				{
					transaction.Rollback();
					throw;
				}
			}
			resourceList = null;
		}
        /// <summary>
        /// DeleteExistingResource deletes the existing resource to make way for the new resource
        /// </summary>
        /// <param name="transaction">The SqlTransaction of which this process is a part</param>
		protected virtual void DeleteExistingResource(SqlTransaction transaction)
		{
			// delete all of the existing resource values
			if (cultureInfo.Equals(CultureInfo.InvariantCulture))
			{
				string deleteCommandText = 
					"DELETE FROM ResourceSets WHERE ResourceSetName=@resourceSetName AND Culture IS NULL";

                using (SqlCommand deleteCommand = new SqlCommand(deleteCommandText, transaction.Connection, transaction))
                {
                    deleteCommand.Parameters.Add("@resourceSetName", SqlDbType.NVarChar, 100).Value = baseNameField;
                    deleteCommand.ExecuteNonQuery();
                }
			}
			else
			{
				string deleteCommandText = 
					"DELETE FROM ResourceSets WHERE ResourceSetName=@resourceSetName AND Culture=@culture";

                using (SqlCommand deleteCommand = new SqlCommand(deleteCommandText, transaction.Connection, transaction))
                {
                    deleteCommand.Parameters.Add("@resourceSetName", SqlDbType.NVarChar, 100).Value = baseNameField;
                    deleteCommand.Parameters.Add("@culture", SqlDbType.NVarChar, 20).Value = cultureInfo.ToString();
                    deleteCommand.ExecuteNonQuery();
                }
			}
		}
        /// <summary>
        /// InsertResource inserts a given resource entry for the base name and culture
        /// </summary>
        /// <param name="transaction">The SqlTransaction of which this process is a part</param>
        /// <param name="resourceName">The name of the resource</param>
        /// <param name="resourceValue">The value of the resource</param>
		protected virtual void InsertResource(SqlTransaction transaction, string resourceName, object resourceValue)
		{
            if (resourceValue is ResXDataNode)
            {
                ResXDataNode resXDataNode = (ResXDataNode)resourceValue;
                resourceValue = resXDataNode.GetValue((ITypeResolutionService)null);
            }

            string insertCommandText;
			if (cultureInfo.Equals(CultureInfo.InvariantCulture))
			{
				if (resourceValue is System.String)
					insertCommandText = 
						"INSERT INTO ResourceSets (ResourceSetName, ResourceName, ResourceValue) " +
						"VALUES (@resourceSetName, @resourceName, @resourceValue)";
				else
					insertCommandText = 
						"INSERT INTO ResourceSets (ResourceSetName, ResourceName, ResourceValue, ResourceType) " +
						"VALUES (@resourceSetName, @resourceName, @resourceValue, @resourceType)";
			}
			else
			{
				if (resourceValue is System.String)
					insertCommandText = 
						"INSERT INTO ResourceSets (ResourceSetName, Culture, ResourceName, ResourceValue) " +
						"VALUES (@resourceSetName, @culture, @resourceName, @resourceValue)";
				else
					insertCommandText = 
						"INSERT INTO ResourceSets (ResourceSetName, Culture, ResourceName, ResourceValue, ResourceType) " +
						"VALUES (@resourceSetName, @culture, @resourceName, @resourceValue, @resourceType)";
			}

            using (SqlCommand insertCommand = new SqlCommand(insertCommandText, transaction.Connection, transaction))
            {
                insertCommand.Parameters.Add(new SqlParameter("@resourceSetName", baseNameField));
                if (!cultureInfo.Equals(CultureInfo.InvariantCulture))
                    insertCommand.Parameters.Add(new SqlParameter("@culture", cultureInfo.ToString()));
                insertCommand.Parameters.Add(new SqlParameter("@resourceName", resourceName));
                insertCommand.Parameters.Add(new SqlParameter("@resourceValue", resourceValue.ToString()));
                if (!(resourceValue is System.String))
                    insertCommand.Parameters.Add(new SqlParameter("@resourceType", resourceValue.GetType().AssemblyQualifiedName));

                insertCommand.ExecuteNonQuery();
            }
		}
	}
}
