// File           : ResourceManagerSetter.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.CodeDom;

namespace Internationalization.Resources
{
    /// <summary>
    /// ResourceManagerSetter is a component that initializes the resources local variable in
    /// Form.InitializeComponent to a ResourceManagerProvider
    /// </summary>
    /// <remarks>ResourceManagerProvider is a provider for ResourceManagers. The purpose of the
    /// ResourceManagerSetter is to free Windows Forms from a dependence on the ResourceManager class
    /// that works solely with resources that are embedded in an assembly. The ResourceManagerSetter component
    /// achieves this by injecting the following new line into the Form.InitializeComponent method:-
    /// resources = Internationalization.Resources.ResourceManagerProvider.GetResourceManager(typeof(Form1));
    /// </remarks>
	[DesignerSerializer(typeof(ResourceManagerSetterSerializer), typeof(CodeDomSerializer))]
	public class ResourceManagerSetter : System.ComponentModel.Component
	{
	}
    /// <summary>
    /// ResourceManagerSetterSerializer is a CodeDomSerializer which changes the Form.InitializeComponent method
    /// </summary>
	public class ResourceManagerSetterSerializer : CodeDomSerializer
	{
        /// <summary>
        /// Deserializes the specified serialized CodeDOM object into an object
        /// </summary>
        /// <param name="manager">A serialization manager interface that is used during the deserialization process</param>
        /// <param name="codeDomObject">A serialized CodeDOM object to deserialize</param>
        /// <returns>The deserialized CodeDOM object</returns>
		public override object Deserialize(IDesignerSerializationManager manager, object codeDomObject)  
		{
			CodeDomSerializer baseSerializer = (CodeDomSerializer)
				manager.GetSerializer(typeof(ResourceManagerSetter).BaseType, typeof(CodeDomSerializer));
			return baseSerializer.Deserialize(manager, codeDomObject);
		}
        /// <summary>
        /// Serializes the specified object into a CodeDOM object
        /// </summary>
        /// <param name="manager">The serialization manager to use during serialization</param>
        /// <param name="value">The object to serialize</param>
        /// <returns>A CodeDOM object representing the object that has been serialized</returns>
		public override object Serialize(IDesignerSerializationManager manager, object value) 
		{
			CodeDomSerializer baseSerializer = 
				(CodeDomSerializer) manager.GetSerializer(typeof(ResourceManagerSetter).BaseType, typeof(CodeDomSerializer));

			object codeObject = baseSerializer.Serialize(manager, value);

			if (codeObject is CodeStatementCollection)
			{
				CodeStatementCollection statements = (CodeStatementCollection) codeObject;

				CodeTypeDeclaration classTypeDeclaration =
					(CodeTypeDeclaration) manager.GetService(typeof(CodeTypeDeclaration));

				CodeExpression typeofExpression = new CodeTypeOfExpression(classTypeDeclaration.Name);

                CodeDirectionExpression outResourceExpression = new CodeDirectionExpression(
                    FieldDirection.Out, new CodeVariableReferenceExpression("resources"));

				CodeExpression rightCodeExpression = 
					new CodeMethodInvokeExpression(new CodeTypeReferenceExpression("Internationalization.Resources.ResourceManagerProvider"), "GetResourceManager", 
					new CodeExpression[] {typeofExpression, outResourceExpression});

				statements.Insert(0, new CodeExpressionStatement(rightCodeExpression));
			}
			return codeObject;
		}
	}
}
