// File           : ResourceManagerProvider.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Reflection;
using System.ComponentModel;

namespace Internationalization.Resources
{
	/// <summary>
	/// ResourceManagerProvider provides ResourceManager objects according to the ResourceManager type set
	/// </summary>
	/// <remarks>ResourceManagerProvider provides an abstraction from the actual ResourceManager class used and 
	/// therefore avoids hard coding the names of ResourceManager classes throughout an application.
	/// </remarks>
	public class ResourceManagerProvider
	{
		private static Type resourceManagerType = typeof(ResourceManager);

        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given Type
        /// </summary>
        /// <param name="type">The Type for which a ResourceManager should be returned</param>
        /// <param name="resourceManager">resourceManager is set to a ComponentResourceManager for the given Type</param>
        public static void GetResourceManager(Type type, out ComponentResourceManager resourceManager)
        {
            resourceManager = (ComponentResourceManager)Activator.CreateInstance(resourceManagerType, new object[] { type });
        }
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given base name and assembly
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">The assembly (if any) in which the resources are found</param>
        /// <param name="resourceManager">resourceManager is set to a ComponentResourceManager for the given base name and assembly</param>
        public static void GetResourceManager(string baseName, Assembly assembly, out ComponentResourceManager resourceManager)
        {
            resourceManager = (ComponentResourceManager)
                Activator.CreateInstance(resourceManagerType, new object[] { baseName, assembly });
        }
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given base name and assembly
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">The assembly (if any) in which the resources are found</param>
        /// <param name="usingResourceSet">The Type of the ResourceSet (if different from the default Type)</param>
        /// <param name="resourceManager">resourceManager is set to a ComponentResourceManager for the given base name and assembly</param>
        public static void GetResourceManager(string baseName, Assembly assembly, Type usingResourceSet,
            out ComponentResourceManager resourceManager)
        {
            resourceManager = (ComponentResourceManager)
                Activator.CreateInstance(resourceManagerType, new object[] { baseName, assembly, usingResourceSet });
        }
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given Type
        /// </summary>
        /// <param name="type">The Type for which a ResourceManager should be returned</param>
        /// <param name="resourceManager">resourceManager is set to a ResourceManager for the given Type</param>
        public static void GetResourceManager(Type type, out ResourceManager resourceManager)
        {
            resourceManager = (ResourceManager)Activator.CreateInstance(resourceManagerType, new object[] { type });
        }
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given base name and assembly
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">The assembly (if any) in which the resources are found</param>
        /// <param name="resourceManager">resourceManager is set to a ResourceManager for the given base name and assembly</param>
        public static void GetResourceManager(string baseName, Assembly assembly, out ResourceManager resourceManager)
        {
            resourceManager = (ResourceManager)
                Activator.CreateInstance(resourceManagerType, new object[] { baseName, assembly });
        }
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given base name and assembly
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">The assembly (if any) in which the resources are found</param>
        /// <param name="usingResourceSet">The Type of the ResourceSet (if different from the default Type)</param>
        /// <param name="resourceManager">resourceManager is set to a ResourceManager for the given base name and assembly</param>
        public static void GetResourceManager(string baseName, Assembly assembly, Type usingResourceSet,
            out ResourceManager resourceManager)
        {
            resourceManager = (ResourceManager)
                Activator.CreateInstance(resourceManagerType, new object[] { baseName, assembly, usingResourceSet });
        }
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given Type
        /// </summary>
        /// <param name="type">The Type for which a ResourceManager should be returned</param>
        /// <returns>A ResourceManager for the given Type</returns>
		public static ResourceManager GetResourceManager(Type type)
		{
			return (ResourceManager) Activator.CreateInstance(resourceManagerType, new object[] {type});
		}
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given base name and assembly
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">The assembly (if any) in which the resources are found</param>
        /// <returns>A ResourceManager for the given base name and assembly</returns>
		public static ResourceManager GetResourceManager(string baseName, Assembly assembly)
		{
            try
            {
                return (ResourceManager)
                    Activator.CreateInstance(resourceManagerType, new object[] { baseName, assembly });
            }
            catch (MissingMethodException)
            {
                // The resource manager Type does not have a constructor that accepts baseName and assembly
                // (it is probably ComponentResourceManager) so try again with just a Type parameter.
                return (ResourceManager)
                    Activator.CreateInstance(resourceManagerType, new object[] { assembly.GetType(baseName) });
            }
		}
        /// <summary>
        /// GetResourceManager returns a ResourceManager for the given base name and assembly
        /// </summary>
        /// <param name="baseName">The qualified base name which the resources represent</param>
        /// <param name="assembly">The assembly (if any) in which the resources are found</param>
        /// <param name="usingResourceSet">The Type of the ResourceSet (if different from the default Type)</param>
        /// <returns>A ResourceManager for the given base name and assembly</returns>
		public static ResourceManager GetResourceManager(string baseName, Assembly assembly, Type usingResourceSet)
		{
			return (ResourceManager) 
				Activator.CreateInstance(resourceManagerType, new object[] {baseName, assembly, usingResourceSet});
		}
        /// <summary>
        /// ResourceManagerType is the Type of the ResourceManager which the provider providers
        /// </summary>
        /// <remarks>The Type must be a ResourceManager or a descendant of ResourceManager</remarks>
		public static Type ResourceManagerType
		{
			get {return resourceManagerType;}
			set
			{
				if (value.FullName == "System.Resources.ResourceManager" ||
					value.IsSubclassOf(typeof(ResourceManager)))
					resourceManagerType = value;
				else
					throw new ApplicationException(ResourceManagerProviderResources.InvalidResourceManagerType);
			}
		}
	}
}
