﻿// File           : BuildManagerHelper.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Web;
using System.Web.Compilation;
using System.Globalization;
using System.Web.Hosting;

namespace Internationalization.Web.Compilation
{
    /// <summary>
    /// BuildManagerHelper exposes features of the BuildManager
    /// </summary>
    public class BuildManagerHelper
    {
        /// <summary>
        /// GetGlobalResourceAssembly gets the global resource assembly that is used by the ResourceManager
        /// </summary>
        /// <returns>The resource assembly that contains the resources</returns>
        public static Assembly GetGlobalResourceAssembly()
        {
            return (Assembly)ReflectionHelper.GetInternalStaticProperty(typeof(BuildManager), "AppResourcesAssembly");
        }
        /// <summary>
        /// GetLocalResourceAssembly gets the resource assembly that is used by the ResourceManager
        /// </summary>
        /// <returns>The resource assembly that contains the resources</returns>
        public static Assembly GetLocalResourceAssembly(string virtualPath)
        {
            try
            {
                object virtualPathParentObject = CreateVirtualPathParent(virtualPath);
                string cacheKey = GetLocalResourcesAssemblyName(virtualPathParentObject);
                Object buildResult = GetBuildResultFromCache(cacheKey);
                if (buildResult != null)
                    return GetBuildResultResultAssembly(buildResult);

                return null;
            }
            catch
            {
                return null;
            }
        }
        private static string GetLocalResourcesAssemblyName(object virtualPathObject)
        {
            MethodInfo methodInfo = typeof(BuildManager).GetMethod("GetLocalResourcesAssemblyName",
                System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.NonPublic);
            if (methodInfo == null)
                return String.Empty;

            return (string)methodInfo.Invoke(null, new object[] { virtualPathObject });
        }
        private static object CreateVirtualPathParent(string virtualPath)
        {
            Assembly systemWebAssembly = Assembly.GetAssembly(typeof(BuildManager));
            Type virtualPathType = systemWebAssembly.GetType("System.Web.VirtualPath", false, true);
            MethodInfo methodInfo = virtualPathType.GetMethod("Create",
                System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.Public,
                null, new Type[] { typeof(String) }, null);
            if (methodInfo == null)
                return String.Empty;

            object virtualPathObject = methodInfo.Invoke(null, new object[] { virtualPath });
            if (virtualPathObject == null)
                return null;

            PropertyInfo propertyInfo = virtualPathType.GetProperty("Parent");
            if (propertyInfo == null)
                return null;

            return propertyInfo.GetValue(virtualPathObject, new object[] { });
        }
        /// <summary>
        /// GetBuildResultFromCache gets the build result from the given cache key
        /// </summary>
        /// <param name="cacheKey">The cache key</param>
        /// <returns>The build result from the given cache key</returns>
        private static object GetBuildResultFromCache(string cacheKey)
        {
            MethodInfo methodInfo = typeof(BuildManager).GetMethod("GetBuildResultFromCache",
                System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.NonPublic,
                null, new Type[] { typeof(String) }, null);
            if (methodInfo == null)
                return String.Empty;

            return methodInfo.Invoke(null, new object[] { cacheKey });
        }
        /// <summary>
        /// GetBuildResultResultAssembly gets the assembly for the given build result
        /// </summary>
        /// <param name="buildResult">The build result</param>
        /// <returns>The assembly for the given build result</returns>
        private static Assembly GetBuildResultResultAssembly(object buildResult)
        {
            Type buildResultType = buildResult.GetType();
            PropertyInfo propertyInfo = buildResultType.GetProperty(
                "ResultAssembly", BindingFlags.NonPublic | BindingFlags.Instance);
            if (propertyInfo == null)
                return null;

            return (Assembly)propertyInfo.GetValue(buildResult, new object[] { });
        }
    }
    /// <summary>
    /// ReflectionHelper is a utility class for helping with Reflection
    /// </summary>
    public class ReflectionHelper
    {
        /// <summary>
        /// GetInternalStaticProperty gets the value of the given non-public internal static property
        /// </summary>
        /// <param name="type">The Type to get the property value from</param>
        /// <param name="propertyName">The name of the property to get the value for</param>
        /// <returns>The value of the property</returns>
        public static object GetInternalStaticProperty(Type type, string propertyName)
        {
            PropertyInfo propertyInfo = type.GetProperty(propertyName,
                System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.NonPublic);
            if (propertyInfo == null)
                return null;
            else
                return propertyInfo.GetValue(null, null);
        }
    }
}
