// File           : StronglyTypedResourceBuilderHelper.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;
using System.Resources;
using System.CodeDom;
using System.CodeDom.Compiler;
using Microsoft.CSharp;
using System.Resources.Tools;
using System.Reflection;
using System.Diagnostics;

namespace Resources.Tools
{
    /// <summary>
    /// ResourceManagerGetStatementChange specifies the kind of change that should be made to the generated
    /// Get statements
    /// </summary>
    public enum ResourceManagerGetStatementChange { Standard, Global };

    /// <summary>
    /// StronglyTypedResourceBuilderHelper is a helper class for StronglyTypedResourceBuilder
    /// </summary>
    /// <remarks>Use StronglyTypedResourceBuilderHelper.GenerateClass to generate a strongly typed resource
    /// class which uses the ResourceManagerProvider class instead of the ResourceManager class</remarks>
    public class StronglyTypedResourceBuilderHelper
    {
        /// <summary>
        /// GenerateClass generates a strongly typed C# class from a resource
        /// </summary>
        /// <param name="resxFilename">The filename of a .resx resources file</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="classFilename">The file name in which the generated class is placed</param>
        public static void GenerateClass(string resxFilename, string nameSpace, string className, string classFilename)
        {
            GenerateClass(resxFilename, nameSpace, className, classFilename, false);
        }
        /// <summary>
        /// GenerateClass generates a strongly typed C# class from a resource
        /// </summary>
        /// <param name="resxFilename">The filename of a .resx resources file</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="classFilename">The file name in which the generated class is placed</param>
        /// <param name="internalClass">True if the generated class should be internal</param>
        public static void GenerateClass(string resxFilename, string nameSpace, string className, string classFilename, bool internalClass)
        {
            Hashtable resources = GetResources(resxFilename);

            if (className == null || className == String.Empty)
                className = Path.GetFileNameWithoutExtension(resxFilename);

            if (classFilename == null || classFilename == String.Empty)
                classFilename = Path.ChangeExtension(resxFilename, ".cs");

            GenerateClass(resources, nameSpace, className, classFilename, internalClass);
        }
        /// <summary>
        /// GenerateClass generates a strongly typed C# class from a resource
        /// </summary>
        /// <param name="resourceList">The IDictionary of resources</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="classFilename">The file name in which the generated class is placed</param>
        public static void GenerateClass(IDictionary resourceList, string nameSpace, string className, string classFilename)
        {
            GenerateClass(resourceList, nameSpace, className, classFilename, false);
        }
        /// <summary>
        /// GenerateClass generates a strongly typed C# class from a resource
        /// </summary>
        /// <param name="resourceList">The IDictionary of resources</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="classFilename">The file name in which the generated class is placed</param>
        /// <param name="internalClass">True if the generated class should be internal</param>
        public static void GenerateClass(IDictionary resourceList, string nameSpace, string className, string classFilename, bool internalClass)
        {
            string[] unmatchable;
            CodeDomProvider codeDomProvider = new CSharpCodeProvider();
            CodeCompileUnit codeCompileUnit = GenerateCodeCompileUnit(resourceList, className, nameSpace, codeDomProvider, internalClass, out unmatchable);

            using (TextWriter writer = new StreamWriter(classFilename))
            {
                codeDomProvider.GenerateCodeFromCompileUnit(codeCompileUnit, writer, new CodeGeneratorOptions());
            }
        }
        /// <summary>
        /// GenerateCodeCompileUnit generates a strongly typed CodeCompileUnit from a resource
        /// </summary>
        /// <param name="resourceList">The IDictionary of resources</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="codeDomProvider">The CodeDomProvider to generate the CodeCompileUnit for</param>
        /// <param name="internalClass">True if the generated class should be internal</param>
        /// <param name="unmatchable">The unmatchable strings (an out parameter)</param>
        /// <returns>A CodeCompileUnit representing the strongly typed resource class</returns>
        public static CodeCompileUnit GenerateCodeCompileUnit(
            IDictionary resourceList, string className, string nameSpace, CodeDomProvider codeDomProvider,
            bool internalClass, out string[] unmatchable)
        {
            return GenerateCodeCompileUnit(resourceList, className, nameSpace, codeDomProvider,
            internalClass, out unmatchable, ResourceManagerGetStatementChange.Standard);
        }
        /// <summary>
        /// GenerateCodeCompileUnit generates a strongly typed CodeCompileUnit from a resource
        /// </summary>
        /// <param name="resourceList">The IDictionary of resources</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="codeDomProvider">The CodeDomProvider to generate the CodeCompileUnit for</param>
        /// <param name="internalClass">True if the generated class should be internal</param>
        /// <param name="unmatchable">The unmatchable strings (an out parameter)</param>
        /// <param name="getStatementChange">The change that should be made to the Get statements</param>
        /// <returns>A CodeCompileUnit representing the strongly typed resource class</returns>
        public static CodeCompileUnit GenerateCodeCompileUnit(
            IDictionary resourceList, string className, string nameSpace, CodeDomProvider codeDomProvider,
            bool internalClass, out string[] unmatchable, ResourceManagerGetStatementChange getStatementChange)
        {
            return GenerateCodeCompileUnit(resourceList, className, nameSpace, codeDomProvider, 
                internalClass, out unmatchable, getStatementChange, true);
        }
        /// <summary>
        /// GenerateCodeCompileUnit generates a strongly typed CodeCompileUnit from a resource
        /// </summary>
        /// <param name="resourceList">The IDictionary of resources</param>
        /// <param name="className">The name of the generated class</param>
        /// <param name="nameSpace">The namespace in which the generated class should belong</param>
        /// <param name="codeDomProvider">The CodeDomProvider to generate the CodeCompileUnit for</param>
        /// <param name="internalClass">True if the generated class should be internal</param>
        /// <param name="unmatchable">The unmatchable strings (an out parameter)</param>
        /// <param name="getStatementChange">The change that should be made to the Get statements</param>
        /// <param name="changeResourceManager">True if the resource manager statement should be changed</param>
        /// <returns>A CodeCompileUnit representing the strongly typed resource class</returns>
        public static CodeCompileUnit GenerateCodeCompileUnit(
            IDictionary resourceList, string className, string nameSpace, CodeDomProvider codeDomProvider,
            bool internalClass, out string[] unmatchable, ResourceManagerGetStatementChange getStatementChange,
            bool changeResourceManager)
        {
            Hashtable resources = (Hashtable)resourceList;

            CodeCompileUnit codeCompileUnit =
                StronglyTypedResourceBuilder.Create(resources, className, nameSpace,
                codeDomProvider, internalClass, out unmatchable);

            if (changeResourceManager)
                ChangeResourceManager(className, codeCompileUnit, getStatementChange);

            if (!internalClass)
            {
                MakeConstructorPublic(codeCompileUnit);
            }
            return codeCompileUnit;
        }
        private static Hashtable GetResources(string resxFilename)
        {
            ResXResourceReader reader = new ResXResourceReader(resxFilename);
            Hashtable resources = new Hashtable();
            try
            {
                IDictionaryEnumerator enumerator = reader.GetEnumerator();
                while (enumerator.MoveNext())
                {
                    resources.Add(enumerator.Key.ToString(), enumerator.Value);
                }
            }
            finally
            {
                reader.Close();
            }
            return resources;
        }
        private static void MakeConstructorPublic(CodeCompileUnit codeCompileUnit)
        {
            CodeNamespace codeNamespace = codeCompileUnit.Namespaces[0];
            CodeTypeDeclaration codeTypeDeclaration = codeNamespace.Types[0];
            CodeConstructor codeConstructor = GetCodeConstructor(codeTypeDeclaration);
            codeConstructor.Attributes = MemberAttributes.Public;
        }
        private static void ChangeResourceManager(
            string className, CodeCompileUnit codeCompileUnit, ResourceManagerGetStatementChange getStatementChange)
        {
            CodeNamespace codeNamespace = codeCompileUnit.Namespaces[0];
            CodeTypeDeclaration codeTypeDeclaration = codeNamespace.Types[0];
            CodeMemberProperty codeMemberProperty = GetCodeMemberProperty(codeTypeDeclaration, "ResourceManager");
            if (codeMemberProperty != null)
                ChangeResourceManagerGetStatements(
                    codeNamespace, codeTypeDeclaration, codeMemberProperty, getStatementChange);
        }
        private static void ChangeResourceManagerGetStatements(
            CodeNamespace codeNamespace, CodeTypeDeclaration codeTypeDeclaration,
            CodeMemberProperty codeMemberProperty, ResourceManagerGetStatementChange getStatementChange)
        {
            // change the statements from:-
            // if ((resourceMan == null))
            // {
            //     System.Resources.ResourceManager temp = new System.Resources.ResourceManager("WindowsApplication2.strings", typeof(strings).Assembly);
            //     resourceMan = temp;
            // }
            // return resourceMan;
            //
            // to this:-
            //
            // if ((resourceMan == null))
            // {
            //     System.Resources.ResourceManager temp = Internationalization.ResourceManagerProvider.GetResourceManager("WindowsApplication2.strings", typeof(strings).Assembly);
            //     resourceMan = temp;
            // }
            //
            // or this:-
            //
            // if ((resourceMan == null))
            // {
            //     System.Resources.ResourceManager temp = Internationalization.ResourceManagerProvider.GetResourceManager("WindowsApplication2.strings", "Resources.TestResources", global::System.Reflection.Assembly.Load("App_GlobalResources"));
            //     resourceMan = temp;
            // }
            CodeTypeReference resourceManagerTypeReference = new CodeTypeReference(typeof(ResourceManager));
            CodeFieldReferenceExpression resMgrFieldReferenceExpression = new CodeFieldReferenceExpression(null, "resourceMan");
            CodeExpression ifExpression = new CodeBinaryOperatorExpression(resMgrFieldReferenceExpression, CodeBinaryOperatorType.IdentityEquality, new CodePrimitiveExpression(null));
            
            CodeExpression assemblyExpression;
            if (getStatementChange == ResourceManagerGetStatementChange.Global)
            {
                assemblyExpression = new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(
                    new CodeTypeReferenceExpression(
                    new CodeTypeReference(typeof(Assembly), CodeTypeReferenceOptions.GlobalReference)), "Load"),
                    new CodeExpression[] { new CodePrimitiveExpression("App_GlobalResources") });
            }
            else
                assemblyExpression = new CodePropertyReferenceExpression(new CodeTypeOfExpression(new CodeTypeReference(codeTypeDeclaration.Name)), "Assembly");

            CodeExpression[] resourceManagerParameterExpressions = new CodeExpression[2]
				{
					new CodePrimitiveExpression(codeNamespace.Name + "." + codeTypeDeclaration.Name),
					assemblyExpression
				};
            // CodeObjectCreateExpression newResourceManagerExpression = new CodeObjectCreateExpression(resourceManagerTypeReference, resourceManagerParameterExpressions);
            CodeExpression newResourceManagerExpression =
                new CodeMethodInvokeExpression(new CodeTypeReferenceExpression("Internationalization.Resources.ResourceManagerProvider"), "GetResourceManager",
                resourceManagerParameterExpressions);

            CodeStatement[] ifStatements = new CodeStatement[2]
			{
				new CodeVariableDeclarationStatement(resourceManagerTypeReference, "temp", newResourceManagerExpression),
				// "resourceMan = temp;"
				new CodeAssignStatement(resMgrFieldReferenceExpression, new CodeVariableReferenceExpression("temp"))
			};

            // now that we have collected all of the code assign it to the property's get statements
            CodeStatementCollection statements = new CodeStatementCollection();
            statements.Add(new CodeConditionStatement(ifExpression, ifStatements));
            statements.Add(new CodeMethodReturnStatement(resMgrFieldReferenceExpression));

            codeMemberProperty.GetStatements.Clear();
            codeMemberProperty.GetStatements.AddRange(statements);
        }
        private static CodeMemberProperty GetCodeMemberProperty(CodeTypeDeclaration codeTypeDeclaration, string propertyName)
        {
            foreach (CodeTypeMember codeTypeMember in codeTypeDeclaration.Members)
            {
                if (codeTypeMember is CodeMemberProperty)
                {
                    CodeMemberProperty codeMemberProperty = (CodeMemberProperty)codeTypeMember;
                    if (codeMemberProperty.Name == propertyName)
                        return codeMemberProperty;
                }
            }
            return null;
        }
        private static CodeConstructor GetCodeConstructor(CodeTypeDeclaration codeTypeDeclaration)
        {
            foreach (CodeTypeMember codeTypeMember in codeTypeDeclaration.Members)
            {
                if (codeTypeMember is CodeConstructor)
                {
                    return (CodeConstructor)codeTypeMember;
                }
            }
            return null;
        }
    }
}
