// File           : AlternateSortOrders.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Globalization;

namespace Globalization
{
    /// <summary>
    /// AlternateSortOrders is a support class to get an array of LCIDs of alternate sort orders
    /// </summary>
    public class AlternateSortOrders
    {
        /// <summary>
        /// GetAlternateSortOrders gets an array of LCIDs of cultures with alternate sort orders
        /// </summary>
        /// <returns>An array of LCIDs of cultures with alternate sort orders</returns>
        public static int[] GetAlternateSortOrders()
        {
            const uint LCID_ALTERNATE_SORTS = 4;
            alternateSortOrders = new List<int>();
            EnumSystemLocales(new LocaleEnumProc(AlternateSortsCallback), LCID_ALTERNATE_SORTS);
            int[] alternateSortOrdersArray = new int[alternateSortOrders.Count];
            alternateSortOrders.CopyTo(alternateSortOrdersArray);
            return alternateSortOrdersArray;
        }

        /// <summary>
        /// LocaleEnumProc is a delegate for a method which is called back from EnumSystemLocales
        /// </summary>
        /// <param name="lcidString">A string containing the LCID of a culture with an alternate sort order</param>
        /// <returns>True to continue processing; False to stop processing</returns>
        protected delegate bool LocaleEnumProc(string lcidString);

        /// <summary>
        /// EnumSystemLocales is a Win32 function which enumerates through system locales calling a callback method
        /// </summary>
        /// <param name="lpLocaleEnumProc">The LocaleEnumProc to call back for each locale</param>
        /// <param name="dwFlags">The flags specifying which locales to enumerate</param>
        /// <returns>True if successfull</returns>
        [DllImport("kernel32.dll")]
        protected static extern bool EnumSystemLocales(LocaleEnumProc lpLocaleEnumProc, uint dwFlags);

        /// <summary>
        /// alternateSortOrders is the internal stash of LCIDs collected by AlternateSortsCallback
        /// </summary>
        protected static List<int> alternateSortOrders;

        /// <summary>
        /// AlternateSortsCallback is the method called back by EnumSystemLocales
        /// </summary>
        /// <param name="lcidString">The string of an LCID</param>
        /// <returns>True to continue processing; False to stop processing</returns>
        protected static bool AlternateSortsCallback(string lcidString)
        {
            int LCID;
            if (Int32.TryParse(lcidString, NumberStyles.AllowHexSpecifier, null, out LCID))
                alternateSortOrders.Add(LCID);

            return true;
        }
    }
}
