// File           : CultureInfoEx.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;

namespace Globalization
{
    /// <summary>
    /// CultureInfoEx is an extended CultureInfo class
    /// </summary>
    /// <remarks>CultureInfoEx adds a postal code class to the CultureInfo class.
    /// The purpose of the implementation, however, is to provide a placeholder
    /// which can be used for extending the CultureInfo class with further culture-specific
    /// data and functionality.</remarks>
    public class CultureInfoEx: CultureInfo
    {
        /// <summary>
        /// Initializes a new instance of the CultureInfoEx class 
        /// </summary>
        /// <param name="culture">A predefined CultureInfo identifier or the LCID property of an existing CultureInfo</param>
        public CultureInfoEx(int culture)
            : base(culture)
        {
        }
        /// <summary>
        /// Initializes a new instance of the CultureInfoEx class 
        /// </summary>
        /// <param name="name">A predefined CultureInfo name or the Name of an existing CultureInfo</param>
        public CultureInfoEx(string name)
            : base(name)
        {
        }
        /// <summary>
        /// Initializes a new instance of the CultureInfoEx class 
        /// </summary>
        /// <param name="culture">A predefined CultureInfo identifier or the LCID property of an existing CultureInfo</param>
        /// <param name="useUserOverride">A Boolean that denotes whether to use the user-selected culture
        /// settings (true) or the default culture settings (false)</param>
        public CultureInfoEx(int culture, bool useUserOverride)
            : base(culture, useUserOverride)
        {
        }
        /// <summary>
        /// Initializes a new instance of the CultureInfoEx class 
        /// </summary>
        /// <param name="name">A predefined CultureInfo name or the Name of an existing CultureInfo</param>
        /// <param name="useUserOverride">A Boolean that denotes whether to use the user-selected culture
        /// settings (true) or the default culture settings (false)</param>
        public CultureInfoEx(string name, bool useUserOverride): base(name, useUserOverride)
        {
        }
        private static CultureInfoEx invariantCulture;
        /// <summary>
        /// Gets the CultureInfoEx that is culture-independent (invariant)
        /// </summary>
        public new static CultureInfoEx InvariantCulture
        {
            get
            {
                if (invariantCulture == null)
                {
                    invariantCulture = new CultureInfoEx(0x7f, false);
                    // need to set the invariant culture to read only
                    // but this information is internal and inaccessible
                }
                return invariantCulture;
            }
        }
        /// <summary>
        /// Gets the CultureInfoEx that represents the parent culture of the current CultureInfoEx
        /// </summary>
        public new CultureInfoEx Parent
        {
            get
            {
                CultureInfo parent = base.Parent;
                if (CultureInfo.InvariantCulture.Equals(parent))
                    return CultureInfoEx.InvariantCulture;
                else
                    // change the type of the parent to CultureInfoEx
                    return new CultureInfoEx(parent.Name, UseUserOverride);
            }
        }

        private PostalCodeInfo postalCodeInfo;
        /// <summary>
        /// PostalCode is the PostalCode object which represents the postal code associated with this culture
        /// </summary>
        /// <remarks>The PostalCode is only associated with culture-specific cultures</remarks>
        public PostalCodeInfo PostalCode
        {
            get
            {
                CheckNeutral(this);
                if (postalCodeInfo == null)
                    postalCodeInfo = new PostalCodeInfo(Name);
                return postalCodeInfo;
            }
            set {postalCodeInfo = value;}
        }
        /// <summary>
        /// CheckNeutral enforces that the given CultureInfo is not culture-neutral by throwing an exception
        /// </summary>
        /// <param name="culture">The CultureInfo to check</param>
        protected static void CheckNeutral(CultureInfo culture)
        {
            if (culture.IsNeutralCulture)
            {
                throw new NotSupportedException(EnvironmentEx.GetResourceString(
                    "Argument_CultureInvalidFormat", new object[1] { culture.Name }));
            }
        }
    }
}
