// File           : GlobalizationUtilities.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Windows.Forms;
using System.Globalization;
using System.Collections;

namespace Internationalization.Common
{
    /// <summary>
    /// GlobalizationUtilities is a collection of utilities to aid globalization
    /// </summary>
    public class GlobalizationUtilities
	{
        /// <summary>
        /// GetPersistentCultureName returns an identifier for a CultureInfo which can be safely persisted
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo to get the identifier for</param>
        /// <returns>A string which can be safely persisted</returns>
        /// <remarks>The string can subsequently be used for re-creating a CultureInfo by passing it
        /// to a CultureInfo constructor. The string respects custom cultures and alternative sort orders.
        /// </remarks>
        public static string GetPersistentCultureName(CultureInfo cultureInfo)
        {
            if ((CultureTypes.UserCustomCulture & cultureInfo.CultureTypes) != (CultureTypes)0)
                return cultureInfo.Name;
            else
                return cultureInfo.CompareInfo.Name;
        }
        /// <summary>
        /// IsRightToLeft returns true if a control's RightToLeft is Yes or it is Inherit and it's parent is Yes
        /// </summary>
        /// <param name="control">The control to test RightToLeft for</param>
        /// <returns>True if the control's RightToLeft is Yes or it is Inherit and it's parent is Yes</returns>
		public static bool IsRightToLeft(Control control)
		{
			if (control.RightToLeft == RightToLeft.Yes)
				return true;
			else if (control.RightToLeft == RightToLeft.No)
				return false;
			else if (control.Parent == null)
				return false;
			else
				return IsRightToLeft(control.Parent);
		}
        /// <summary>
        /// RtlMessageBoxOptions returns the correct MessageBoxOptions based upon the Right-To-Left settings
        /// of the given control
        /// </summary>
        /// <param name="control">The control which contains RTL or LTR settings</param>
        /// <returns>The MessageBoxOptions for the given control</returns>
        /// <remarks>Use this method in conjunction with MessageBox.Show to reduce the complexity of such
        /// calls while still making the message box adapt to RTL or LTR cultures correctly.</remarks>
		public static MessageBoxOptions RtlMessageBoxOptions(Control control)
		{
			if (IsRightToLeft(control))
				return MessageBoxOptions.RightAlign | MessageBoxOptions.RtlReading;
			else
				return (MessageBoxOptions) 0;
		}
        /// <summary>
        /// RtlMessageBoxOptions returns the correct MessageBoxOptions based upon the Right-To-Left settings
        /// of the current UI culture
        /// </summary>
        /// <returns>The MessageBoxOptions for the current UI culture</returns>
        /// <remarks>Use this method in conjunction with MessageBox.Show to reduce the complexity of such
        /// calls while still making the message box adapt to RTL or LTR cultures correctly.</remarks>
        public static MessageBoxOptions RtlMessageBoxOptions()
        {
            return RtlMessageBoxOptions(CultureInfo.CurrentUICulture);
        }
        /// <summary>
        /// RtlMessageBoxOptions returns the correct MessageBoxOptions based upon the Right-To-Left settings
        /// of the given CultureInfo
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo that indicates whether the culture is LTR or RTL</param>
        /// <returns>The MessageBoxOptions for the given culture</returns>
        /// <remarks>Use this method in conjunction with MessageBox.Show to reduce the complexity of such
        /// calls while still making the message box adapt to RTL or LTR cultures correctly.</remarks>
        public static MessageBoxOptions RtlMessageBoxOptions(CultureInfo cultureInfo)
        {
            if (cultureInfo.TextInfo.IsRightToLeft)
                return MessageBoxOptions.RightAlign | MessageBoxOptions.RtlReading;
            else
                return (MessageBoxOptions)0;
        }
        /// <summary>
        /// SetInputLanguage sets the keyboard input language to the installed keyboard which matches the CurrentUICulture
        /// </summary>
        /// <returns>True if an installed keyboard language could be found which matched the CurrentUICulture</returns>
        /// <remarks>This method is typically used after changing the CurrentUICulture</remarks>
        public static bool SetInputLanguage()
		{
			return SetInputLanguage(CultureInfo.CurrentUICulture);
		}
        /// <summary>
        /// SetInputLanguage sets the keyboard input language to the installed keyboard which matches the given culture
        /// </summary>
        /// <param name="cultureInfo">The CultureInfo which determines the keyboard input language</param>
        /// <returns>True if an installed keyboard language could be found which matched the given culture</returns>
        /// <remarks>This method is typically used after changing the CurrentUICulture</remarks>
        public static bool SetInputLanguage(CultureInfo cultureInfo)
		{
            // This method has been updated since the book was written.
            // See GlobalizationUtilities.cs.Original to see the original method.
            InputLanguage newInputLanguage = InputLanguage.FromCulture(cultureInfo);
            if (newInputLanguage == null)
                return false;

            InputLanguage.CurrentInputLanguage = newInputLanguage;

            return true;
		}
        /// <summary>
        /// GetRegions gets a list of RegionInfo objects
        /// </summary>
        /// <returns>An array of RegionInfo objects</returns>
        public static RegionInfo[] GetRegions()
        {
            Hashtable regionInfos = new Hashtable();
            foreach (CultureInfo cultureInfo in CultureInfo.GetCultures(CultureTypes.SpecificCultures))
            {
                RegionInfo regionInfo = new RegionInfo(cultureInfo.Name);
                if (regionInfos[regionInfo.ThreeLetterISORegionName] == null)
                    regionInfos.Add(regionInfo.ThreeLetterISORegionName, regionInfo);
            }
            RegionInfo[] regionInfoArray = new RegionInfo[regionInfos.Count];
            regionInfos.Values.CopyTo(regionInfoArray, 0);
            return regionInfoArray;
        }
    }
}
