// File           : ResourceUtilities.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Resources;
using System.Reflection;

namespace Internationalization.Common
{
	/// <summary>
	/// ResourceUtilities is a collection of utilities to aid in resource manipulation
	/// </summary>
	public class ResourceUtilities
	{
        /// <summary>
        /// IsLocalizable determines if a resource contains a string which should be localized
        /// </summary>
        /// <param name="entry">The DictionaryEntry representing the resource</param>
        /// <returns>True if the resource is a string which should be localized</returns>
        /// <remarks>This method is deprecated because it is not suitable for use with DictionaryEntry
        /// objects which are data nodes. In such entries the DictionaryEntry.Value is a
        /// data node such as ResXDataNode and this is not a string so this method always
        /// returns false. Use the overloaded IsLocalizable instead and pass the value of
        /// the data node instead of the data node itself.</remarks>
        [Obsolete]
        public static bool IsLocalizable(DictionaryEntry entry)
        {
            return IsLocalizable(entry.Key.ToString(), entry.Value);
        }
        /// <summary>
        /// IsLocalizable determines if a resource contains a string which should be localized
        /// </summary>
        /// <param name="resourceKey">The key used to identify the resource</param>
        /// <param name="resourceValue">The value of the resource</param>
        /// <returns>True if the resource is a string which should be localized</returns>
        public static bool IsLocalizable(string resourceKey, object resourceValue)
        {
			if (resourceKey == "$this.Text" || ! resourceKey.StartsWith("$"))
			{
				if (resourceValue == null || (
					resourceValue.GetType().ToString() == "System.String" &&
                    ! (resourceKey.StartsWith(">>") || IsXml(resourceValue.ToString())
                    || resourceKey.EndsWith(".ImageUrl"))))
					return true;
			}
			return false;
		}
        /// <summary>
        /// IsXml returns true if a string starts with XML
        /// </summary>
        /// <param name="xml">The string which may or may not contain XML</param>
        /// <returns>True if the string starts with XML</returns>
        protected static bool IsXml(string xml)
        {
            return xml.StartsWith("<?xml ");
        }
        /// <summary>
        /// GetResourceSetTable gets the Hashtable of resource entries represented by the ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to get the Hashtable from</param>
        /// <returns>The Hashtable of resource entries represented by the ResourceSet</returns>
        public static Hashtable GetResourceSetTable(ResourceSet resourceSet)
        {
            FieldInfo fieldInfo =
                resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
            if (fieldInfo != null)
            {
                object tableObject = fieldInfo.GetValue(resourceSet);
                if (tableObject != null)
                    return (Hashtable)tableObject;
            }
            return null;
        }
        /// <summary>
        /// SetResourceSetTable sets the Hashtable of resource entries represented by the ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet to set the Hashtable of</param>
        /// <param name="hashtable">The Hashtable of resource entries to set the ResourceSet to</param>
        public static void SetResourceSetTable(ResourceSet resourceSet, Hashtable hashtable)
        {
            FieldInfo fieldInfo =
                resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
            if (fieldInfo != null)
            {
                fieldInfo.SetValue(resourceSet, hashtable);
            }
        }
    }
}
