// File           : AltaVistaTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;

namespace Internationalization.Translation
{
	/// <summary>
    /// AltaVistaTranslator is an HTML translator for babelfish.altavista.com
	/// </summary>
	public class AltaVistaTranslator: HtmlTranslator
	{
        /// <summary>
        /// Constructs an AltaVistaTranslator object
        /// </summary>
		public AltaVistaTranslator():
            base(AltaVistaTranslatorResources.Name, @"http://babelfish.altavista.com/tr", new string[,]
			{
				{"en", "zh-CHS"},
				{"en", "zh-CHT"},
				{"en", "nl"},
				{"en", "fr"},
				{"en", "de"},
				{"en", "el"},
				{"en", "it"},
				{"en", "ja"},
				{"en", "ko"},
				{"en", "pt"},
				{"en", "ru"},
				{"en", "es"},
				{"zh-CHS", "en"},
				{"zh-CHT", "en"},
				{"nl", "en"},
				{"nl", "fr"},
				{"fr", "nl"},
				{"fr", "en"},
				{"fr", "de"},
				{"fr", "el"},
				{"fr", "it"},
				{"fr", "pt"},
				{"fr", "es"},
				{"de", "en"},
				{"de", "fr"},
				{"el", "en"},
				{"el", "fr"},
				{"it", "en"},
				{"it", "fr"},
				{"ja", "en"},
				{"ko", "en"},
				{"pt", "en"},
				{"pt", "fr"},
				{"ru", "en"},
				{"es", "en"},
				{"es", "fr"}
			})
		{
		}
        /// <summary>
        /// DotNetLanguageCodeToLanguageCode converts a .NET language code to an AltaVista language code
        /// </summary>
        /// <param name="language">The .NET language code</param>
        /// <returns>An AltaVista language code</returns>
		protected virtual string DotNetLanguageCodeToLanguageCode(string language)
		{
			// check for a couple of adjustments to the language codes
			if (language == "zh-CHS")
				// Chinese (Simplified)
				return "zh";
			else if (language == "zh-CHT")
				// Chinese (Traditional)
				return "zt";
			else
				return language;
		}
        /// <summary>
        /// GetPostData gets the website-specific post data for sending to the URL
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to be translated</param>
        /// <returns>The post data to send to the URL</returns>
        public override string GetPostData(string inputLanguage, string outputLanguage, string text)
		{
			string languagePair = DotNetLanguageCodeToLanguageCode(inputLanguage) + 
				"_" + DotNetLanguageCodeToLanguageCode(outputLanguage);

			return "doit=done&intl=1&tt=urltext&trtext=" + Encode(text) + "&lp=" + languagePair;
		}
        /// <summary>
        /// GetTranslation extracts the translated text from the inner text returned from the website
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="innerText">The inner text returned from the website</param>
        /// <returns>The translated text</returns>
        /// <remarks>This is the most fragile part of an HtmlTranslator's implementation.
        /// If this translator fails check this method first. The most likely reason for
        /// failure is that the resulting HTML (and therefore inner text) has changed.</remarks>
        protected override string GetTranslation(string inputLanguage, string outputLanguage, string innerText)
		{
			int index = innerText.IndexOf("Babel Fish Translation");
			if (index == -1)
				return String.Empty;

			innerText = innerText.Substring(index + 2);

			index = innerText.IndexOf(":");
			if (index == -1)
				return String.Empty;

			innerText = innerText.Substring(index + 3);

			index = innerText.IndexOf("Translate again");
			if (index == -1)
				return String.Empty;

			return innerText.Substring(0, index - 2).TrimEnd(new char[] {' ', (char) 10, (char) 13});
		}
	}
}
