// File           : FreeTranslationTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;

namespace Internationalization.Translation
{
	/// <summary>
	/// FreeTranslationTranslator is an HTML translator for freetranslation.com
	/// </summary>
	public class FreeTranslationTranslator: HtmlTranslator
	{
        /// <summary>
        /// Constructs a FreeTranslationTranslator object
        /// </summary>
		public FreeTranslationTranslator():
            base(FreeTranslationTranslatorResources.Name, @"http://ets.freetranslation.com", new string[,]
				{
					{"en", "es"},
					{"en", "fr"},
					{"en", "de"},
					{"en", "it"},
					{"en", "nl"},
					{"en", "pt"},
					{"en", "ru"},
					{"en", "no"},
					{"en", "zh-CHS"},
					{"en", "zh-CHT"},
					{"es", "en"},
					{"fr", "en"},
					{"de", "en"},
					{"it", "en"},
					{"pt", "en"},
					{"ru", "en"}
				})
		{
		}
        /// <summary>
        /// LanguageName converts the .NET language code to a FreeTranslation language code
        /// </summary>
        /// <param name="languageCode">The .NET language code</param>
        /// <returns>A FreeTranslation language code</returns>
		protected virtual string LanguageName(string languageCode)
		{
			// deal with the irregular language names first
			if (languageCode == "zh-CHS")
				return "SimplifiedChinese";
			else if (languageCode == "zh-CHT")
				return "TraditionalChinese";
			else
				return new CultureInfo(languageCode).EnglishName;
		}
        /// <summary>
        /// GetPostData gets the website-specific post data for sending to the URL
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to be translated</param>
        /// <returns>The post data to send to the URL</returns>
        public override string GetPostData(string inputLanguage, string outputLanguage, string text)
		{
			string inputLanguageName  = LanguageName(inputLanguage);
			string outputLanguageName = LanguageName(outputLanguage);
			string languagePair = inputLanguageName + "%2F" + outputLanguageName;
            return "sequence=core&mode=html&charset=UTF-8&template=results_en-us.htm&language=" + languagePair + "&srctext=" + Encode(text);
		}
        /// <summary>
        /// GetTranslation extracts the translated text from the inner text returned from the website
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="innerText">The inner text returned from the website</param>
        /// <returns>The translated text</returns>
        /// <remarks>This is the most fragile part of an HtmlTranslator's implementation.
        /// If this translator fails check this method first. The most likely reason for
        /// failure is that the resulting HTML (and therefore inner text) has changed.</remarks>
        protected override string GetTranslation(string inputLanguage, string outputLanguage, string innerText)
		{
			string startText = "Your FREE Translation *";

			int index = innerText.IndexOf(startText);
			if (index == -1)
				return String.Empty;

			innerText = innerText.Substring(index + startText.Length);

            string endText = "Have this text professionally translated";
			index = innerText.IndexOf(endText);
            if (index == -1)
            {
                index = innerText.IndexOf("Instantly translate text from any application on your PC!");
                if (index == -1)
                    return String.Empty;
            }

			return innerText.Substring(0, index - 2);
		}
	}
}
