// File           : Office2003ResearchServicesTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Text;
using Internationalization.Translation;

namespace Internationalization.Translation.ResearchServices
{
	/// <summary>
	/// Office2003ResearchServicesTranslator is a translator which uses Office 2003 Research Services
	/// </summary>
	public class Office2003ResearchServicesTranslator: Translator
	{
		private ResearchServiceProviderCollection providers;

        /// <summary>
        /// Constructs an Office2003ResearchServicesTranslator object
        /// </summary>
        public Office2003ResearchServicesTranslator(): base(Office2003ResearchServicesTranslatorResources.Name)
		{
			InitializeProviders();
		}
        /// <summary>
        /// InitializeProviders initializes the Office 2003 Research Service providers
        /// </summary>
		protected virtual void InitializeProviders()
		{
			providers = new ResearchServiceProviderCollection();
			providers.Add(new WorldLingoServiceProvider());
		}
        /// <summary>
        /// LanguagePairs is an array of supported language pair strings e.g. {{"en", "fr"}, {"en", "de"}}
        /// </summary>
        public override string[,] LanguagePairs
		{
			get
			{
				if (providers.Count == 0)
					// there are no providers
					return null;
				else if (providers.Count == 1)
				{
					// there is only one provider so just return its language pairs
					return providers[0].LanguagePairs;
				}
				else
				{
					// Combine all of the language pairs from all of the providers together.
					// get the total number of language pairs in all providers
					int languagePairCount = 0;
					foreach(ResearchServiceProvider provider in providers)
					{
						languagePairCount += provider.LanguagePairs.GetLength(0);
					}
                    string[,] languagePairs = new string[languagePairCount, 2];
					int languagePairNumber = 0;
					foreach(ResearchServiceProvider provider in providers)
					{
						string[,] providerLanguagePairs = provider.LanguagePairs;
						for(int pairNumber = 0; pairNumber < providerLanguagePairs.GetLength(0); pairNumber++)
						{
							languagePairs[languagePairNumber, 0] = providerLanguagePairs[pairNumber, 0];
							languagePairs[languagePairNumber, 1] = providerLanguagePairs[pairNumber, 1];
							languagePairNumber++;
						}
					}
					return languagePairs;
				}
			}
		}
        /// <summary>
        /// IsSupported returns true if the given language pair is supported
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>True if the given language pair is supported by the translator</returns>
        public override bool IsSupported(string inputLanguage, string outputLanguage)
		{
			foreach(ResearchServiceProvider provider in providers)
			{
				if (provider.IsSupported(inputLanguage, outputLanguage))
					return true;
			}
			return false;
		}
        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
		{
			foreach(ResearchServiceProvider provider in providers)
			{
				if (provider.IsSupported(inputLanguage, outputLanguage))
					return provider.Translate(inputLanguage, outputLanguage, text);
			}

			throw new LanguageCombinationNotSupportedException(
				LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                inputLanguage, outputLanguage);
		}
	}
}
