// File           : ResearchServices.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using Microsoft.Win32;

namespace Internationalization.Translation.ResearchServices
{
    /// <summary>
    /// ResearchServiceProvider is a base class for Office 2003 Research Service providers
    /// </summary>
	public class ResearchServiceProvider
	{
		private string id;
		private string aboutPath;
		private string name;
		private string queryPath;
		private string registrationPath;
		private string revision;
		private string status;
		private string type;
		private string updateStatus;
		private ResearchServiceCollection services = new ResearchServiceCollection();
        /// <summary>
        /// Constructs a ResearchServiceProvider object
        /// </summary>
        /// <param name="name">The name of the Research Service Provider</param>
		public ResearchServiceProvider(string name)
		{
			this.name = name;
		}
        /// <summary>
        /// LoadFromRegistry populates the ResearchServiceProvider with the information from the registry
        /// </summary>
        /// <param name="providerGuid">The GUID of the Research Service Provider</param>
		public void LoadFromRegistry(string providerGuid)
		{
			string sourcesKeyName = @"Software\Microsoft\Office\11.0\Common\Research\Sources";
			RegistryKey providerKey = Registry.CurrentUser.OpenSubKey(sourcesKeyName + @"\" + providerGuid);

			if (providerKey == null)
				throw new TranslationException(String.Format(
                    ResearchServicesResources.RegistryKeyNotFound, providerGuid));

			Name             = providerKey.GetValue("ProviderName").ToString();
			Id               = providerGuid;
			AboutPath        = providerKey.GetValue("AboutPath").ToString();
			QueryPath        = providerKey.GetValue("QueryPath").ToString();
			RegistrationPath = providerKey.GetValue("RegistrationPath").ToString();
			Revision         = providerKey.GetValue("Revision").ToString();
			Status           = providerKey.GetValue("Status").ToString();
			Type             = providerKey.GetValue("Type").ToString();
			UpdateStatus     = providerKey.GetValue("UpdateStatus").ToString();

			// load the provider's services
			string[] serviceKeyNames = providerKey.GetSubKeyNames();
			foreach(string serviceKeyName in serviceKeyNames)
			{
				RegistryKey serviceKey = Registry.CurrentUser.OpenSubKey(
					sourcesKeyName + @"\" + providerGuid + @"\" + serviceKeyName);

				if (serviceKey.GetValue("CategoryId").ToString() == "907149312") // "0x36120000"
				{
					ResearchService service = 
						new ResearchService(this, serviceKey.GetValue("ServiceName").ToString());
					service.Id = serviceKeyName;

					object sourceData = serviceKey.GetValue("SourceData");
					// ("SourceData" ends with "/1" for keyword translators)
					if (sourceData != null && 
						(sourceData.ToString().EndsWith(@"/1") || sourceData.ToString().EndsWith(@"/4")))
					{
						service.SourceData = sourceData.ToString();

						Services.Add(service);
					}
				}
			}
		}
        /// <summary>
        /// The path to "About" information
        /// </summary>
		public string AboutPath
		{
			get {return aboutPath;}
			set {aboutPath = value;}
		}
        /// <summary>
        /// The GUID identifying the provider
        /// </summary>
		public string Id
		{
			get {return id;}
			set {id = value;}
		}
        /// <summary>
        /// The name of the provider
        /// </summary>
		public string Name
		{
			get {return name;}
			set {name = value;}
		}
        /// <summary>
        /// The path used for queries
        /// </summary>
		public string QueryPath
		{
			get {return queryPath;}
			set {queryPath = value;}
		}
        /// <summary>
        /// The path used for registration
        /// </summary>
		public string RegistrationPath
		{
			get {return registrationPath;}
			set {registrationPath = value;}
		}
        /// <summary>
        /// The revision number of the provider
        /// </summary>
		public string Revision
		{
			get {return revision;}
			set {revision = value;}
		}
        /// <summary>
        /// The status of the provider
        /// </summary>
		public string Status
		{
			get {return status;}
			set {status = value;}
		}
        /// <summary>
        /// The type of the provider
        /// </summary>
		public string Type
		{
			get {return type;}
			set {type = value;}
		}
        /// <summary>
        /// The update status of the provider
        /// </summary>
		public string UpdateStatus
		{
			get {return updateStatus;}
			set {updateStatus = value;}
		}
        /// <summary>
        /// The collection of services provided by the provider
        /// </summary>
		public ResearchServiceCollection Services
		{
			get {return services;}
			set {services = value;}
		}
        /// <summary>
        /// IsSupported returns true if the given language pair is supported
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>True if the given language pair is supported by the translator</returns>
        public bool IsSupported(string inputLanguage, string outputLanguage)
		{
			return GetService(inputLanguage, outputLanguage) != null;
		}
        /// <summary>
        /// GetService gets the service which supports the given language pair
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>The ResearchService which supports the given language pair</returns>
		protected ResearchService GetService(string inputLanguage, string outputLanguage)
		{
			foreach(ResearchService service in services)
			{
				if (service.IsSupported(inputLanguage, outputLanguage))
					return service;
			}
			return null;
		}
        /// <summary>
        /// LanguagePairs is an array of language pair strings supported by the translator e.g. {{"en", "fr"}, {"en", "de"}}
        /// </summary>
        public string[,] LanguagePairs
		{
			get
			{
				string[,] languagePairs = new string[services.Count, 2];
				int languagePairNumber = 0;
				foreach(ResearchService service in services)
				{
					languagePairs[languagePairNumber, 0] = service.FromCulture.Name;
					languagePairs[languagePairNumber, 1] = service.ToCulture.Name;
					languagePairNumber++;
				}
				return languagePairs;
			}
		}
        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        /// <remarks>This implementation serves only to aid in development. In nearly all cases this method
        /// will be overridden by the subclass.</remarks>
        public virtual string Translate(string inputLanguage, string outputLanguage, string text)
		{
			return text;
		}
	}
    /// <summary>
    /// ResearchService encapsulates a single service provided by the provider
    /// </summary>
    /// <remarks>The service represents a single language pair (which manifests in the FromCulture and
    /// ToCulture properties)</remarks>
	public class ResearchService
	{
		private string id;
		private string name;
		private string sourceData;
		private ResearchServiceProvider provider;
		private CultureInfo fromCulture;
		private CultureInfo toCulture;
        /// <summary>
        /// Constructs a ResearchService object
        /// </summary>
        /// <param name="provider">The ResearchServiceProvider to which this service belongs</param>
        /// <param name="name">The name of the service</param>
		public ResearchService(ResearchServiceProvider provider, string name)
		{
			this.provider = provider;
			this.name = name;
		}
        /// <summary>
        /// The ID of the service
        /// </summary>
		public string Id
		{
			get {return id;}
			set {id = value;}
		}
        /// <summary>
        /// The name of the service
        /// </summary>
		public string Name
		{
			get {return name;}
			set {name = value;}
		}
        /// <summary>
        /// The "source data" of the service
        /// </summary>
        /// <remarks>Setting the SourceData also sets the FromCulture and ToCulture properties.
        /// SourceData should be in the format fromLCID/toLCID[/serviceType] (e.g. 1033/1036/4).</remarks>
		public string SourceData
		{
			get {return sourceData;}
			set
			{
				sourceData = value;
				if (sourceData != null && sourceData.Length >= 4)
				{
					fromCulture = GetCultureFromCode(sourceData.Substring(0, 4));
					if (sourceData.Length >= 9)
						toCulture = GetCultureFromCode(sourceData.Substring(5, 4));
					else
						toCulture = null;
				}
				else
				{
					fromCulture = null;
					toCulture = null;
				}
			}
		}
        /// <summary>
        /// GetCultureFromCode returns a CultureInfo from an LCID string
        /// </summary>
        /// <param name="code">An LCID passed as a string</param>
        /// <returns>A CultureInfo object</returns>
		protected virtual CultureInfo GetCultureFromCode(string code)
		{
			CultureInfo[] cultureInfos = CultureInfo.GetCultures(CultureTypes.AllCultures);
			foreach(CultureInfo cultureInfo in cultureInfos)
			{
				if (cultureInfo.LCID.ToString() == code)
					return cultureInfo;
			}
			return null;
		}
        /// <summary>
        /// Provider is the ResearchServiceProvider to which the service belongs
        /// </summary>
		public ResearchServiceProvider Provider
		{
			get {return provider;}
			set {provider = value;}
		}
        /// <summary>
        /// FromCulture is the CultureInfo of the "from" or "input" language
        /// </summary>
		public CultureInfo FromCulture
		{
			get {return fromCulture;}
		}
        /// <summary>
        /// ToCulture is the CultureInfo of the "to" or "output" language
        /// </summary>
		public CultureInfo ToCulture
		{
			get {return toCulture;}
		}
        /// <summary>
        /// IsSupported returns true if the given language pair is supported by the translator
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>True if the given language pair is supported by the translator</returns>
        public bool IsSupported(string inputLanguage, string outputLanguage)
		{
			return fromCulture != null && fromCulture.Name.StartsWith(inputLanguage) 
				&& toCulture != null && toCulture.Name.StartsWith(outputLanguage);
		}
	}
}
