// File           : WorldLingoServiceProvider.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Text;
using System.IO;
using System.Net;
using System.Xml;
using System.Text.RegularExpressions;

namespace Internationalization.Translation.ResearchServices
{
	/// <summary>
	/// WorldLingoServiceProvider is a ResearchServiceProvider for WorldLingo services
	/// </summary>
	public class WorldLingoServiceProvider: ResearchServiceProvider
	{
        /// <summary>
        /// Constructs a WorldLingoServiceProvider object
        /// </summary>
		public WorldLingoServiceProvider(): base(String.Empty)
		{
			LoadFromRegistry("{ADD57F66-4BB5-4C48-879B-2F70199EEBBE}");
		}
        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
		{
			// get the research service which supports this language pair
			ResearchService service = GetService(inputLanguage, outputLanguage);
			if (service == null)
				throw new LanguageCombinationNotSupportedException(
					LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

			// get the query XML string
			string queryXml = GetQueryXml(Translator.ConvertToEscapedNumerics(text), service.Id, "(11.0.6360)");

			// send the query and get the response
			string responseXml = Query(queryXml);

			// extract the translated text from the response
			return GetTranslatedText(responseXml);
		}
        /// <summary>
        /// GetQueryXml gets the XML query packet which is sent to the provider
        /// </summary>
        /// <param name="queryWord">The word(s) to the translated</param>
        /// <param name="queryDomain">The GUID of the service used for the translation</param>
        /// <param name="build">The build number of the service</param>
        /// <returns>The XML query packet which is sent to the provider</returns>
		protected virtual string GetQueryXml(string queryWord, string queryDomain, string build)
		{
			MemoryStream queryMemoryStream = new MemoryStream();
			XmlTextWriter queryDocument = new XmlTextWriter(queryMemoryStream, null);
			queryDocument.WriteStartElement("QueryPacket");
			queryDocument.WriteAttributeString("xmlns", null, "urn:Microsoft.Search.Query");
			queryDocument.WriteAttributeString("revision", null, "1");
			queryDocument.WriteAttributeString("build", null, build);

			queryDocument.WriteStartElement("Query");
			queryDocument.WriteAttributeString("domain", null, queryDomain);

			queryDocument.WriteStartElement("QueryId");
			queryDocument.WriteString("{" + Guid.NewGuid().ToString() + "}");
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("OriginatorId");
			queryDocument.WriteString("{F6FF7BE0-F39C-4ddc-A7D0-09A4C6C647A5}");
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("SupportedFormats");
			queryDocument.WriteStartElement("Format");
			queryDocument.WriteAttributeString("revision", null, "1");
			queryDocument.WriteString("urn:Microsoft.Search.Response.Content:Content");
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("Context");
			queryDocument.WriteStartElement("QueryText");
			queryDocument.WriteAttributeString("type", null, "STRING");
			queryDocument.WriteAttributeString("language", null, "en-gb");
			queryDocument.WriteString(queryWord);
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("LanguagePreference");
			queryDocument.WriteString("en-us");
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("Requery");
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("Range");
			queryDocument.WriteAttributeString("id", null, "result");
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("OfficeContext");
			queryDocument.WriteAttributeString("xmlns", null, "urn:Microsoft.Search.Query.Office.Context");
			queryDocument.WriteAttributeString("revision", null, "1");
			queryDocument.WriteStartElement("UserPreferences");
			queryDocument.WriteStartElement("ParentalControl");
			queryDocument.WriteString("false");
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("ServiceData");
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("ApplicationContext");
			queryDocument.WriteStartElement("Name");
			queryDocument.WriteString("Microsoft Office Word");
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("Version");
			queryDocument.WriteString(build);
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("SystemInformation");
			queryDocument.WriteStartElement("SkuLanguage");
			queryDocument.WriteString("en-us");
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("LanguagePack");
			queryDocument.WriteString("en-us");
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("InterfaceLanguage");
			queryDocument.WriteString("en-us");
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("Location");
			queryDocument.WriteString("GB");
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("QueryLanguage");
			queryDocument.WriteString("en-gb");
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("KeyboardLanguage");
			queryDocument.WriteString("en-gb");
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();

			queryDocument.WriteStartElement("Keywords");
			queryDocument.WriteAttributeString("xmlns", null, "urn:Microsoft.Search.Query.Office.Keywords");
			queryDocument.WriteAttributeString("revision", null, "1");
			queryDocument.WriteStartElement("QueryText");
			queryDocument.WriteString(queryWord);
			queryDocument.WriteEndElement();
			queryDocument.WriteStartElement("Keyword");
			queryDocument.WriteStartElement("Word");
			queryDocument.WriteString(queryWord);
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();
			
			queryDocument.WriteEndElement();
			queryDocument.WriteEndElement();
			queryDocument.Close();
			return System.Text.Encoding.UTF8.GetString(queryMemoryStream.ToArray());
		}
        /// <summary>
        /// GetTranslatedText extracts the translated text from the XML response packet returned from the provider
        /// </summary>
        /// <param name="responseXml">The XML response packet returned from the provider</param>
        /// <returns>The translated text extracted from the XML response packet</returns>
		protected virtual string GetTranslatedText(string responseXml)
		{
			XmlTextReader responseReader = new XmlTextReader(new StringReader(responseXml));
			responseReader.MoveToContent();
			string translatedText = String.Empty;
			while (responseReader.Read())
			{
				if (responseReader.NodeType == XmlNodeType.Element && responseReader.Name == "c:P")
				{
					translatedText = HtmlToText(responseReader.ReadInnerXml());
					translatedText = translatedText.Trim(new char[] {(char) 10, (char) 13});
				}
			}
			return translatedText;
		}
        /// <summary>
        /// HtmlToText returns plain text from a string of HTML
        /// </summary>
        /// <param name="html">The HTML containing the plain text</param>
        /// <returns>The plain text</returns>
		protected virtual string HtmlToText(string html)
		{
			Regex regex = new Regex(@"<(.|\n)+?>", RegexOptions.IgnoreCase);
			return regex.Replace(html, String.Empty);
		}
        /// <summary>
        /// Query sends an XML query to the provider and returns the XML response
        /// </summary>
        /// <param name="queryXml">The XML query packet to send to the provider</param>
        /// <returns>The XML response packet returned from the provider</returns>
		protected virtual string Query(string queryXml)
		{
			string queryPacket = CreateSoapPacket(queryXml);
			string responsePacket =	SendRequest(QueryPath, queryPacket);
			return RemoveSoapPacket(responsePacket);
		}
        /// <summary>
        /// CreateSoapPacket creates a SOAP query packet for an XML packet
        /// </summary>
        /// <param name="xml">The XML to include in the SOAP packet</param>
        /// <returns>A SOAP query packet which includes the given XML</returns>
		protected virtual string CreateSoapPacket(string xml)
		{
			string packetStart = @"<?xml version=""1.0"" encoding=""utf-8""?><s:Envelope xmlns:s=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:xsd=""http://www.w3.org/2001/XMLSchema"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"">" +
				@"<s:Body><Query xmlns=""urn:Microsoft.Search""><queryXml xsi:type=""xsd:string"">";

			string packetEnd = @"</queryXml></Query></s:Body></s:Envelope>";

			return packetStart + XmlToText(xml) + packetEnd;
		}
        /// <summary>
        /// XmlToText converts an XML packet to a format suitable for inclusion in a SOAP packet
        /// </summary>
        /// <param name="xml">The XML packet to format</param>
        /// <returns>The formatted XML packet</returns>
		protected virtual string XmlToText(string xml)
		{
			xml = xml.Replace("<", "&lt;");
			xml = xml.Replace(">", "&gt;");
			xml = xml.Replace(@"""", "&#39;");
			return xml;
		}
        /// <summary>
        /// TextToXml converts formatted text to XML
        /// </summary>
        /// <param name="text">Formatted text</param>
        /// <returns>An XML packet</returns>
		protected virtual string TextToXml(string text)
		{
			text = text.Replace("&lt;" , "<"  );
			text = text.Replace("&gt;" , ">"  );
			text = text.Replace("&#39;", @"""");
			text = text.Replace("&amp;", @"&");
			text = Translator.ConvertFromEscapedNumerics(text);
			return text;
		}
        /// <summary>
        /// SendRequest sends the given SOAP request to the given URL
        /// </summary>
        /// <param name="url">The URL to send the request to</param>
        /// <param name="soapPacket">The SOAP request to send to the URL</param>
        /// <returns>The SOAP response returned from the URL</returns>
		protected string SendRequest(string url, string soapPacket)
		{
			HttpWebRequest httpWebRequest = (HttpWebRequest) WebRequest.Create(url);
			httpWebRequest.ContentType = "text/xml; charset=utf-8";
			httpWebRequest.Headers.Add("SOAPAction: urn:Microsoft.Search/Query");
			httpWebRequest.Method = "POST";
			httpWebRequest.ProtocolVersion = HttpVersion.Version10;

            Stream stream = httpWebRequest.GetRequestStream();
			StreamWriter streamWriter = new StreamWriter(stream);
			streamWriter.Write(soapPacket);
			streamWriter.Close();

			WebResponse webResponse = httpWebRequest.GetResponse();
			Stream responseStream = webResponse.GetResponseStream();
			StreamReader responseStreamReader = new StreamReader(responseStream);
			return responseStreamReader.ReadToEnd();
		}
        /// <summary>
        /// RemoveSoapPacket extracts the embeeded XML packet from a SOAP packet
        /// </summary>
        /// <param name="soapPacket">The SOAP packet</param>
        /// <returns>The XML contained within the SOAP packet</returns>
		protected string RemoveSoapPacket(string soapPacket)
		{
			XmlTextReader reader = new XmlTextReader(new StringReader(soapPacket));
			reader.MoveToContent();
			while (reader.Read())
			{
				if (reader.NodeType == XmlNodeType.Element && reader.Name == "ns1:QueryResponse")
				{
					reader.MoveToContent();
					reader.Read();
					reader.Read();
					string text = reader.ReadInnerXml();
					return TextToXml(text);
				}
			}
			return soapPacket;
		}
	}
}
