// File           : WebServiceXTranslator.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Runtime.InteropServices;

namespace Internationalization.Translation
{
	/// <summary>
	/// WebServiceXTranslator is a translator which uses a translation service called WebServiceX
	/// </summary>
	public class WebServiceXTranslator: Translator
	{
		private net.webservicex.www.TranslateService translateService;

        /// <summary>
        /// Constructs a WebServiceXTranslator object
        /// </summary>
		public WebServiceXTranslator(): base(WebServiceXTranslatorResources.Name, new string[,]
			{
				{"en", "zh-CHS"},
				{"en", "fr"},
				{"en", "de"},
				{"en", "it"},
				{"en", "ja"},
				{"en", "ko"},
				{"en", "pt"},
				{"en", "es"},
				{"fr", "en"},
				{"fr", "de"},
				{"de", "en"},
				{"de", "fr"},
				{"it", "en"},
				{"es", "en"}
			})
		{
		}
// Scanning the hard coded list is faster than calling the web service but, of course,
// the web service is more likely to be up to date. I opted for performance in this instance.
// Uncomment this code if you prefer accuracy to performance.
//		public override bool IsSupported(string inputLanguage, string outputLanguage)
//		{
//			net.webservicex.www.Language language = TranslationServiceLanguage(inputLanguage, outputLanguage);
//			return ((int) language) > -1;
//		}
        /// <summary>
        /// Translate translates the input text from the input language to the output language
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <param name="text">The text to translate</param>
        /// <returns>The translated text</returns>
        public override string Translate(string inputLanguage, string outputLanguage, string text)
		{
			if (! IsSupported(inputLanguage, outputLanguage))
				throw new LanguageCombinationNotSupportedException(
					LanguageCombinationNotSupportedResources.LanguageCombinationNotSupported,
                    inputLanguage, outputLanguage);

			if (translateService == null)
				translateService = new net.webservicex.www.TranslateService();

			string translatedText = 
				translateService.Translate(TranslationServiceLanguage(inputLanguage, outputLanguage), text);

            if (translatedText.StartsWith("The underlying connection was closed"))
                throw new TranslationException("WebServiceX connection was closed");

			return ConvertFromEscapedNumerics(translatedText);
		}
        /// <summary>
        /// TranslationServiceLanguage returns a corresponding net.webservicex.www.Language for a given language pair
        /// </summary>
        /// <param name="inputLanguage">The language of the input text</param>
        /// <param name="outputLanguage">The language of the output text</param>
        /// <returns>A net.webservicex.www.Language enumeration</returns>
		protected static net.webservicex.www.Language TranslationServiceLanguage(string inputLanguage, string outputLanguage)
		{
			inputLanguage = inputLanguage.ToUpper();
			outputLanguage = outputLanguage.ToUpper();
			if (inputLanguage == "EN")
			{
				if (outputLanguage == "ZH-CHS")
					return net.webservicex.www.Language.EnglishTOChinese;
				else if (outputLanguage == "FR")
					return net.webservicex.www.Language.EnglishTOFrench;
				else if (outputLanguage == "DE")
					return net.webservicex.www.Language.EnglishTOGerman;
				else if (outputLanguage == "IT")
					return net.webservicex.www.Language.EnglishTOItalian;
				else if (outputLanguage == "JA")
					return net.webservicex.www.Language.EnglishTOJapanese;
				else if (outputLanguage == "KO")
					return net.webservicex.www.Language.EnglishTOKorean;
				else if (outputLanguage == "PT")
					return net.webservicex.www.Language.EnglishTOPortuguese;
				else if (outputLanguage == "ES")
					return net.webservicex.www.Language.EnglishTOSpanish;
			}
			else if (inputLanguage == "FR")
			{
				if (outputLanguage == "EN")
					return net.webservicex.www.Language.FrenchTOEnglish;
				else if (outputLanguage == "DE")
					return net.webservicex.www.Language.FrenchTOGerman;
			}
			else if (inputLanguage == "DE")
			{
				if (outputLanguage == "EN")
					return net.webservicex.www.Language.GermanTOEnglish;
				else if (outputLanguage == "FR")
					return net.webservicex.www.Language.GermanTOFrench;
			}
			else if (inputLanguage == "IT")
			{
				if (outputLanguage == "EN")
					return net.webservicex.www.Language.ItalianTOEnglish;
			}
			else if (inputLanguage == "ES")
			{
				if (outputLanguage == "EN")
					return net.webservicex.www.Language.SpanishTOEnglish;
			}
			return (net.webservicex.www.Language) (-1);
		}
	}
}
