// File           : IntegrityCheck.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Globalization;
using System.Collections;
using System.Reflection;
using Internationalization.Common;
using Internationalization.ResourceAdministration;

namespace Internationalization.ResourceAdministrator
{
	/// <summary>
	/// IntegrityCheck performs an integrity check of resources
	/// </summary>
	public class IntegrityCheck
	{
        /// <summary>
        /// Constructs an IntegrityCheck object
        /// </summary>
		public IntegrityCheck()
		{
		}
        /// <summary>
        /// Fix fixes problems found in resources
        /// </summary>
        /// <param name="setOfResourceSetsCollection">The SetOfResourceSetsCollection of the resources to fix</param>
        /// <param name="baseNames">The base names of the resources to fix</param>
        /// <param name="eventHandler">The IntegrityCheckInformationEventHandler to
        /// call when resources are fixed</param>
        /// <param name="translate">Specifies whether new resource entries should be automatically translated</param>
        /// <param name="translationHelper">The ITranslationHelper used to fix integrity problems</param>
        /// <param name="inputLanguage">The language of the invariant resources</param>
		public static void Fix(SetOfResourceSetsCollection setOfResourceSetsCollection, string[] baseNames, 
			IntegrityCheckInformationEventHandler eventHandler, bool translate, ITranslationHelper translationHelper, 
			string inputLanguage)
		{
			Perform(setOfResourceSetsCollection, baseNames, eventHandler, true, translate, translationHelper, inputLanguage);
		}
        /// <summary>
        /// Check checks the integrity of the resources and reports failures to the
        /// IntegrityCheckInformationEventHandler event handler
        /// </summary>
        /// <param name="setOfResourceSetsCollection">The SetOfResourceSetsCollection of the resources to check</param>
        /// <param name="baseNames">The base names of the resources to check</param>
        /// <param name="eventHandler">The IntegrityCheckInformationEventHandler to
        /// call when resources fail the integrity check</param>
		public static void Check(SetOfResourceSetsCollection setOfResourceSetsCollection, string[] baseNames, 
			IntegrityCheckInformationEventHandler eventHandler)
		{
			Perform(setOfResourceSetsCollection, baseNames, eventHandler, false, false, null, null);
		}
        /// <summary>
        /// Perform performs the work of checking or fixing the resources
        /// </summary>
        /// <param name="setOfResourceSetsCollection">The SetOfResourceSetsCollection of the resources to
        /// check/fix</param>
        /// <param name="baseNames">The base names of the resources to check/fix</param>
        /// <param name="eventHandler">The IntegrityCheckInformationEventHandler to
        /// call when resources fail the integrity check or need to be fixed</param>
        /// <param name="fixProblems">True if the resources should be fixed. False if the resources should be
        /// checked.</param>
        /// <param name="translate">Specifies whether new resource entries should be automatically translated</param>
        /// <param name="translationHelper">The ITranslationHelper used to fix integrity problems</param>
        /// <param name="inputLanguage">The language of the invariant resources</param>
        public static void Perform(SetOfResourceSetsCollection setOfResourceSetsCollection, string[] baseNames, 
			IntegrityCheckInformationEventHandler eventHandler, bool fixProblems, bool translate, 
			ITranslationHelper translationHelper, string inputLanguage)
		{
			if (eventHandler != null)
			{
				foreach(string baseName in baseNames)
				{
					SetOfResourceSets setOfResourceSets = setOfResourceSetsCollection.GetSetOfResourceSets(baseName);
					ResourceSet invariantResourceSet = setOfResourceSets.GetResourceSet(CultureInfo.InvariantCulture);
					Hashtable invariantResourceTable = ResourceUtilities.GetResourceSetTable(invariantResourceSet);
					foreach(CultureInfo targetLanguage in TargetLanguages.Languages)
					{
						IResourceGovernor resourceGovernor = setOfResourceSets.GetResourceGovernor(targetLanguage);
						if (! resourceGovernor.ResourceExists())
						{
							if (fixProblems)
							{
								if (ResourceAdministrationHelper.IsWindowsForm(invariantResourceTable))
									// write only those invariant resource entries which should have a translation
									resourceGovernor.WriteResourceSet(
										ResourceAdministrationHelper.CreateCultureNeutralWindowsFormResourceSet(invariantResourceTable));
								else
									// write a copy of the invariant resources to the target language
									resourceGovernor.WriteResourceSet(invariantResourceSet);

								if (translate && translationHelper != null)
									translationHelper.TranslateResourceSet(resourceGovernor);

								eventHandler(null, new IntegrityCheckInformationEventArgs(baseName, targetLanguage, "Resource created"));
							}
							else
								eventHandler(null, new IntegrityCheckInformationEventArgs(baseName, targetLanguage, "Resource does not exist"));
						}
						else
						{
							// compare the resources in the target language with the resources in the invariant
							CompareResources(invariantResourceSet, resourceGovernor, eventHandler, 
								fixProblems, translate, translationHelper, inputLanguage);
						}
					}
				}
			}
		}
        /// <summary>
        /// CompareResources compares resources in the invariant language with resources in the target language
        /// </summary>
        /// <param name="invariantResourceSet">The ResourceSet of invariant resource entries</param>
        /// <param name="resourceGovernor">The IResourceGovernor of the target resources</param>
        /// <param name="eventHandler">The IntegrityCheckInformationEventHandler to
        /// call when resources fail the integrity check or need to be fixed</param>
        /// <param name="fixProblems">True if the resources should be fixed. False if the resources should be
        /// checked.</param>
        /// <param name="translate">Specifies whether new resource entries should be automatically translated</param>
        /// <param name="translationHelper">The ITranslationHelper used to fix integrity problems</param>
        /// <param name="inputLanguage">The language of the invariant resources</param>
		public static void CompareResources(ResourceSet invariantResourceSet, IResourceGovernor resourceGovernor, 
			IntegrityCheckInformationEventHandler eventHandler, bool fixProblems, bool translate, 
            ITranslationHelper translationHelper, string inputLanguage)
		{
			bool writeRequired = false;
			ResourceSet targetResourceSet = resourceGovernor.ReadResourceSet();
			Hashtable invariantTable = GetResourceSetTable(invariantResourceSet);
			Hashtable targetTable    = GetResourceSetTable(targetResourceSet);
			foreach(DictionaryEntry dictionaryEntry in invariantTable)
			{
                ResourcesDataNodeInterpreter dataNodeInterpreter = new ResourcesDataNodeInterpreter(
                    dictionaryEntry.Key.ToString(), dictionaryEntry.Value);
				if (! targetTable.ContainsKey(dataNodeInterpreter.Name) &&
                    ResourceUtilities.IsLocalizable(dataNodeInterpreter.Name, dataNodeInterpreter.Value) && 
					dataNodeInterpreter.Value != null && dataNodeInterpreter.Value is string)
				{
					if (fixProblems)
					{
						string text;
						if (translate && translationHelper != null)
						{
							text = translationHelper.Translate(
                                inputLanguage, resourceGovernor.CultureInfo, dataNodeInterpreter.Value.ToString());
						}
						else
						{
							text = dataNodeInterpreter.Value.ToString();
						}
						targetTable.Add(dataNodeInterpreter.Name, text);
						writeRequired = true;

						if (eventHandler != null)
							eventHandler(null, new IntegrityCheckInformationEventArgs(
								resourceGovernor.BaseName, resourceGovernor.CultureInfo, 
								String.Format(IntegrityCheckResources.KeyAddedToTargetLanguage, dataNodeInterpreter.Name)));
					}
					else if (eventHandler != null)
						eventHandler(null, new IntegrityCheckInformationEventArgs(
							resourceGovernor.BaseName, resourceGovernor.CultureInfo, 
							String.Format(IntegrityCheckResources.KeyDoesNotExist, dataNodeInterpreter.Name)));
				}
			}
			if (writeRequired)
				resourceGovernor.WriteResourceSet(targetResourceSet);
		}
		private static Hashtable GetResourceSetTable(ResourceSet resourceSet)
		{
			FieldInfo fieldInfo = 
				resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
			if (fieldInfo != null)
			{
				object tableObject = fieldInfo.GetValue(resourceSet);
				if (tableObject != null)
					return (Hashtable) tableObject;
			}
			return null;
		}
	}
    /// <summary>
    /// IntegrityCheckInformationEventArgs provides information about an integrity check failure or fix
    /// </summary>
	public class IntegrityCheckInformationEventArgs: EventArgs
	{
		private string baseName;
		private string information;
		private CultureInfo cultureInfo;

        /// <summary>
        /// Constructs a IntegrityCheckInformationEventArgs object
        /// </summary>
        /// <param name="baseName">The base name of the resources</param>
        /// <param name="cultureInfo">The CultureInfo of the resources</param>
        /// <param name="information">Information about the failure or fix</param>
		public IntegrityCheckInformationEventArgs(string baseName, CultureInfo cultureInfo, string information): base()
		{
			this.baseName = baseName;
			this.cultureInfo = cultureInfo;
			this.information = information;
		}
        /// <summary>
        /// BaseName is base name of the resources
        /// </summary>
		public string BaseName
		{
			get {return baseName;}
		}
        /// <summary>
        /// CultureInfo is the CultureInfo of the resources
        /// </summary>
		public CultureInfo CultureInfo
		{
			get {return cultureInfo;}
		}
        /// <summary>
        /// Information is information about the failure or fix
        /// </summary>
		public string Information
		{
			get {return information;}
		}
	}
    /// <summary>
    /// IntegrityCheckInformationEventHandler is an event handler used when the integrity check reports a
    /// failure or a fix
    /// </summary>
    /// <param name="sender">The source of the event</param>
    /// <param name="eventArgs">The IntegrityCheckInformationEventArgs event args which provide information
    /// about the failure or fix</param>
	public delegate void IntegrityCheckInformationEventHandler(object sender, IntegrityCheckInformationEventArgs eventArgs);

    /// <summary>
    /// ITranslationHelper is an interface which provides some translation functionality to the integrity check
    /// </summary>
	public interface ITranslationHelper
	{
        /// <summary>
        /// TranslateResourceSet translates a resource set
        /// </summary>
        /// <param name="resourceGovernor">The IResourceGovernor of the resource set</param>
		void TranslateResourceSet(IResourceGovernor resourceGovernor);
        /// <summary>
        /// Translate translates a single resource string
        /// </summary>
        /// <param name="inputLanguage">The language of the resource string</param>
        /// <param name="targetCultureInfo">The CultureInfo of the target language</param>
        /// <param name="text">The string to translate</param>
        /// <returns>The translated string</returns>
		string Translate(string inputLanguage, CultureInfo targetCultureInfo, string text);
	}
}
