// File           : IntegrityCheckForm.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace Internationalization.ResourceAdministrator
{
	/// <summary>
	/// IntegrityCheckForm provides an integrity check of resources and shows the results to the user
	/// </summary>
	public class IntegrityCheckForm : System.Windows.Forms.Form
	{
		private SetOfResourceSetsCollection setOfResourceSetsCollection;
		private string[] baseNames;
        private IntegrityCheckInformationEventHandler integrityCheckInformationEventHandler;
		private bool translate;
		private string inputLanguage;

		private ITranslationHelper translationHelper;
		private bool problemsFixed = false;
		private DataTable dataTable;

        private System.Windows.Forms.Panel panelButtons;
		private System.Windows.Forms.Button buttonClose;
		private System.Windows.Forms.Button buttonFixProblems;
        private DataGridView dataGridViewInformation;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        /// <summary>
        /// Constructs a IntegrityCheckForm object
        /// </summary>
        /// <param name="setOfResourceSetsCollection">The SetOfResourceSetsCollection of the resources to check</param>
        /// <param name="baseNames">The base names of the resources to check</param>
        /// <param name="integrityCheckInformationEventHandler">The IntegrityCheckInformationEventHandler to
        /// call when resources fail the integrity check</param>
        /// <param name="translationHelper">The ITranslationHelper used to fix integrity problems</param>
        /// <param name="translate">Specifies whether new resource entries should be automatically translated</param>
        /// <param name="inputLanguage">The language of the invariant resources</param>
		public IntegrityCheckForm(
			SetOfResourceSetsCollection setOfResourceSetsCollection, string[] baseNames, 
			IntegrityCheckInformationEventHandler integrityCheckInformationEventHandler,
			ITranslationHelper translationHelper, bool translate, string inputLanguage)
		{
			this.setOfResourceSetsCollection = setOfResourceSetsCollection;
			this.baseNames = baseNames;
			this.integrityCheckInformationEventHandler = integrityCheckInformationEventHandler;
			this.translationHelper = translationHelper;
			this.translate = translate;
			this.inputLanguage = inputLanguage;
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			InitializeDataTable();
            dataGridViewInformation.DataSource = dataTable.DataSet;
            dataGridViewInformation.DataMember = dataTable.TableName;
            dataGridViewInformation.Columns["Base Name"  ].Width = 100;
            dataGridViewInformation.Columns["Culture"    ].Width = 50;
            dataGridViewInformation.Columns["Information"].Width = 330;
			panelButtons.Visible = false;
		}
		private void InitializeDataTable()
		{
			dataTable = new DataTable("Information");
			dataTable.Columns.Add("Base Name");
			dataTable.Columns.Add("Culture");
			dataTable.Columns.Add("Information");
			DataSet dataSet = new DataSet("Information");
			dataSet.Tables.Add(dataTable);
		}
        /// <summary>
        /// AddInformation adds information about an integrity check failure
        /// </summary>
        /// <param name="eventArgs">An IntegrityCheckInformationEventArgs providing the information about the
        /// failure</param>
		public virtual void AddInformation(IntegrityCheckInformationEventArgs eventArgs)
		{
			string tab = ((char) Keys.Tab).ToString();

			DataRow dataRow = dataTable.NewRow();
			dataRow["Base Name"  ] = eventArgs.BaseName;
			dataRow["Culture"    ] = eventArgs.CultureInfo.Name;
			dataRow["Information"] = eventArgs.Information;
			dataTable.Rows.Add(dataRow);

			Application.DoEvents();
		}
        /// <summary>
        /// InformationCount is the number of integrity failures
        /// </summary>
		public int InformationCount
		{
			get {return dataTable.Rows.Count;}
		}
        /// <summary>
        /// ShowButtons shows the integrity check buttons
        /// </summary>
		public void ShowButtons()
		{
			panelButtons.Visible = true;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(IntegrityCheckForm));
            this.panelButtons = new System.Windows.Forms.Panel();
            this.buttonFixProblems = new System.Windows.Forms.Button();
            this.buttonClose = new System.Windows.Forms.Button();
            this.dataGridViewInformation = new System.Windows.Forms.DataGridView();
            this.panelButtons.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.dataGridViewInformation)).BeginInit();
            this.SuspendLayout();
            // 
            // panelButtons
            // 
            this.panelButtons.Controls.Add(this.buttonFixProblems);
            this.panelButtons.Controls.Add(this.buttonClose);
            resources.ApplyResources(this.panelButtons, "panelButtons");
            this.panelButtons.Name = "panelButtons";
            // 
            // buttonFixProblems
            // 
            resources.ApplyResources(this.buttonFixProblems, "buttonFixProblems");
            this.buttonFixProblems.Name = "buttonFixProblems";
            this.buttonFixProblems.Click += new System.EventHandler(this.buttonFixProblems_Click);
            // 
            // buttonClose
            // 
            this.buttonClose.DialogResult = System.Windows.Forms.DialogResult.OK;
            resources.ApplyResources(this.buttonClose, "buttonClose");
            this.buttonClose.Name = "buttonClose";
            // 
            // dataGridViewInformation
            // 
            this.dataGridViewInformation.AllowUserToAddRows = false;
            this.dataGridViewInformation.AllowUserToDeleteRows = false;
            this.dataGridViewInformation.ColumnHeadersHeightSizeMode = System.Windows.Forms.DataGridViewColumnHeadersHeightSizeMode.AutoSize;
            resources.ApplyResources(this.dataGridViewInformation, "dataGridViewInformation");
            this.dataGridViewInformation.Name = "dataGridViewInformation";
            this.dataGridViewInformation.ReadOnly = true;
            // 
            // IntegrityCheckForm
            // 
            this.AcceptButton = this.buttonClose;
            resources.ApplyResources(this, "$this");
            this.Controls.Add(this.dataGridViewInformation);
            this.Controls.Add(this.panelButtons);
            this.Name = "IntegrityCheckForm";
            this.panelButtons.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.dataGridViewInformation)).EndInit();
            this.ResumeLayout(false);

		}
		#endregion

		private void buttonFixProblems_Click(object sender, System.EventArgs e)
		{
			if (dataTable.Rows.Count == 0)
				MessageBox.Show(IntegrityCheckFormResources.ThereAreNoProblemsToFix, IntegrityCheckFormResources.IntegrityCheck);
			else if (problemsFixed)
                MessageBox.Show(IntegrityCheckFormResources.TheProblemsHaveAlreadyBeenFixed, IntegrityCheckFormResources.IntegrityCheck);
			else if (MessageBox.Show(IntegrityCheckFormResources.FixTheProblemsShownHere + System.Environment.NewLine + 
				System.Environment.NewLine + IntegrityCheckFormResources.EnsureThatYouHaveWriteAccess,
				IntegrityCheckFormResources.IntegrityCheck, MessageBoxButtons.YesNo) == DialogResult.Yes)
			{
				Cursor.Current = Cursors.WaitCursor;
				try
				{
					InitializeDataTable();

					IntegrityCheck.Fix(setOfResourceSetsCollection, baseNames, 
						integrityCheckInformationEventHandler, translate, translationHelper, inputLanguage);

					problemsFixed = true;
					MessageBox.Show(
						IntegrityCheckFormResources.TheProblemsListedHaveBeenFixed + System.Environment.NewLine + System.Environment.NewLine +
						IntegrityCheckFormResources.AddResourceFilesToVisualStudioProject,
						IntegrityCheckFormResources.IntegrityCheck);
				}
				catch(Exception exception)
				{
					MessageBox.Show(String.Format(IntegrityCheckFormResources.AnErrorOccurred, exception.Message));
				}
				finally
				{
					Cursor.Current = Cursors.Default;
				}
			}
		}
	}
}
