// File           : SetOfResourceSetsCollection.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Globalization;
using Internationalization.Common;
using Internationalization.ResourceAdministration;

namespace Internationalization.ResourceAdministrator
{
	/// <summary>
	/// SetOfResourceSetsCollection is a collection of SetOfResourceSets objects
	/// </summary>
    /// <remarks>A SetOfResourceSets is a set of ResourceSet objects for the same base name. So Form1.resx,
    /// Form1.fr.resx, Form1.fr-FR.resx and Form1.de.resx make up a SetOfResourceSets for "Form1".
    /// A SetOfResourceSetsCollection is the collection of SetOfResourceSets for all base names.</remarks>
	public class SetOfResourceSetsCollection
	{
		private Hashtable setOfResourceSetsCollection = new Hashtable();
		private IResourcesGovernor resourcesGovernor;

        /// <summary>
        /// Constructs a SetOfResourceSetsCollection object
        /// </summary>
        /// <param name="resourcesGovernor">The IResourcesGovernor which retrieves resources from the source</param>
		public SetOfResourceSetsCollection(IResourcesGovernor resourcesGovernor)
		{
			this.resourcesGovernor = resourcesGovernor;
		}
        /// <summary>
        /// The SetOfResourceSetsCollection indexer which gets a SetOfResourceSets for a given base name
        /// </summary>
        /// <param name="baseName">The base name to get the SetOfResourceSets for</param>
        /// <returns>The SetOfResourceSets for a given base name</returns>
		public SetOfResourceSets this[string baseName]
		{
			get {return (SetOfResourceSets) setOfResourceSetsCollection[baseName];}
		}
        /// <summary>
        /// GetSetOfResourceSets gets a SetOfResourceSets for a given base name
        /// </summary>
        /// <param name="baseName">The base name of the resources</param>
        /// <returns>The SetOfResourceSets for a given base name</returns>
        /// <remarks>GetSetOfResourceSets gets the SetOfResourceSets either by loading from a cache (if it is in
        /// the cache or by retrieving a new SetOfResourceSets and caching it</remarks>
		public virtual SetOfResourceSets GetSetOfResourceSets(string baseName)
		{
			// see if the set of ResourceSets is already in the cache
			SetOfResourceSets setOfResourceSets = (SetOfResourceSets) setOfResourceSetsCollection[baseName];
			if (setOfResourceSets == null)
			{
				// it isn't in the cache so get a new SetOfResourceSets and add it to the cache
				setOfResourceSets = CreateSetOfResourceSets(baseName);
				setOfResourceSetsCollection.Add(baseName, setOfResourceSets);
			}
			return setOfResourceSets;
		}
        /// <summary>
        /// CreateSetOfResourceSets creates a SetOfResourceSets for a given base name
        /// </summary>
        /// <param name="baseName">The base name to create the SetOfResourceSets for</param>
        /// <returns>The SetOfResourceSets for a given base name</returns>
        /// <remarks>The SetOfResourceSets includes entries for an invariant ResourceSet and each
        /// CultureInfo in TargetLanguages.Languages</remarks>
		public virtual SetOfResourceSets CreateSetOfResourceSets(string baseName)
		{
			SetOfResourceSets setOfResourceSets = new SetOfResourceSets(baseName);

			// add the ResourceSet for the invariant culture
			AddResourceSet(setOfResourceSets, baseName, CultureInfo.InvariantCulture);
			// add the ResourceSets for all of the target languages
			foreach(CultureInfo cultureInfo in TargetLanguages.Languages)
			{
				AddResourceSet(setOfResourceSets, baseName, cultureInfo);
			}
			return setOfResourceSets;
		}
		private void AddResourceSet(SetOfResourceSets setOfResourceSets, string baseName, CultureInfo cultureInfo)
		{
			IResourceGovernor resourceGovernor = resourcesGovernor.GetResourceGovernor(baseName, cultureInfo);
			if (resourceGovernor.ResourceExists())
				setOfResourceSets.AddResourceSet(resourceGovernor);
			else
				setOfResourceSets.AddResourceSet(resourceGovernor, null);
		}
	}
}
