// File           : DbResourcesGovernor.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Data.SqlClient;
using System.Collections;
using System.Collections.Specialized;
using System.Resources;
using Internationalization.Resources;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// DbResourcesGovernor manages a complete set of database based resources
    /// </summary>
    /// <remarks>Take care to notice the distinction between two similarly named classes:
    /// DbResourcesGovernor and DbResourceGovernor. The former managers a set of resources whereas the latter manages
    /// a single resource. Further note the potential terminology confusion between a set of resources, a resource
    /// and a resource entry. A set of resources would be, for example, the complete set of base name/culture
    /// combinations. A resource would be a single base name/culture combination and a resource entry would be
    /// one entry within a single base name/culture combination.</remarks>
    public class DbResourcesGovernor : ResourcesGovernor, IResourcesGovernor
	{
        private string connectionString;

        /// <summary>
        /// Constructs a DbResourcesGovernor
        /// </summary>
        /// <param name="connectionString">The connection string to connect to the database</param>
		public DbResourcesGovernor(string connectionString)
		{
			this.connectionString = connectionString;
		}
        /// <summary>
        /// Constructs a DbResourcesGovernor
        /// </summary>
        public DbResourcesGovernor()
		{
			this.connectionString = DbResourceManager.ConnectionString;
		}
        /// <summary>
        /// GetInvariantCultureBaseNames gets an array of invariant culture base names in the database
        /// </summary>
        /// <returns>An array of invariant culture base names in the database</returns>
        /// <remarks>Assuming that the database contains resources for Form1 (Invariant), Form1 (French),
        /// Form2 (Invariant), Form2 (French) and Form2 (French (France)) this method returns {"Form1", "Form2"}.</remarks>
        public string[] GetInvariantCultureBaseNames()
		{
			StringCollection baseNames = new StringCollection();
			SqlConnection connection = new SqlConnection(connectionString);
			try
			{
				connection.Open();
				SqlCommand command = new SqlCommand(
					"SELECT DISTINCT ResourceSetName FROM ResourceSets WHERE Culture IS NULL", connection);
				SqlDataReader dataReader = command.ExecuteReader();
				while (dataReader.Read())
				{
					baseNames.Add(dataReader["ResourceSetName"].ToString());
				}
				dataReader.Close();

				// convert the StringCollection to an array
				string[] invariantCultureBaseNames = new string[baseNames.Count];
				baseNames.CopyTo(invariantCultureBaseNames, 0);
				return invariantCultureBaseNames;
			}
			finally
			{
				connection.Close();
			}
		}
        /// <summary>
        /// GetExistingCultures gets an array of CultureInfo objects for resources which exist for the given
        /// base name
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <returns>An array of CultureInfo objects for resources which exist for the given base name</returns>
        /// <remarks>For a database containing Form1 (Invariant), Form1 (French) and Form1 (French (France))
        /// resources this method returns {invariant CultureInfo, French CultureInfo, French (France) CultureInfo}
        /// when the base name given is "Form1".</remarks>
        public CultureInfo[] GetExistingCultures(string baseName)
        {
            ArrayList cultureInfos = new ArrayList();
            CultureInfo[] allCultures = CultureInfo.GetCultures(CultureTypes.AllCultures);
            SqlConnection connection = new SqlConnection(connectionString);
            try
            {
                connection.Open();
                SqlCommand command = new SqlCommand(
                    "SELECT DISTINCT Culture FROM ResourceSets WHERE ResourceSetName=@ResourceSetName", connection);
                command.Parameters.AddWithValue("@ResourceSetName", baseName);
                SqlDataReader dataReader = command.ExecuteReader();
                while (dataReader.Read())
                {
                    string cultureName = dataReader["Culture"].ToString();
                    if (cultureName == null)
                        cultureInfos.Add(CultureInfo.InvariantCulture);
                    else
                    {
                        CultureInfo cultureInfo = GetCultureInfo(allCultures, cultureName);
                        if (cultureInfo != null)
                            cultureInfos.Add(cultureInfo);
                    }
                }
                dataReader.Close();

                return (CultureInfo[]) cultureInfos.ToArray(typeof(CultureInfo));
            }
            finally
            {
                connection.Close();
            }
        }
        /// <summary>
        /// GetCultureInfo provides a fast lookup for a CultureInfo object by culture name
        /// </summary>
        /// <param name="allCultures">An array of CultureInfo objects on which to perform the lookup</param>
        /// <param name="cultureName">The RFC 1766 name of the culture</param>
        /// <returns>A CultureInfo for the given name or null if the name is not found</returns>
        protected virtual CultureInfo GetCultureInfo(CultureInfo[] allCultures, string cultureName)
        {
            foreach(CultureInfo cultureInfo in allCultures)
            {
                if (cultureName == cultureInfo.Name)
                    return cultureInfo;
            }
            return null;
        }
        /// <summary>
        /// GetResourceGovernor gets an IResourceGovernor for the given base name and culture
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="cultureInfo">The CultureInfo of the resource</param>
        /// <returns>An IResourceGovernor for the given base name and culture</returns>
        public IResourceGovernor GetResourceGovernor(string baseName, CultureInfo cultureInfo)
		{
			return new DbResourceGovernor(connectionString, baseName, cultureInfo);
		}
        /// <summary>
        /// IsFileBased returns false because the resources are not file based
        /// </summary>
        public bool IsFileBased
        {
            get {return false;}
        }
        /// <summary>
        /// IsDatabaseBased returns true because the resources are database based
        /// </summary>
        public bool IsDatabaseBased
        {
            get {return true;}
        }
        /// <summary>
        /// GetDatabaseName gets the database name from a connection string
        /// </summary>
        /// <param name="connectionString">A connection string containing a database name</param>
        /// <returns>The database name contained within the connection string</returns>
        protected static string GetDatabaseName(string connectionString)
        {
            string databaseName = GetParameter("DATABASE", connectionString);
            if (databaseName == null)
            {
                databaseName = GetParameter("INITIAL CATALOG", connectionString);
                if (databaseName == null)
                    return String.Empty;
            }
            return databaseName;
        }
        /// <summary>
        /// GetParameter gets the value of a named parameter from a connection string
        /// </summary>
        /// <param name="parameterName">The name of the parameter in the connection string</param>
        /// <param name="connectionString">A connection string containing the parameter name</param>
        /// <returns>The value of the named parameter from the connection string</returns>
        protected static string GetParameter(string parameterName, string connectionString)
        {
            int equalsLength = 1;
            int index = connectionString.ToUpper().IndexOf(parameterName + "=");
            if (index == -1)
            {
                index = connectionString.ToUpper().IndexOf(parameterName + " =");
                equalsLength = 2;
            }
            if (index > -1)
            {
                string parameterValue = connectionString.Substring(index + parameterName.Length + equalsLength);
                int terminatorIndex = parameterValue.IndexOf(";");
                if (terminatorIndex > -1)
                    parameterValue = parameterValue.Substring(0, terminatorIndex);
                return parameterValue.Trim();
            }
            return null;
        }
        /// <summary>
        /// RemoveDatabaseName removes the database parameter from a connection string
        /// </summary>
        /// <param name="connectionString">A connection string containing a database name</param>
        /// <returns>The connection string without the database parameter (and value)</returns>
        protected static string RemoveDatabaseName(string connectionString)
        {
            connectionString = RemoveParameter("DATABASE", connectionString);
            connectionString = RemoveParameter("INITIAL CATALOG", connectionString);
            return connectionString;
        }
        /// <summary>
        /// RemoveDatabaseName removes a given parameter from a connection string
        /// </summary>
        /// <param name="parameterName">The name of the parameter in the connection string</param>
        /// <param name="connectionString">A connection string containing the parameter name</param>
        /// <returns>The connection string without the parameter (and value)</returns>
        protected static string RemoveParameter(string parameterName, string connectionString)
        {
            int index = connectionString.ToUpper().IndexOf(parameterName + "=");
            if (index == -1)
                index = connectionString.ToUpper().IndexOf(parameterName + " =");
            if (index > -1)
            {
                int terminatorIndex = connectionString.IndexOf(";", index);
                if (terminatorIndex > -1)
                    connectionString = connectionString.Substring(0, index).TrimEnd() +
                        connectionString.Substring(terminatorIndex + 1).TrimEnd();
                else
                    connectionString = connectionString.Substring(0, index).TrimEnd();

            }
            return connectionString;
        }
        /// <summary>
        /// DatabaseExists returns true if the database in the connection string exists
        /// </summary>
        /// <param name="connectionString">The connection string to connect to the database</param>
        /// <returns>True if the database in the connection string exists</returns>
        protected static bool DatabaseExists(string connectionString)
        {
            return DatabaseExists(connectionString, GetDatabaseName(connectionString));
        }
        /// <summary>
        /// DatabaseExists returns true if the given database exists
        /// </summary>
        /// <param name="connectionString">The connection string to connect to the database</param>
        /// <param name="databaseName">The name of the database to test for existence</param>
        /// <returns>True if the given database exists</returns>
        protected static bool DatabaseExists(string connectionString, string databaseName)
        {
            if (databaseName == null || databaseName == String.Empty)
                return false;

            string query = 
                "IF EXISTS (SELECT name FROM master.dbo.sysdatabases WHERE name = @DatabaseName) " +
                "   SELECT 1; " + 
                "ELSE " +
                "   SELECT 0; ";

            connectionString = RemoveDatabaseName(connectionString);
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                connection.Open();
                SqlCommand command = new SqlCommand(query, connection);
                command.Parameters.AddWithValue("@DatabaseName", databaseName);
                object result = command.ExecuteScalar();
                connection.Close();
                return result.ToString() == "1";
            }
        }
        /// <summary>
        /// Exists returns true if the database exists
        /// </summary>
        public bool Exists
        {
            get {return DatabaseExists(connectionString);}
        }
        /// <summary>
        /// Create creates the database (and resource table)
        /// </summary>
        public void Create()
        {
            // create the database
            string databaseName = GetDatabaseName(connectionString);
            string noDatabaseConnectionString = RemoveDatabaseName(connectionString);
            using (SqlConnection connection = new SqlConnection(noDatabaseConnectionString))
            {
                connection.Open();

                string path = DatabasePath;
                if (! path.EndsWith(@"\"))
                    path += @"\";

                string dataFileName = path + databaseName + "_data.mdf";
                string logFileName = path + databaseName + "_log.ldf";
                string logName = databaseName + "_log";
                string createDatabaseCommandString =
                    "CREATE DATABASE [" + databaseName + "] ON PRIMARY" +
                    "(Name=" + databaseName +", filename = '" + dataFileName + "')log on" +
                    "(Name=" + logName +", filename='" + logFileName + "')";

                SqlCommand createDatabaseCommand = new SqlCommand(createDatabaseCommandString, connection);

                createDatabaseCommand.ExecuteNonQuery();

                connection.Close();
            }

            // create the table
            using (SqlConnection connection = new SqlConnection(connectionString))
            {
                connection.Open();

                string createTableCommandString = @"
					CREATE TABLE [ResourceSets] (
					[ResourceID] [int] IDENTITY (1, 1) NOT NULL ,
					[ResourceSetName] [nvarchar] (50) NOT NULL ,
					[Culture] [nvarchar] (10) NULL ,
					[ResourceName] [nvarchar] (100) NOT NULL ,
					[ResourceValue] [nvarchar] (200) NOT NULL ,
					[ResourceType] [nvarchar] (250),
					CONSTRAINT [PK_ResourceSets] PRIMARY KEY  CLUSTERED 
					(
					[ResourceID]
					)  ON [PRIMARY] 
					) ON [PRIMARY]";

                SqlCommand createTableCommand = new SqlCommand(createTableCommandString, connection);

                createTableCommand.ExecuteNonQuery();

                connection.Close();
            }
        }
        private static string databasePath = @"C:\Program Files\Microsoft SQL Server\MSSQL\Data";
        /// <summary>
        /// DatabasePath is the path to the database
        /// </summary>
        public static string DatabasePath
        {
            get {return databasePath;}
            set {databasePath = value;}
        }
    }

    /// <summary>
    /// DbResourceGovernor manages a database based resource
    /// </summary>
    public class DbResourceGovernor : ResourceGovernor, IResourceGovernor
	{
		private string connectionString;

        /// <summary>
        /// Constructs a DbResourceGovernor object
        /// </summary>
        /// <param name="connectionString">The connection string to connect to the database</param>
        /// <param name="baseName">The base name of the resource (e.g. "Form1")</param>
        /// <param name="cultureInfo">The CultureInfo which represents the culture of the resource</param>
        public DbResourceGovernor(string connectionString, string baseName, CultureInfo cultureInfo)
		{
			Initialize(connectionString, baseName, cultureInfo);
		}
        /// <summary>
        /// Initialize initializes a DbResourceGovernor object
        /// </summary>
        /// <param name="connectionString">The connection string to connect to the database</param>
        /// <param name="baseName">The base name of the resource (e.g. "Form1")</param>
        /// <param name="cultureInfo">The CultureInfo which represents the culture of the resource</param>
        protected void Initialize(string connectionString, string baseName, CultureInfo cultureInfo)
		{
			this.connectionString = connectionString;
			base.Initialize(baseName, cultureInfo);
		}
        /// <summary>
        /// ReadResourceSet reads the resource entries from the database into a ResourceSet
        /// </summary>
        /// <returns>A ResourceSet containing the resource entries read from the database</returns>
        public override ResourceSet ReadResourceSet()
		{
			DbResourceManager.ConnectionString = connectionString;
			return ReadResourceSet(new DbResourceReader(BaseName, CultureInfo));
		}
        /// <summary>
        /// WriteResourceSet writes resource entries in a ResourceSet back to the database
        /// </summary>
        /// <param name="resourceSet">The ResourceSet containing the resource entries to write</param>
        public virtual void WriteResourceSet(ResourceSet resourceSet)
		{
			DbResourceManager.ConnectionString = connectionString;
			WriteResourceSet(new DbResourceWriter(BaseName, CultureInfo), resourceSet);
		}
        /// <summary>
        /// ResourceExists returns true if the resource exists
        /// </summary>
        /// <returns>This method always returns true for database based resources</returns>
        /// <remarks>As all resources are stored in a table the concept of a resource existing
        /// or not is not quite as clear as it is for a file-based resource</remarks>
		public bool ResourceExists()
		{
			return true;
		}
        /// <summary>
        /// ResourceIsReadOnly returns true if the resource is read-only
        /// </summary>
        /// <returns>True if the resource is read-only</returns>
        public bool ResourceIsReadOnly()
		{
			return false;
		}
	}
}
