// File           : FileBasedResourcesGovernor.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Reflection;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// FileBasedResourcesGovernor is an abstract base class for managing a complete set of file based resources
    /// </summary>
    /// <remarks>Take care to notice the distinction between two similarly named classes:
    /// FileBasedResourcesGovernor and FileBasedResourceGovernor. The former managers a set of resources whereas the
    /// latter manages a single resource. Further note the potential terminology confusion between a set of
    /// resources, a resource and a resource entry. A set of resources would be, for example, the complete set of
    /// resx files in a folder. A resource would be a single resx file and a resource entry would be one entry
    /// within a single resx file.</remarks>
    public abstract class FileBasedResourcesGovernor : ResourcesGovernor
    {
        private string extension;
        private string path;

        /// <summary>
        /// Constructs a FileBasedResourcesGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        /// <param name="extension">The extension used by the resource files without the leading period
        /// (e.g. "resx", "resources")</param>
        protected FileBasedResourcesGovernor(string path, string extension)
        {
            Initialize(path, extension);
        }
        /// <summary>
        /// Initialize initailizes a FileBasedResourcesGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        /// <param name="extension">The extension used by the resource files without the leading period
        /// (e.g. "resx", "resources")</param>
        protected virtual void Initialize(string path, string extension)
        {
            this.extension = extension;
            this.path = path;
        }
        /// <summary>
        /// GetInvariantCultureBaseNames gets an array of invariant culture base names in the source of resources
        /// </summary>
        /// <returns>An array of invariant culture base names in the source of resources</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// Form2.resx, Form2.fr.resx and Form2.fr-FR.resx files this method returns {"Form1", "Form2"}.</remarks>
        public string[] GetInvariantCultureBaseNames()
        {
            // get all of the resource files in the specified path
            string[] filenames = Directory.GetFiles(path, "*." + extension);

            // the list contains invariant, culture neutral and culture specific files
            // but we only want invariant files
            CultureInfo[] allCultures = CultureInfo.GetCultures(CultureTypes.AllCultures);
            StringCollection invariantFilenames = new StringCollection();
            foreach(string filename in filenames)
            {
                string filenameWithoutExtension = Path.GetFileNameWithoutExtension(filename);
                int index = filenameWithoutExtension.LastIndexOf(".");
                if (index > -1)
                {
                    // The filename (without the extension) has a dot in it. This is either
                    // because the filename normally has a dot (e.g. "System.Windows.Forms.dll")
                    // or it has a culture name in it (e.g. "Form1.de.resx"). We need to decide
                    // which it is.
                    // "cultureName" will be something like "Forms" or "de" (from these two examples).
                    if (! IsValidCultureName(allCultures, filenameWithoutExtension.Substring(index + 1)))
                        // this is not a culture name so therefore it is an invariant culture file
                        invariantFilenames.Add(filenameWithoutExtension);
                }
                else
                    invariantFilenames.Add(filenameWithoutExtension);
            }

            // convert the StringCollection to an array
            string[] invariantNames = new string[invariantFilenames.Count];
            invariantFilenames.CopyTo(invariantNames, 0);
            return invariantNames;
        }
        /// <summary>
        /// GetExistingCultures gets an array of CultureInfo objects for resources which exist for the given
        /// base name
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <returns>An array of CultureInfo objects for resources which exist for the given base name</returns>
        /// <remarks>For a resx resources governor for a folder containing Form1.resx, Form1.fr.resx,
        /// and Form1.fr-FR.resx files this method returns {invariant CultureInfo, French CultureInfo,
        /// French (France) CultureInfo} when the base name given is "Form1".</remarks>
        public CultureInfo[] GetExistingCultures(string baseName)
        {
            // get all of the resource files for baseName in the specified path
            string[] filenames = Directory.GetFiles(path, baseName + ".*." + extension);

            CultureInfo[] allCultures = CultureInfo.GetCultures(CultureTypes.AllCultures);
            ArrayList cultureInfos = new ArrayList();
            string invariantFileName;
            if (path.EndsWith(Path.DirectorySeparatorChar.ToString()))
                invariantFileName = path + baseName + "." + extension;
            else
                invariantFileName = path + Path.DirectorySeparatorChar + baseName + "." + extension;

            if (File.Exists(invariantFileName))
                cultureInfos.Add(CultureInfo.InvariantCulture);

            foreach(string filename in filenames)
            {
                string filenameWithoutExtension = Path.GetFileNameWithoutExtension(filename);
                int index = filenameWithoutExtension.LastIndexOf(".");
                if (index > -1)
                {
                    // The filename (without the extension) has a dot in it. This is either
                    // because the filename normally has a dot (e.g. "System.Windows.Forms.dll")
                    // or it has a culture name in it (e.g. "Form1.de.resx"). We need to decide
                    // which it is.
                    // "cultureName" will be something like "Forms" or "de" (from these two examples).
                    CultureInfo cultureInfo = GetCultureInfo(allCultures, filenameWithoutExtension.Substring(index + 1));
                    if (cultureInfo != null)
                        cultureInfos.Add(cultureInfo);
                }
            }
            return (CultureInfo[]) cultureInfos.ToArray(typeof(CultureInfo));
        }
        /// <summary>
        /// GetCultureInfo provides a fast lookup for a CultureInfo object by culture name
        /// </summary>
        /// <param name="allCultures">An array of CultureInfo objects on which to perform the lookup</param>
        /// <param name="cultureName">The RFC 1766 name of the culture</param>
        /// <returns>A CultureInfo for the given name or null if the name is not found</returns>
        protected virtual CultureInfo GetCultureInfo(CultureInfo[] allCultures, string cultureName)
        {
            foreach(CultureInfo cultureInfo in allCultures)
            {
                if (cultureName == cultureInfo.Name)
                    return cultureInfo;
            }
            return null;
        }
        /// <summary>
        /// IsValidCultureName returns true if the given culture name is known
        /// </summary>
        /// <param name="allCultures">An array of CultureInfo objects on which to perform the lookup</param>
        /// <param name="cultureName">The RFC 1766 name of the culture</param>
        /// <returns>True if the given culture name is known</returns>
        protected virtual bool IsValidCultureName(CultureInfo[] allCultures, string cultureName)
        {
            foreach(CultureInfo cultureInfo in allCultures)
            {
                if (cultureName == cultureInfo.Name)
                    return true;
            }
            return false;
        }
        /// <summary>
        /// IsFileBased returns true if the resources are file based
        /// </summary>
        public bool IsFileBased
        {
            get {return true;}
        }
        /// <summary>
        /// IsDatabaseBased returns true if the resources are database based
        /// </summary>
        public bool IsDatabaseBased
        {
            get {return false;}
        }
        /// <summary>
        /// Exists returns true if the resource path exists
        /// </summary>
        public bool Exists
        {
            get {return Directory.Exists(path);}
        }
        /// <summary>
        /// Create creates the resource path
        /// </summary>
        public void Create()
        {
            Directory.CreateDirectory(path);
        }
    }

    /// <summary>
    /// FileBasedResourceGovernor is an abstract for managing a file based resource
    /// </summary>
    public abstract class FileBasedResourceGovernor : ResourceGovernor
    {
        private string extension;
        private string path;

        /// <summary>
        /// Constructs a FileBasedResourceGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        /// <param name="baseName">The base name of the resource (e.g. "Form1")</param>
        /// <param name="cultureInfo">The CultureInfo which represents the culture of the resource</param>
        /// <param name="extension">The extension used by the resource files without the leading period
        /// (e.g. "resx", "resources")</param>
        protected FileBasedResourceGovernor(string path, string baseName, CultureInfo cultureInfo, string extension)
        {
            Initialize(path, baseName, cultureInfo, extension);
        }
        /// <summary>
        /// Initialize initializes a FileBasedResourceGovernor object
        /// </summary>
        /// <param name="path">The path to the resource file</param>
        /// <param name="baseName">The base name of the resource (e.g. "Form1")</param>
        /// <param name="cultureInfo">The CultureInfo which represents the culture of the resource</param>
        /// <param name="extension">The extension used by the resource files without the leading period
        /// (e.g. "resx", "resources")</param>
        protected void Initialize(string path, string baseName, CultureInfo cultureInfo, string extension)
        {
            this.extension = extension;
            this.path = path;
            base.Initialize(baseName, cultureInfo);
        }
        /// <summary>
        /// Filename is the fully qualified name of the resource file
        /// </summary>
        /// <remarks>For example if the path is "c:\apps\CRM", the base name is "Form1", the culture is
        /// "fr" and the extension is "resx" then Filename is "c:\apps\CRM\Form1.fr.resx"</remarks>
        protected string Filename
        {
            get
            {
                string fileName;
                if (CultureInfo.Equals(CultureInfo.InvariantCulture))
                    fileName = BaseName + "." + extension;
                else
                    fileName = BaseName + "." + CultureInfo.Name + "." + extension;

                return System.IO.Path.Combine(path, fileName);
            }
        }
        /// <summary>
        /// The path to the resource file
        /// </summary>
        protected string Path
        {
            get { return path; }
        }
        /// <summary>
        /// ResourceExists determines if the resource file exists
        /// </summary>
        /// <returns>True if the resource file exists</returns>
        public bool ResourceExists()
        {
            return File.Exists(Filename);
        }
        /// <summary>
        /// ResourceIsReadOnly returns true if the resource file is read-only
        /// </summary>
        /// <returns>True if the resource file is read-only</returns>
        /// <remarks>ResourceIsReadOnly returns true if the file is marked as read-only. This is often true for
        /// files which are under version control and have not been checked out.</remarks>
        public bool ResourceIsReadOnly()
        {
            FileAttributes attributes = File.GetAttributes(Filename);
            return ((attributes & FileAttributes.ReadOnly) == FileAttributes.ReadOnly);
        }
    }
}
