// File           : ResourcesDataNodeInterpreter.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.ComponentModel.Design;
using System.Reflection;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ResourcesDataNodeInterpreter is an interpreter for resources data nodes
    /// </summary>
    /// <remarks>The problem that this class solves is this: The ResXDataNode class exists to provide a
    /// data node for resx files. The problem is that it exists *only* for resx files. No part of this class
    /// can be reused for other resource sources. A database resource manager or web service resource manager
    /// could support additional data node concepts such as comments but the .NET Framework does not offer any
    /// help in this area. The ResourcesDataNodeInterpreter class exists to offer a uniform way of interpreting
    /// data nodes regardless of their implementation (e.g. ResXDataNode, IResourcesDataNode or raw data).</remarks>
    public class ResourcesDataNodeInterpreter
    {
        private string name;
        private object dataValue;
        /// <summary>
        /// Constructs a ResourcesDataNodeInterpreter object
        /// </summary>
        /// <param name="name">The name of the resource entry</param>
        /// <param name="value">The value of the resource entry</param>
        public ResourcesDataNodeInterpreter(string name, object value)
        {
            this.name = name;
            this.dataValue = value;
        }
        /// <summary>
        /// IsDataNode returns true if the data represented by the ResourcesDataNodeInterpreter is a data node
        /// (as opposed to raw data)
        /// </summary>
        /// <remarks>The data is considered to be a data node if it is either a ResXDataNode or it supports
        /// the IResourcesDataNode interface</remarks>
        public bool IsDataNode
        {
            get { return dataValue is ResXDataNode || dataValue is IResourcesDataNode; }
        }
        /// <summary>
        /// DataNode is the data node represented by the ResourcesDataNodeInterpreter
        /// </summary>
        /// <remarks>Note that DataNode is not necessarily the same as Value. If the data is a data node then
        /// the Value property is the value of the data node and not the data node itself.</remarks>
        public object DataNode
        {
            get {return dataValue;}
        }
        /// <summary>
        /// Comment is the comment associated with the data node (or null if the data is raw data)
        /// </summary>
        public string Comment
        {
            get
            {
                if (dataValue is ResXDataNode)
                    return ((ResXDataNode)dataValue).Comment;
                else if (dataValue is IResourcesDataNode)
                    return ((IResourcesDataNode)dataValue).Comment;
                else
                    return null;
            }
            set
            {
                if (dataValue is ResXDataNode)
                    ((ResXDataNode)dataValue).Comment = value;
                else if (dataValue is IResourcesDataNode)
                    ((IResourcesDataNode)dataValue).Comment = value;
            }
        }
        /// <summary>
        /// Name is the name of the resource entry
        /// </summary>
        /// <remarks>If the data is a data node then setting Name also sets the data node's Name property</remarks>
        public string Name
        {
            get { return name; }
            set
            {
                name = value;
                if (dataValue is ResXDataNode)
                    ((ResXDataNode)dataValue).Name = value;
                else if (dataValue is IResourcesDataNode)
                    ((IResourcesDataNode)dataValue).Name = value;
            }
        }
        /// <summary>
        /// Value is the value of the resource entry
        /// </summary>
        /// <remarks>Note that Value is not necessarily the same as DataNode. If the data is a data node then
        /// the Value property is the value of the data node and not the data node itself.</remarks>
        public object Value
        {
            get
            {
                if (dataValue is ResXDataNode)
                    return ((ResXDataNode)dataValue).GetValue((ITypeResolutionService) null);
                else if (dataValue is IResourcesDataNode)
                    return ((IResourcesDataNode)dataValue).Value;
                else
                    return dataValue;
            }
            set
            {
                if (dataValue is ResXDataNode)
                    SetResXDataNodeValue((ResXDataNode)dataValue, value);
                else if (dataValue is IResourcesDataNode)
                    ((IResourcesDataNode)dataValue).Value = value;
                else
                    dataValue = value;
            }
        }
        /// <summary>
        /// FileRef is the ResourcesFileRefInterpreter of the data node
        /// </summary>
        public ResourcesFileRefInterpreter FileRef
        {
            get
            {
                if (dataValue is ResXDataNode)
                {
                    ResXFileRef fileRef = ((ResXDataNode)dataValue).FileRef;
                    if (fileRef == null)
                        return null;
                    else
                        return new ResourcesFileRefInterpreter(fileRef);
                }
                else if (dataValue is IResourcesDataNode)
                {
                    IResourcesFileRef fileRef = ((IResourcesDataNode)dataValue).FileRef;
                    if (fileRef == null)
                        return null;
                    else
                        return new ResourcesFileRefInterpreter(fileRef);
                }
                else
                    return null;
            }
        }

        private void SetResXDataNodeValue(ResXDataNode resXDataNode, object value)
        {
            // the intention of this code is to execute the following line:-
            // resXDataNode.nodeInfo.ValueData = value;
            // However, nodeInfo and ValueData are private so Reflection is required to make the assignment.
            FieldInfo nodeInfoFieldInfo = typeof(ResXDataNode).GetField("nodeInfo", BindingFlags.Instance | BindingFlags.NonPublic);
            if (nodeInfoFieldInfo != null)
            {
                object nodeInfo = nodeInfoFieldInfo.GetValue(resXDataNode);
                if (nodeInfo != null)
                {
                    Type nodeInfoType = nodeInfo.GetType();
                    FieldInfo valueDataFieldInfo = nodeInfoType.GetField("ValueData", BindingFlags.Instance | BindingFlags.NonPublic);
                    if (valueDataFieldInfo != null)
                        valueDataFieldInfo.SetValue(nodeInfo, value);
                }
            }
        }
    }
}
