// File           : ResourcesFileRefInterpreter.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Text;
using System.Resources;
using System.Reflection;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ResourcesFileRefInterpreter is an interpreter for resources file references
    /// </summary>
    /// <remarks>The problem that this class solves is this: The ResXFileRef class exists to provide a
    /// file reference for resx files. The problem is that it exists *only* for resx files. No part of this class
    /// can be reused for other resource sources. Other resource managers could support additional
    /// the concept of a file reference but the .NET Framework does not offer any help in this area.
    /// The ResourcesFileRefInterpreter class exists to offer a uniform way of interpreting file references
    /// regardless of their implementation (e.g. ResXFileRef, IResourcesFileRef).</remarks>
    public class ResourcesFileRefInterpreter
    {
        private ResXFileRef resXFileRef;
        private IResourcesFileRef resourcesFileRef;

        /// <summary>
        /// Constructs a ResourcesFileRefInterpreter object for a ResXFileRef
        /// </summary>
        /// <param name="resXFileRef">The ResXFileRef which the interpreter interprets</param>
        public ResourcesFileRefInterpreter(ResXFileRef resXFileRef)
        {
            this.resXFileRef = resXFileRef;
        }
        /// <summary>
        /// Constructs a ResourcesFileRefInterpreter object for a IResourcesFileRef
        /// </summary>
        /// <param name="resourcesFileRef">The IResourcesFileRef which the interpreter interprets</param>
        public ResourcesFileRefInterpreter(IResourcesFileRef resourcesFileRef)
        {
            this.resourcesFileRef = resourcesFileRef;
        }
        /// <summary>
        /// FileName is the name of the file in the file reference
        /// </summary>
        public string FileName
        {
            get
            {
                if (resourcesFileRef != null)
                    return resourcesFileRef.FileName;
                else
                    return resXFileRef.FileName;
            }
            set
            {
                if (resourcesFileRef != null)
                    resourcesFileRef.FileName = value;
                else
                    SetPrivateField(resXFileRef, "fileName", value);
            }
        }
        /// <summary>
        /// TypeName is the string of the type of data in the file
        /// </summary>
        public string TypeName
        {
            get
            {
                if (resourcesFileRef != null)
                    return resourcesFileRef.TypeName;
                else
                    return resXFileRef.TypeName;
            }
            set
            {
                if (resourcesFileRef != null)
                    resourcesFileRef.TypeName = value;
                else
                    SetPrivateField(resXFileRef, "typeName", value);
            }
        }
        /// <summary>
        /// TextFileEncoding is the encoding mechanism of the file if the file is a text file
        /// </summary>
        public Encoding TextFileEncoding
        {
            get
            {
                if (resourcesFileRef != null)
                    return resourcesFileRef.TextFileEncoding;
                else
                    return resXFileRef.TextFileEncoding;
            }
            set
            {
                if (resourcesFileRef != null)
                    resourcesFileRef.TextFileEncoding = value;
                else
                    SetPrivateField(resXFileRef, "textFileEncoding", value);
            }
        }
        /// <summary>
        /// SetPrivateField sets a private field using Reflection
        /// </summary>
        /// <param name="target">The object which contains the private field</param>
        /// <param name="fieldName">The name of the private field</param>
        /// <param name="value">The value to set the property to</param>
        /// <remarks>This method exists because the ResXFileRef class does not support a public
        /// interface for setting the TextFileEncoding property</remarks>
        protected virtual void SetPrivateField(object target, string fieldName, object value)
        {
            FieldInfo fieldInfo = target.GetType().GetField(fieldName, BindingFlags.Instance | BindingFlags.NonPublic);
            if (fieldInfo != null)
                fieldInfo.SetValue(target, value);
        }
    }
}
