// File           : ResourcesResourcesGovernor.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Resources;
using System.Globalization;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ResourcesResourcesGovernor manages a complete set of .resources file resources
    /// </summary>
    /// <remarks>Take care to notice the distinction between two similarly named classes:
    /// ResourcesResourcesGovernor and ResourcesResourceGovernor. The former managers a set of resources whereas the
    /// latter manages a single resource. Further note the potential terminology confusion between a set of
    /// resources, a resource and a resource entry. A set of resources would be the complete set of
    /// .resources files in a folder. A resource would be a single .resources file and a resource entry would be
    /// one entry within a single .resources file.</remarks>
    public class ResourcesResourcesGovernor : FileBasedResourcesGovernor, IResourcesGovernor
	{
		private string path;

        /// <summary>
        /// Constructs a ResourcesResourcesGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        public ResourcesResourcesGovernor(string path)
            : base(path, "resources")
		{
			this.path = path;
		}
        /// <summary>
        /// GetResourceGovernor gets an IResourceGovernor for the given base name and culture
        /// </summary>
        /// <param name="baseName">The base name of the resource</param>
        /// <param name="cultureInfo">The CultureInfo of the resource</param>
        /// <returns>An IResourceGovernor for the given base name and culture</returns>
        /// <remarks>The IResourceGovernor is implemented by the ResourcesResourceGovernor class</remarks>
        public virtual IResourceGovernor GetResourceGovernor(string baseName, CultureInfo cultureInfo)
		{
			return new ResourcesResourceGovernor(path, baseName, cultureInfo);
		}
	}
    /// <summary>
    /// ResourcesResourceGovernor manages a .resources file resource
    /// </summary>
    public class ResourcesResourceGovernor : FileBasedResourceGovernor, IResourceGovernor
	{
        /// <summary>
        /// Constructs a ResourcesResourceGovernor object
        /// </summary>
        /// <param name="path">The path to the resource files</param>
        /// <param name="baseName">The base name of the resource (e.g. "Form1")</param>
        /// <param name="cultureInfo">The CultureInfo which represents the culture of the resource</param>
        public ResourcesResourceGovernor(string path, string baseName, CultureInfo cultureInfo)
            :
			base(path, baseName, cultureInfo, "resources")
		{
		}
        /// <summary>
        /// ReadResourceSet reads the resource entries from the source into a ResourceSet
        /// </summary>
        /// <returns>A ResourceSet containing the resource entries read from the source</returns>
        public override ResourceSet ReadResourceSet()
		{
			return ReadResourceSet(new ResourceReader(Filename));
		}
        /// <summary>
        /// WriteResourceSet writes resource entries in a ResourceSet back to the source
        /// </summary>
        /// <param name="resourceSet">The ResourceSet containing the resource entries to write</param>
        public virtual void WriteResourceSet(ResourceSet resourceSet)
		{
			WriteResourceSet(new ResourceWriter(Filename), resourceSet);
		}
	}
}
