// File           : ResourceEditorControl.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Resources;
using System.IO;
using System.Globalization;
using Internationalization.Common;
using ResourceEditorControlLibrary;

namespace Internationalization.ResourceAdministration
{
    /// <summary>
    /// ResourceEditorControl is a Windows Forms control for editing resources
    /// </summary>
    /// <remarks>The Resource Editor control provides the same functionality as the Resource Editor in
    /// Visual Studio with various enhancements notably that it is not limited to managing resx files</remarks>
    public partial class ResourceEditorControl : UserControl
    {
        private ResourceSet resourceSet;
        private Hashtable resourceSetTable;
        private bool readOnly = false;
        private bool allowComments = false;
        private bool useFileRefs = false;

        /// <summary>
        /// Constructs a ResourceEditorControl object
        /// </summary>
        public ResourceEditorControl()
        {
            InitializeComponent();
            ShowPage(tabPageStrings);
        }

        private void ShowPage(TabPage tabPage)
        {
            if (tabPage == null)
                tabPage = tabControlResources.SelectedTab;
            else
                tabControlResources.SelectedTab = tabPage;

            toolStripMenuItemResourceType.Text = tabPage.Text;
            toolStripMenuItemResourceType.Image = TabPageToMenuItem(tabPage).Image;
            ListView listView = TabPageToListView(tabPage);
            if (listView == null)
                toolStripMenuItemView.Enabled = false;
            else
            {
                toolStripMenuItemView.Enabled = true;
                CheckViewMenuItem(ViewToViewTypeMenuItem(listView.View), true);
            }
            UpdateRemoveResourceStatus(tabPage);
        }
        private void UpdateRemoveResourceStatus(TabPage tabPage)
        {
            ListView listView = TabPageToListView(tabPage);
            if (listView == null)
            {
                DataGridView dataGridView = TabPageToDataGridView(tabPage);
                if (dataGridView == null)
                    toolStripMenuItemRemoveResource.Enabled = false;
                else
                    UpdateRemoveResourceStatus(dataGridView);
            }
            else
                UpdateRemoveResourceStatus(listView);
        }
        private void UpdateRemoveResourceStatus(ListView listView)
        {
            toolStripMenuItemRemoveResource.Enabled = listView.SelectedIndices.Count > 0;
        }
        private void UpdateRemoveResourceStatus(DataGridView dataGridView)
        {
            toolStripMenuItemRemoveResource.Enabled = dataGridView.CurrentRow != null;
        }
        private void CheckViewMenuItem(ToolStripMenuItem toolStripMenuItem, bool check)
        {
            if (toolStripMenuItem == toolStripMenuItemViewDetails)
            {
                toolStripMenuItemViewDetails.Checked = check;
                toolStripMenuItemViewInList.Checked = !check;
                toolStripMenuItemViewThumbnails.Checked = !check;
            }
            else if (toolStripMenuItem == toolStripMenuItemViewInList)
            {
                toolStripMenuItemViewDetails.Checked = !check;
                toolStripMenuItemViewInList.Checked = check;
                toolStripMenuItemViewThumbnails.Checked = !check;
            }
            else if (toolStripMenuItem == toolStripMenuItemViewThumbnails)
            {
                toolStripMenuItemViewDetails.Checked = !check;
                toolStripMenuItemViewInList.Checked = !check;
                toolStripMenuItemViewThumbnails.Checked = check;
            }
        }
        private ToolStripMenuItem ViewToViewTypeMenuItem(View view)
        {
            switch (view)
            {
                case View.List: { return toolStripMenuItemViewInList; }
                case View.Details: { return toolStripMenuItemViewDetails; }
                case View.LargeIcon: { return toolStripMenuItemViewThumbnails; }
                default: { return null; }
            }
        }

        private ToolStripMenuItem TabPageToMenuItem(TabPage tabPage)
        {
            if (tabPage == tabPageStrings)
                return toolStripMenuItemStrings;
            else if (tabPage == tabPageImages)
                return toolStripMenuItemImages;
            else if (tabPage == tabPageIcons)
                return toolStripMenuItemIcons;
            else if (tabPage == tabPageAudio)
                return toolStripMenuItemAudio;
            else if (tabPage == tabPageFiles)
                return toolStripMenuItemFiles;
            else if (tabPage == tabPageOther)
                return toolStripMenuItemOther;
            else
                return null;
        }
        private ListView TabPageToListView(TabPage tabPage)
        {
            if (tabPage == tabPageStrings)
                return null;
            else if (tabPage == tabPageImages)
                return listViewImages;
            else if (tabPage == tabPageIcons)
                return listViewIcons;
            else if (tabPage == tabPageAudio)
                return listViewAudio;
            else if (tabPage == tabPageFiles)
                return listViewFiles;
            else if (tabPage == tabPageOther)
                return null;
            else
                return null;
        }
        private DataGridView TabPageToDataGridView(TabPage tabPage)
        {
            if (tabPage == tabPageStrings)
                return dataGridViewResourceStrings;
            else if (tabPage == tabPageImages)
                return null;
            else if (tabPage == tabPageIcons)
                return null;
            else if (tabPage == tabPageAudio)
                return null;
            else if (tabPage == tabPageFiles)
                return null;
            else if (tabPage == tabPageOther)
                return dataGridViewResourceOther;
            else
                return null;
        }

        private void toolStripMenuItemStrings_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageStrings);
        }

        private void toolStripMenuItemImages_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageImages);
        }

        private void toolStripMenuItemIcons_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageIcons);
        }

        private void toolStripMenuItemAudio_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageAudio);
        }

        private void toolStripMenuItemFiles_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageFiles);
        }

        private void toolStripMenuItemOther_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageOther);
        }

        private void SetListViewView(ToolStripMenuItem toolStripMenuItem, View view)
        {
            CheckViewMenuItem(toolStripMenuItem, true);
            ListView listView = TabPageToListView(tabControlResources.SelectedTab);
            if (listView != null)
                listView.View = view;
        }

        private void toolStripMenuItemViewInList_Click(object sender, EventArgs e)
        {
            SetListViewView(toolStripMenuItemViewInList, View.List);
        }

        private void toolStripMenuItemViewDetails_Click(object sender, EventArgs e)
        {
            SetListViewView(toolStripMenuItemViewDetails, View.Details);
        }

        private void toolStripMenuItemViewThumbnails_Click(object sender, EventArgs e)
        {
            SetListViewView(toolStripMenuItemViewThumbnails, View.LargeIcon);
        }

        private void tabControlResources_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowPage(null);
        }
        /// <summary>
        /// IsStringResource returns true if the ResourcesDataNodeInterpreter represents a string
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter containing the resource entry</param>
        /// <returns>True if the ResourcesDataNodeInterpreter represents a string</returns>
        protected virtual bool IsStringResource(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            return dataNodeInterpreter.FileRef == null && dataNodeInterpreter.Value is string &&
                ResourceUtilities.IsLocalizable(dataNodeInterpreter.Name, dataNodeInterpreter.Value);
        }
        /// <summary>
        /// IsImageResource returns true if the ResourcesDataNodeInterpreter represents an image
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter containing the resource entry</param>
        /// <returns>True if the ResourcesDataNodeInterpreter represents an image (i.e. a Bitmap object)</returns>
        protected virtual bool IsImageResource(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            return dataNodeInterpreter.Value is Bitmap;
        }
        /// <summary>
        /// IsIconResource returns true if the ResourcesDataNodeInterpreter represents an icon
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter containing the resource entry</param>
        /// <returns>True if the ResourcesDataNodeInterpreter represents an icon</returns>
        protected virtual bool IsIconResource(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            return dataNodeInterpreter.Value is Icon;
        }
        /// <summary>
        /// IsAudioResource returns true if the ResourcesDataNodeInterpreter represents audio
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter containing the resource entry</param>
        /// <returns>True if the ResourcesDataNodeInterpreter represents audio. The definition of audio is that
        /// it is a MemoryStream.</returns>
        protected virtual bool IsAudioResource(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            return dataNodeInterpreter.Value is MemoryStream;
        }
        /// <summary>
        /// IsFileResource returns true if the ResourcesDataNodeInterpreter represents a file
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter containing the resource entry</param>
        /// <returns>True if the ResourcesDataNodeInterpreter represents a file reference</returns>
        protected virtual bool IsFileResource(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            return dataNodeInterpreter.IsDataNode && dataNodeInterpreter.FileRef != null
                && ! (IsImageResource(dataNodeInterpreter) || IsIconResource(dataNodeInterpreter)
                || IsAudioResource(dataNodeInterpreter));
        }
        /// <summary>
        /// IsOtherResource returns true if the ResourcesDataNodeInterpreter does not represent a string,
        /// image, icon, audio or file
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter containing the resource entry</param>
        /// <returns>True if the ResourcesDataNodeInterpreter does not represent a string, image, icon, audio
        /// or file</returns>
        protected virtual bool IsOtherResource(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            return !(IsStringResource(dataNodeInterpreter) || IsImageResource(dataNodeInterpreter) ||
                IsIconResource(dataNodeInterpreter) || IsAudioResource(dataNodeInterpreter) ||
                IsFileResource(dataNodeInterpreter));
        }
        /// <summary>
        /// FillResourceStringDataTable fills a DataTable with the string resource entries in a Hashtable of
        /// resource entries
        /// </summary>
        /// <param name="dataTable">The DataTable to fill with resource entries</param>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        protected virtual void FillResourceStringDataTable(DataTable dataTable, Hashtable resourceSetTable)
        {
            foreach (DictionaryEntry entry in resourceSetTable)
            {
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (IsStringResource(dataNodeInterpreter))
                {
                    DataRow dataRow = dataTable.NewRow();
                    dataRow["Name"] = entry.Key.ToString();
                    dataRow["Value"] = dataNodeInterpreter.Value.ToString();
                    dataRow["Comment"] = dataNodeInterpreter.Comment;
                    dataTable.Rows.Add(dataRow);
                }
            }
            dataTable.AcceptChanges();
        }
        /// <summary>
        /// FillResourceOtherDataTable fills a DataTable with the "other" resource entries in a Hashtable of
        /// resource entries. "Other" resources are all resources which are not strings, images, icons, audio
        /// or files.
        /// </summary>
        /// <param name="dataTable">The DataTable to fill with resource entries</param>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        protected virtual void FillResourceOtherDataTable(DataTable dataTable, Hashtable resourceSetTable)
        {
            foreach (DictionaryEntry entry in resourceSetTable)
            {
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (IsOtherResource(dataNodeInterpreter))
                {
                    DataRow dataRow = dataTable.NewRow();
                    dataRow["Name"] = dataNodeInterpreter.Name;
                    if (dataNodeInterpreter.Value == null)
                    {
                        dataRow["Type"] = "null";
                        dataRow["Value"] = System.DBNull.Value;
                    }
                    else
                    {
                        dataRow["Type"] = dataNodeInterpreter.Value.GetType().FullName;
                        dataRow["Value"] = dataNodeInterpreter.Value;
                    }
                    dataTable.Rows.Add(dataRow);
                }
            }
            dataTable.AcceptChanges();
        }
        /// <summary>
        /// CreateResourceStringDataTable creates a DataTable for string resource entries and adds it to the
        /// given DataSet
        /// </summary>
        /// <param name="dataSet">The DataSet to add the DataTable to</param>
        /// <returns>A DataTable for string resource entries</returns>
        protected virtual DataTable CreateResourceStringDataTable(DataSet dataSet)
        {
            DataTable dataTable = new DataTable("Strings");

            dataTable.Columns.Add("Name"); //, typeof(String)).MaxLength = 50;

            dataTable.Columns.Add("Value");

            dataTable.Columns.Add("Comment");

            if (dataSet != null)
                dataSet.Tables.Add(dataTable);

            return dataTable;
        }
        /// <summary>
        /// CreateResourceOtherDataTable creates a DataTable for "other" resource entries and adds it to the
        /// given DataSet. "Other" resources are all resources which are not strings, images, icons, audio or files.
        /// </summary>
        /// <param name="dataSet">The DataSet to add the DataTable to</param>
        /// <returns>A DataTable for string resource entries</returns>
        protected virtual DataTable CreateResourceOtherDataTable(DataSet dataSet)
        {
            DataTable dataTable = new DataTable("Other");

            dataTable.Columns.Add("Name");

            dataTable.Columns.Add("Type");

            dataTable.Columns.Add("Value", typeof(object));

            if (dataSet != null)
                dataSet.Tables.Add(dataTable);

            return dataTable;
        }

        /// <summary>
        /// ResourceSet is the ResourceSet which is maintained by the ResourceEditorControl
        /// </summary>
        [Browsable(false)]
        public ResourceSet ResourceSet
        {
            get { return resourceSet; }
            set
            {
                if (value != null)
                {
                    resourceSet = value;
                    resourceSetTable = ResourceUtilities.GetResourceSetTable(resourceSet);
                    DataSet dataSet = new DataSet("Resources");

                    InitializeResourceStrings(resourceSetTable, dataSet);
                    InitializeResourceImages(resourceSetTable);
                    InitializeResourceIcons(resourceSetTable);
                    InitializeResourceAudio(resourceSetTable);
                    InitializeResourceFiles(resourceSetTable);
                    InitializeResourceOther(resourceSetTable, dataSet);

                    InitializeCommentsStatus();
                    InitializeFileRefsStatus();
                    InitializeReadOnlyStatus();
                }
            }
        }
        // protected delegate bool IsResourceType(DictionaryEntry entry);

        /// <summary>
        /// InitializeResourceStrings initializes the internal cache of string resource entries
        /// </summary>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        /// <param name="dataSet">The DataSet to which a new DataTable of string resource entries is added</param>
        protected virtual void InitializeResourceStrings(Hashtable resourceSetTable, DataSet dataSet)
        {
            DataTable dataTable = CreateResourceStringDataTable(dataSet);
            FillResourceStringDataTable(dataTable, resourceSetTable);
            dataTable.RowChanged += new DataRowChangeEventHandler(dataTable_RowChanged);
            dataTable.RowDeleting += new DataRowChangeEventHandler(dataTable_RowDeleting);

            dataGridViewResourceStrings.DataSource = dataTable.DataSet;
            dataGridViewResourceStrings.DataMember = dataTable.TableName;
            dataGridViewResourceStrings.Columns["Name"].Width = 200;
            dataGridViewResourceStrings.Columns["Value"].Width = 340;
            // dataGridViewResourceStrings.AutoResizeColumns();
        }
        /// <summary>
        /// InitializeResourceImages initializes the internal cache of image resource entries
        /// </summary>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        protected virtual void InitializeResourceImages(Hashtable resourceSetTable)
        {
            listViewImages.SmallImageList = new ImageList();
            listViewImages.LargeImageList = new ImageList();
            listViewImages.LargeImageList.ImageSize = new System.Drawing.Size(96, 96);
            listViewImages.Items.Clear();
            foreach (DictionaryEntry entry in resourceSetTable)
            {
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (IsImageResource(dataNodeInterpreter))
                    AddResourceImage(dataNodeInterpreter);
            }
        }
        /// <summary>
        /// AddResourceImage adds an image resource entry to the images ListView
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter which represents the image
        /// resource entry</param>
        protected virtual void AddResourceImage(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            listViewImages.LargeImageList.Images.Add((Bitmap)dataNodeInterpreter.Value);
            listViewImages.SmallImageList.Images.Add((Bitmap)dataNodeInterpreter.Value);
            listViewImages.Items.Add(new ListViewItemDataNodeInterpreter(
                dataNodeInterpreter.Name, listViewImages.Items.Count, dataNodeInterpreter));
        }
        /// <summary>
        /// InitializeResourceIcons initializes the internal cache of icon resource entries
        /// </summary>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        protected virtual void InitializeResourceIcons(Hashtable resourceSetTable)
        {
            listViewIcons.SmallImageList = new ImageList();
            listViewIcons.LargeImageList = new ImageList();
            listViewIcons.LargeImageList.ImageSize = new System.Drawing.Size(96, 96);
            listViewIcons.Items.Clear();
            foreach (DictionaryEntry entry in resourceSetTable)
            {
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (IsIconResource(dataNodeInterpreter))
                    AddResourceIcon(dataNodeInterpreter);
            }
        }
        /// <summary>
        /// AddResourceIcon adds an icon resource entry to the icons ListView
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter which represents the icon
        /// resource entry</param>
        protected virtual void AddResourceIcon(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            listViewIcons.SmallImageList.Images.Add((Icon)dataNodeInterpreter.Value);
            listViewIcons.LargeImageList.Images.Add((Icon)dataNodeInterpreter.Value);
            listViewIcons.Items.Add(new ListViewItemDataNodeInterpreter(
                dataNodeInterpreter.Name, listViewIcons.Items.Count, dataNodeInterpreter));
        }
        /// <summary>
        /// InitializeResourceAudio initializes the internal cache of audio resource entries
        /// </summary>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        protected virtual void InitializeResourceAudio(Hashtable resourceSetTable)
        {
            listViewAudio.Items.Clear();
            foreach (DictionaryEntry entry in resourceSetTable)
            {
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (IsAudioResource(dataNodeInterpreter))
                    AddResourceAudio(dataNodeInterpreter);
            }
        }
        /// <summary>
        /// AddResourceAudio adds an audio resource entry to the audio ListView
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter which represents the audio
        /// resource entry</param>
        protected virtual void AddResourceAudio(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            listViewAudio.Items.Add(new ListViewItemDataNodeInterpreter(
                dataNodeInterpreter.Name, 0, dataNodeInterpreter));
        }
        /// <summary>
        /// InitializeResourceFiles initializes the internal cache of file resource entries
        /// </summary>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        protected virtual void InitializeResourceFiles(Hashtable resourceSetTable)
        {
            listViewFiles.Items.Clear();
            foreach (DictionaryEntry entry in resourceSetTable)
            {
                ResourcesDataNodeInterpreter dataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (IsFileResource(dataNodeInterpreter))
                    AddResourceFile(dataNodeInterpreter);
            }
        }
        /// <summary>
        /// AddResourceFile adds a file resource entry to the files ListView
        /// </summary>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter which represents the file
        /// resource entry</param>
        protected virtual void AddResourceFile(ResourcesDataNodeInterpreter dataNodeInterpreter)
        {
            listViewFiles.Items.Add(new ListViewItemDataNodeInterpreter(
                dataNodeInterpreter.Name, 0, dataNodeInterpreter));
        }
        /// <summary>
        /// InitializeResourceOther initializes the internal cache of other resource entries
        /// </summary>
        /// <param name="resourceSetTable">The Hashtable of resource entries</param>
        /// <param name="dataSet">The DataSet to which a new DataTable of other resource entries is added</param>
        protected virtual void InitializeResourceOther(Hashtable resourceSetTable, DataSet dataSet)
        {
            DataTable dataTable = CreateResourceOtherDataTable(dataSet);
            FillResourceOtherDataTable(dataTable, resourceSetTable);
            dataTable.RowChanged += new DataRowChangeEventHandler(dataTable_RowChanged);

            dataGridViewResourceOther.DataSource = dataTable.DataSet;
            dataGridViewResourceOther.DataMember = dataTable.TableName;
            dataGridViewResourceOther.AllowUserToAddRows = false;
            dataGridViewResourceOther.AllowUserToDeleteRows = false;
            dataGridViewResourceOther.Columns["Name"].Width = 200;
            dataGridViewResourceOther.Columns["Type"].Width = 150;
            dataGridViewResourceOther.Columns["Type"].ReadOnly = true;
            dataGridViewResourceOther.Columns["Value"].Width = 240;
            // dataGridViewResourceOther.AutoResizeColumns();
        }
        /// <summary>
        /// UseFileRefs specifies whether file references should be created for new resources based on files
        /// </summary>
        /// <remarks>If UseFileRefs is false the complete contents of the file are stored as the resource data
        /// instead of just a file reference</remarks>
        [Browsable(false)]
        public bool UseFileRefs
        {
            get { return useFileRefs; }
            set
            {
                useFileRefs = value;
                InitializeFileRefsStatus();
            }
        }
        private void InitializeFileRefsStatus()
        {
            if (useFileRefs)
            {
                toolStripMenuItemFiles.Enabled = true;
                tabPageFiles.Enabled = true;
            }
            else
            {
                toolStripMenuItemFiles.Enabled = false;
                tabPageFiles.Enabled = false;
            }
        }
        /// <summary>
        /// AllowComments specifies whether the comments should be allowed (and displayed)
        /// </summary>
        /// <remarks>AllowComments should be set to true if the existing ResourceSet uses data nodes and
        /// new resource entries should be created with data nodes</remarks>
        [Browsable(false)]
        public bool AllowComments
        {
            get { return allowComments; }
            set
            {
                allowComments = value;
                InitializeCommentsStatus();
            }
        }
        private void InitializeCommentsStatus()
        {
            DataGridViewColumn dataGridViewColumn = dataGridViewResourceStrings.Columns["Comment"];
            if (dataGridViewColumn != null)
                dataGridViewColumn.Visible = allowComments;
        }
        /// <summary>
        /// ReadOnly is true of the ResourceEditorControl should allow read-only access to the ResourceSet
        /// </summary>
        /// <remarks>Set ReadOnly to true if the source of the resources is read only (e.g. Form1.resx is
        /// read-only because it is under version control and has not been checked out)</remarks>
        [Browsable(false)]
        public bool ReadOnly
        {
            get { return readOnly; }
            set
            {
                readOnly = value;
                InitializeReadOnlyStatus();
            }
        }
        private void InitializeReadOnlyStatus()
        {
            if (resourceSet != null)
            {
                if (readOnly)
                {
                    dataGridViewResourceStrings.ReadOnly = true;
                    dataGridViewResourceOther  .ReadOnly = true;
                    toolStripMenuItemAddResource.Enabled = false;
                    toolStripMenuItemRemoveResource.Enabled = false;
                    listViewImages.LabelEdit = false;
                    listViewIcons.LabelEdit = false;
                    listViewAudio.LabelEdit = false;
                    listViewFiles.LabelEdit = false;
                }
                else
                {
                    dataGridViewResourceStrings.ReadOnly = false;
                    dataGridViewResourceOther  .ReadOnly = false;
                    toolStripMenuItemAddResource.Enabled = true;
                    toolStripMenuItemRemoveResource.Enabled = true;
                    listViewImages.LabelEdit = true;
                    listViewIcons.LabelEdit = true;
                    listViewAudio.LabelEdit = true;
                    listViewFiles.LabelEdit = true;
                }
            }
        }
        private void dataTable_RowChanged(object sender, DataRowChangeEventArgs e)
        {
            ApplyChangeToResources(e);
        }
        private void dataTable_RowDeleting(object sender, DataRowChangeEventArgs e)
        {
            ApplyChangeToResources(e);
        }

        /// <summary>
        /// DataNodeCreating is called by CreateDataNode to create a new data node object
        /// </summary>
        /// <remarks>Set the DataNodeCreating event to an appropriate handler to create a more complex and
        /// appropriate data node such as a ResXDataNode or IResourcesDataNode</remarks>
        public event DataNodeCreatingEventHandler DataNodeCreating;

        /// <summary>
        /// FileRefCreating is called by CreateFileRef to create a new file reference object
        /// </summary>
        /// <remarks>Set the FileRefCreating event to an appropriate handler to create an appropriate file
        /// reference object such as a ResXFileRef or IResourcesFileRef</remarks>
        public event FileRefCreatingEventHandler FileRefCreating;

        /// <summary>
        /// ResourceAbsoluteFileName is called by GetTargetResourceAbsoluteFileName to get an absolute file name
        /// for a resource
        /// </summary>
        public event ResourceFileNameEventHandler ResourceAbsoluteFileName;

        /// <summary>
        /// ResourceRelativeFileName is called by GetTargetResourceRelativeFileName to get a relative file name
        /// for a resource
        /// </summary>
        public event ResourceFileNameEventHandler ResourceRelativeFileName;

        /// <summary>
        /// ResourceSetChanging is called when the ResourceSet is being changed by the ResourceEditorControl
        /// </summary>
        /// <remarks>The type of change is specified by the ResourceSetAction in the event arguments</remarks>
        public event ResourceSetChangingEventHandler ResourceSetChanging;

        /// <summary>
        /// CreateDataNode creates a data node for use within the ResourceEditorControl
        /// </summary>
        /// <param name="name">The name of the data node</param>
        /// <param name="value">The value of the data node</param>
        /// <returns>A data node object appropriate for the ResourceSet</returns>
        /// <remarks>By default, a data node is simply the value passed to CreateDataNode. Set the
        /// DataNodeCreating event to an appropriate handler to create a more complex and appropriate data
        /// node such as a ResXDataNode or IResourcesDataNode.</remarks>
        protected virtual object CreateDataNode(string name, object value)
        {
            if (DataNodeCreating != null)
            {
                DataNodeCreatingEventArgs eventArgs = new DataNodeCreatingEventArgs(name, value);
                DataNodeCreating(this, eventArgs);
                return eventArgs.Value;
            }
            else
                return value;
        }
        /// <summary>
        /// CreateFileRef creates a file reference for use within the ResourceEditorControl
        /// </summary>
        /// <param name="fileName">The name of the file to reference</param>
        /// <param name="typeName">The string of the Type of the data in the file</param>
        /// <returns>A file reference object appropriate for the ResourceSet</returns>
        /// <remarks>By default, a file reference is null. Set the FileRefCreating event to an appropriate
        /// handler to create an appropriate file reference object such as a ResXFileRef or IResourcesFileRef.
        /// </remarks>
        protected virtual object CreateFileRef(string fileName, string typeName)
        {
            if (FileRefCreating != null)
            {
                FileRefCreatingEventArgs eventArgs = new FileRefCreatingEventArgs(fileName, typeName);
                FileRefCreating(this, eventArgs);
                return eventArgs.FileRef;
            }
            else
                return null;
        }
        private void ApplyChangeToResources(DataRowChangeEventArgs eventArgs)
        {
            string oldResourceKey = null;
            object oldResourceValue = null;
            string oldResourceComment = null;
            string resourceKey;
            object resourceValue;
            string resourceComment;

            if (eventArgs.Action == DataRowAction.Add || eventArgs.Action == DataRowAction.Change ||
                eventArgs.Action == DataRowAction.Delete)
            {
                if (eventArgs.Action == DataRowAction.Delete)
                {
                    resourceKey = eventArgs.Row["Name", DataRowVersion.Original].ToString();
                    resourceValue = eventArgs.Row["Value", DataRowVersion.Original];
                    resourceComment = eventArgs.Row["Comment", DataRowVersion.Original].ToString();
                }
                else
                {
                    resourceKey = eventArgs.Row["Name"].ToString();
                    resourceValue = eventArgs.Row["Value"];
                    resourceComment = eventArgs.Row["Comment"].ToString();
                }

                switch (eventArgs.Action)
                {
                    case DataRowAction.Add:
                        {
                            object dataNode = CreateDataNode(resourceKey, resourceValue.ToString());
                            resourceSetTable.Add(resourceKey, dataNode);
                            break;
                        }
                    case DataRowAction.Delete:
                        {
                            if (resourceSetTable.ContainsKey(resourceKey))
                                resourceSetTable.Remove(resourceKey);
                            break;
                        }
                    case DataRowAction.Change:
                        {
                            if (eventArgs.Row.HasVersion(DataRowVersion.Original))
                            {
                                oldResourceValue = eventArgs.Row["Value", DataRowVersion.Original];
                                oldResourceKey = eventArgs.Row["Name", DataRowVersion.Original].ToString();
                                oldResourceComment = eventArgs.Row["Comment", DataRowVersion.Original].ToString();
                            }
                            else
                            {
                                // this is an edit of a row which was previously added so it doesn't have an Original value
                                oldResourceValue = eventArgs.Row["Value"];
                                oldResourceKey = eventArgs.Row["Name"].ToString();
                                oldResourceComment = eventArgs.Row["Comment"].ToString();
                            }

                            object dataNode = CreateDataNode(resourceKey, resourceValue.ToString());
                            ResourcesDataNodeInterpreter dataNodeInterpreter =
                                new ResourcesDataNodeInterpreter(resourceKey, dataNode);
                            dataNodeInterpreter.Comment = resourceComment;
                            if (oldResourceKey != resourceKey)
                            {
                                // remove the old key and add the new key
                                resourceSetTable.Remove(oldResourceKey);
                                resourceSetTable.Add(resourceKey, dataNodeInterpreter.DataNode);
                            }
                            else
                                resourceSetTable[resourceKey] = dataNodeInterpreter.DataNode;

                            break;
                        }
                }
                if (ResourceSetChanging != null)
                {
                    ResourceSetAction action = DataRowActionToResourceSetAction(eventArgs.Action);
                    ResourceSetChanging(this, new ResourceSetChangeEventArgs(resourceSet, action,
                        resourceKey, resourceValue, resourceComment,
                        oldResourceKey, oldResourceValue, oldResourceComment));
                }
            }
        }
        private ResourceSetAction DataRowActionToResourceSetAction(DataRowAction action)
        {
            switch (action)
            {
                case DataRowAction.Add: { return ResourceSetAction.Add; }
                case DataRowAction.Change: { return ResourceSetAction.Change; }
                case DataRowAction.Delete: { return ResourceSetAction.Delete; }
                default: { return (ResourceSetAction)(-1); }
            }
        }

        private void listViewImages_AfterLabelEdit(object sender, System.Windows.Forms.LabelEditEventArgs e)
        {
            string resourceKey = e.Label;
            if (resourceKey != null)
            {
                ListView listView = (ListView)sender;
                ListViewItem listViewItem = listView.SelectedItems[0];
                string oldResourceKey = listViewItem.Text;
                Image resourceValue = listViewItem.ImageList.Images[listViewItem.ImageIndex];

                if (oldResourceKey != resourceKey)
                {
                    // remove the old key and add the new key
                    resourceSetTable.Remove(oldResourceKey);
                    resourceSetTable.Add(resourceKey, resourceValue);

                    if (ResourceSetChanging != null)
                    {
                        ResourceSetChanging(this, new ResourceSetChangeEventArgs(
                            resourceSet, ResourceSetAction.Change,
                            resourceKey, resourceValue, String.Empty,
                            oldResourceKey, resourceValue, String.Empty));
                    }
                }
            }
        }

        private void addNewStringToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowPage(tabPageStrings);
            AddNewResourceString();
        }
        /// <summary>
        /// AddNewResourceString adds a new "empty" resource string to the internal DataTable of string resource
        /// entries
        /// </summary>
        /// <remarks>The new "empty" resource entry has a default name assigned to it</remarks>
        protected virtual void AddNewResourceString()
        {
            DataTable dataTable = ((DataSet) dataGridViewResourceStrings.DataSource).Tables["Strings"];
            DataRow dataRow = dataTable.NewRow();
            dataRow["Name"] = GetNewResourceStringName(dataTable);
            dataTable.Rows.Add(dataRow);
        }
        /// <summary>
        /// GetNewResourceStringName gets a new default name for a new string resource entry
        /// </summary>
        /// <param name="dataTable">The DataTable of string resource entries</param>
        /// <returns>A new default name for a new string resource entry</returns>
        /// <remarks>GetNewResourceStringName gets the next unused resource entry name using the formula:
        /// String[number] where [number] is the next unused number.</remarks>
        protected virtual string GetNewResourceStringName(DataTable dataTable)
        {
            // get the next name for the resource string (e.g. "String1")
            string namePrefix = "String";
            List<int> usedNumbers = new List<int>();
            foreach (DataRow dataRow in dataTable.Rows)
            {
                if (String.Compare(dataRow["Name"].ToString().Substring(0, namePrefix.Length),
                    namePrefix, true, CultureInfo.InvariantCulture) == 0)
                {
                    string numberString = dataRow["Name"].ToString().Substring(namePrefix.Length);
                    int number;
                    if (Int32.TryParse(numberString, out number))
                        usedNumbers.Add(number);
                }
            }
            usedNumbers.Sort();
            int nextAvailableNumber = 1;
            foreach (int usedNumber in usedNumbers)
            {
                if (usedNumber == nextAvailableNumber)
                    nextAvailableNumber++;
                else if (usedNumber > nextAvailableNumber)
                    break;
            }
            return namePrefix + nextAvailableNumber.ToString();
        }

        private void listView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateRemoveResourceStatus((ListView) sender);
        }

        private void dataGridViewResource_UserAddedRow(object sender, DataGridViewRowEventArgs e)
        {
            UpdateRemoveResourceStatus((DataGridView)sender);
        }

        private void dataGridViewResource_UserDeletedRow(object sender, DataGridViewRowEventArgs e)
        {
            UpdateRemoveResourceStatus((DataGridView)sender);
        }

        private void toolStripMenuItemRemoveResource_Click(object sender, EventArgs e)
        {
            RemoveResource();
        }
        private void RemoveResource()
        {
            TabPage tabPage = tabControlResources.SelectedTab;
            ListView listView = TabPageToListView(tabPage);
            if (listView == null)
            {
                DataGridView dataGridView = TabPageToDataGridView(tabPage);
                if (dataGridView.CurrentRow != null)
                {
                    DataRow dataRow = (DataRow) ((DataRowView)dataGridView.CurrentRow.DataBoundItem).Row;
                    dataRow.Table.Rows.Remove(dataRow);
                }
            }
            else
            {
                ListViewItemDataNodeInterpreter listViewItemDataNodeInterpreter = (ListViewItemDataNodeInterpreter) listView.SelectedItems[0];
                ResourcesDataNodeInterpreter dataNodeInterpreter = listViewItemDataNodeInterpreter.DataNodeInterpreter;
                // remove the entry from the ResourceSet
                resourceSetTable.Remove(dataNodeInterpreter.Name);
                if (ResourceSetChanging != null)
                {
                    ResourceSetChangeEventArgs eventArgs =
                        new ResourceSetChangeEventArgs(resourceSet, ResourceSetAction.Delete,
                        dataNodeInterpreter.Name, dataNodeInterpreter.Value, dataNodeInterpreter.Comment,
                        null, null, null);
                    ResourceSetChanging(this, eventArgs);
                }
                listView.SelectedItems[0].Remove();
            }
        }

        private void addExistingFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                string fileName = openFileDialog.FileName;
                bool addResource = true;
                if (useFileRefs)
                {
                    // the file has to be copied into the Resources folder first
                    string targetFileName = GetTargetResourceAbsoluteFileName(fileName);
                    if (String.Compare(targetFileName, fileName, true, CultureInfo.InvariantCulture) != 0)
                    {
                        if (File.Exists(targetFileName))
                        {
                            switch (MessageBox.Show(String.Format(
                            ResourceEditorControlResources.FileExistsReplaceIt,
                            targetFileName),
                            ResourceEditorControlResources.DestinationExists,
                            MessageBoxButtons.YesNoCancel))
                            {
                                case DialogResult.Yes:
                                    {
                                        File.Delete(targetFileName);
                                        File.Copy(fileName, targetFileName);
                                        break;
                                    }
                                case DialogResult.No:
                                    {
                                        break;
                                    }
                                default:
                                    {
                                        addResource = false;
                                        break;
                                    }
                            }
                        }
                        else
                            File.Copy(fileName, targetFileName);

                        fileName = targetFileName;
                        // fileName = GetTargetResourceRelativeFileName(fileName);
                    }
                }

                if (addResource)
                {
                    TabPage tabPage = null;
                    string resourceKey = GetUniqueResourceKey(
                        Path.GetFileNameWithoutExtension(openFileDialog.FileName));
                    object resourceValue = null;

                    string extension = Path.GetExtension(openFileDialog.FileName).ToUpper();
                    if (extension == ".BMP" || extension == ".JPG" || extension == ".JPEG" || extension == ".PNG" ||
                        extension == ".TIF" || extension == ".TIFF")
                    {
                        tabPage = tabPageImages;
                        resourceValue = CreateAndAddResourceImage(resourceKey, fileName);
                    }
                    else if (extension == ".ICO")
                    {
                        tabPage = tabPageIcons;
                        resourceValue = CreateAndAddResourceIcon(resourceKey, fileName);
                    }
                    else if (extension == ".WAV")
                    {
                        tabPage = tabPageAudio;
                        resourceValue = CreateAndAddResourceAudio(resourceKey, fileName);
                    }
                    else
                    {
                        tabPage = tabPageFiles;
                        resourceValue = CreateAndAddResourceFile(resourceKey, fileName);
                    }

                    if (resourceValue != null)
                    {
                        resourceSetTable.Add(resourceKey, resourceValue);

                        if (ResourceSetChanging != null)
                            ResourceSetChanging(this,
                                new ResourceSetChangeEventArgs(resourceSet, ResourceSetAction.Add,
                                resourceKey, resourceValue, null, null, null, null));

                        ShowPage(tabPage);
                    }
                }
            }
        }
        /// <summary>
        /// GetTargetResourceAbsoluteFileName gets the absolute file name for the given filename
        /// </summary>
        /// <param name="fileName">A resource filename</param>
        /// <returns>An absolute resource file name</returns>
        /// <remarks>By default GetTargetResourceAbsoluteFileName simply returns the given filename. Set the
        /// ResourceAbsoluteFileName event to provide a more appropriate implementation for an absolute
        /// filename.</remarks>
        protected virtual string GetTargetResourceAbsoluteFileName(string fileName)
        {
            if (ResourceAbsoluteFileName != null)
            {
                ResourceFileNameEventArgs eventArgs = new ResourceFileNameEventArgs(fileName);
                ResourceAbsoluteFileName(this, eventArgs);
                fileName = eventArgs.FileName;
            }
            return fileName;
        }
        /// <summary>
        /// GetTargetResourceRelativeFileName gets the relative file name for the given filename
        /// </summary>
        /// <param name="fileName">A resource filename</param>
        /// <returns>An relative resource file name</returns>
        /// <remarks>By default GetTargetResourceRelativeFileName simply returns the given filename. Set the
        /// ResourceRelativeFileName event to provide a more appropriate implementation for a relative
        /// filename.</remarks>
        protected virtual string GetTargetResourceRelativeFileName(string fileName)
        {
            if (ResourceRelativeFileName != null)
            {
                ResourceFileNameEventArgs eventArgs = new ResourceFileNameEventArgs(fileName);
                ResourceRelativeFileName(this, eventArgs);
                fileName = eventArgs.FileName;
            }
            return fileName;
        }
        /// <summary>
        /// GetUniqueResourceKey gets a unique resource entry name
        /// </summary>
        /// <param name="resourceKey">A resource entry name to which a unique suffix may be added</param>
        /// <returns>A unique resource entry name</returns>
        /// <remarks>GetUniqueResourceKey tests for the uniqueness of the given key. If the key is already unique
        /// it is returned unchanged. Otherwise it is suffixed with a number to ensure its uniqueness.</remarks>
        protected virtual string GetUniqueResourceKey(string resourceKey)
        {
            if (resourceSetTable.ContainsKey(resourceKey))
            {
                int resourceNumber = 1;
                while (resourceSetTable.ContainsKey(resourceKey + resourceNumber.ToString()))
                {
                    resourceNumber++;
                }
                return resourceKey + resourceNumber.ToString();
            }
            else
                return resourceKey;
        }
        /// <summary>
        /// CreateAndAddResourceImage creates a new image resource entry and adds it to the list of resource entries
        /// </summary>
        /// <param name="resourceKey">The name of the resource entry</param>
        /// <param name="fileName">The filename containing the resources</param>
        /// <returns>The new image resource entry</returns>
        protected virtual object CreateAndAddResourceImage(string resourceKey, string fileName)
        {
            object resourceValue;
            if (useFileRefs)
            {
                object fileRef = CreateFileRef(fileName, typeof(Bitmap).AssemblyQualifiedName);
                if (fileRef == null)
                    resourceValue = new Bitmap(fileName);
                else
                    resourceValue = CreateDataNode(resourceKey, fileRef);
            }
            else
                resourceValue = new Bitmap(fileName);

            AddResourceImage(new ResourcesDataNodeInterpreter(resourceKey, resourceValue));
            return resourceValue;
        }
        /// <summary>
        /// CreateAndAddResourceIcon creates a new icon resource entry and adds it to the list of resource entries
        /// </summary>
        /// <param name="resourceKey">The name of the resource entry</param>
        /// <param name="fileName">The filename containing the resources</param>
        /// <returns>The new icon resource entry</returns>
        protected virtual object CreateAndAddResourceIcon(string resourceKey, string fileName)
        {
            object resourceValue;
            if (useFileRefs)
            {
                object fileRef = CreateFileRef(fileName, typeof(Icon).AssemblyQualifiedName);
                if (fileRef == null)
                    resourceValue = new Icon(fileName);
                else
                    resourceValue = CreateDataNode(resourceKey, fileRef);
            }
            else
                resourceValue = new Icon(fileName);

            AddResourceIcon(new ResourcesDataNodeInterpreter(resourceKey, resourceValue));
            return resourceValue;
        }
        /// <summary>
        /// CreateAndAddResourceAudio creates a new audio resource entry and adds it to the list of resource entries
        /// </summary>
        /// <param name="resourceKey">The name of the resource entry</param>
        /// <param name="fileName">The filename containing the resources</param>
        /// <returns>The new audio resource entry</returns>
        protected virtual object CreateAndAddResourceAudio(string resourceKey, string fileName)
        {
            object resourceValue;
            if (useFileRefs)
            {
                object fileRef = CreateFileRef(fileName, typeof(MemoryStream).AssemblyQualifiedName);
                if (fileRef == null)
                    resourceValue = ReadFileToMemoryStream(fileName);
                else
                    resourceValue = CreateDataNode(resourceKey, fileRef);
            }
            else
                resourceValue = ReadFileToMemoryStream(fileName);

            AddResourceAudio(new ResourcesDataNodeInterpreter(resourceKey, resourceValue));
            return resourceValue;
        }
        /// <summary>
        /// ReadFileToMemoryStream reads a file into a MemoryStream
        /// </summary>
        /// <param name="fileName">A file to read</param>
        /// <returns>A MemoryStream containing the contents of the file</returns>
        protected virtual MemoryStream ReadFileToMemoryStream(string fileName)
        {
            MemoryStream memoryStream = new MemoryStream();
            using (FileStream fileStream = File.OpenRead(fileName))
            {
                byte[] bytes = new byte[1024];
                while (fileStream.Read(bytes, 0, bytes.Length) > 0)
                {
                    memoryStream.Write(bytes, 0, bytes.Length);
                }
            }
            return memoryStream;
        }
        /// <summary>
        /// CreateAndAddResourceFile creates a new file resource entry and adds it to the list of resource entries
        /// </summary>
        /// <param name="resourceKey">The name of the resource entry</param>
        /// <param name="fileName">The filename containing the resources</param>
        /// <returns>The new file resource entry</returns>
        protected virtual object CreateAndAddResourceFile(string resourceKey, string fileName)
        {
            object resourceValue;
            if (useFileRefs)
            {
                object fileRef = CreateFileRef(fileName, typeof(String).AssemblyQualifiedName);
                if (fileRef == null)
                    resourceValue = ReadFileToString(fileName);
                else
                    resourceValue = CreateDataNode(resourceKey, fileRef);
            }
            else
                resourceValue = ReadFileToString(fileName);

            AddResourceFile(new ResourcesDataNodeInterpreter(resourceKey, resourceValue));
            return resourceValue;
        }
        /// <summary>
        /// ReadFileToString reads a file to a string
        /// </summary>
        /// <param name="fileName">A file to read</param>
        /// <returns>A string containing the contents of the file</returns>
        protected virtual string ReadFileToString(string fileName)
        {
            StringBuilder stringBuilder = new StringBuilder();
            using (FileStream fileStream = File.OpenRead(fileName))
            {
                byte[] bytes = new byte[1024];
                UTF8Encoding encoding = new UTF8Encoding(true);
                while (fileStream.Read(bytes, 0, bytes.Length) > 0)
                {
                    stringBuilder.Append(encoding.GetString(bytes));
                }
            }
            return stringBuilder.ToString();
        }
    }
    /// <summary>
    /// ResourceFileNameEventHandler is an event handler used to get absolute and relative resource file names
    /// </summary>
    /// <param name="sender">The source of the event</param>
    /// <param name="eventArgs">The ResourceFileNameEventArgs event arguments passed to the event handler</param>
    public delegate void ResourceFileNameEventHandler(object sender, ResourceFileNameEventArgs eventArgs);

    /// <summary>
    /// FileRefCreatingEventHandler is an event handler used to get a new file reference object
    /// </summary>
    /// <param name="sender">The source of the event</param>
    /// <param name="eventArgs">The FileRefCreatingEventArgs event arguments passed to the event handler</param>
    public delegate void FileRefCreatingEventHandler(object sender, FileRefCreatingEventArgs eventArgs);

    /// <summary>
    /// DataNodeCreatingEventHandler is an event handler used to get a new data node object
    /// </summary>
    /// <param name="sender">The source of the event</param>
    /// <param name="eventArgs">The DataNodeCreatingEventArgs event arguments passed to the event handler</param>
    public delegate void DataNodeCreatingEventHandler(object sender, DataNodeCreatingEventArgs eventArgs);

    /// <summary>
    /// ResourceFileNameEventArgs represent the arguments passed to a ResourceFileNameEventHandler event handler
    /// </summary>
    public class ResourceFileNameEventArgs : EventArgs
    {
        private string fileName;
        /// <summary>
        /// Constructs a ResourceFileNameEventArgs object
        /// </summary>
        /// <param name="fileName">The filename of the resource</param>
        public ResourceFileNameEventArgs(string fileName)
        {
            this.fileName = fileName;
        }
        /// <summary>
        /// The filename of the resource
        /// </summary>
        public string FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }
    }

    /// <summary>
    /// FileRefCreatingEventArgs represent the arguments passed to a FileRefCreatingEventHandler event handler
    /// </summary>
    public class FileRefCreatingEventArgs : EventArgs
    {
        private object fileRef;
        private string typeName;
        private string fileName;
        /// <summary>
        /// Constructs a FileRefCreatingEventArgs object
        /// </summary>
        /// <param name="fileName">The file name of the resource</param>
        /// <param name="typeName">The string of the Type of the data in the file</param>
        public FileRefCreatingEventArgs(string fileName, string typeName)
        {
            this.fileName = fileName;
            this.typeName = typeName;
        }
        /// <summary>
        /// The file name of the resource
        /// </summary>
        public string FileName
        {
            get { return fileName; }
        }
        /// <summary>
        /// The string of the Type of the data in the file
        /// </summary>
        public string TypeName
        {
            get { return typeName; }
            set { typeName = value; }
        }
        /// <summary>
        /// FileRef is the file reference object (if any) representing the resource file
        /// </summary>
        public object FileRef
        {
            get { return fileRef; }
            set { fileRef = value; }
        }
    }

    /// <summary>
    /// DataNodeCreatingEventArgs represent the arguments passed to a DataNodeCreatingEventHandler event handler
    /// </summary>
    public class DataNodeCreatingEventArgs : EventArgs
    {
        private object value;
        private string name;
        /// <summary>
        /// Constructs a DataNodeCreatingEventArgs object
        /// </summary>
        /// <param name="name">The name of the resource entry</param>
        /// <param name="value">The value of the resource entry</param>
        public DataNodeCreatingEventArgs(string name, object value)
        {
            this.name = name;
            this.value = value;
        }
        /// <summary>
        /// The name of the resource entry
        /// </summary>
        public string Name
        {
            get { return name; }
        }
        /// <summary>
        /// The value of the resource entry
        /// </summary>
        public object Value
        {
            get { return value; }
            set { this.value = value; }
        }
    }

    /// <summary>
    /// The ResourceSetChangingEventHandler event handler is used to notify when the ResourceSet is being changed
    /// </summary>
    /// <param name="sender">The source of the event</param>
    /// <param name="eventArgs">The ResourceSetChangeEventArgs arguments passed to the event handler</param>
    /// <remarks>Use this event to propagate changes to the source of the resources</remarks>
    public delegate void ResourceSetChangingEventHandler(object sender, ResourceSetChangeEventArgs eventArgs);

    /// <summary>
    /// ResourceSetAction is the action which is performed on the ResourceSet
    /// </summary>
    public enum ResourceSetAction
    {
        /// <summary>
        /// A resource entry is being added to the ResourceSet
        /// </summary>
        Add,
        /// <summary>
        /// A resource entry is being changed
        /// </summary>
        Change,
        /// <summary>
        /// A resource entry is being deleted
        /// </summary>
        Delete
    };

    /// <summary>
    /// ResourceSetChangeEventArgs represent the arguments passed to a ResourceSetChangingEventHandler event handler
    /// </summary>
    public class ResourceSetChangeEventArgs : EventArgs
    {
        private ResourceSet resourceSet;
        private ResourceSetAction action;
        private string resourceKey;
        private object resourceValue;
        private string resourceComment;
        private string oldResourceKey;
        private object oldResourceValue;
        private string oldResourceComment;

        /// <summary>
        /// Constructs a ResourceSetChangeEventArgs object
        /// </summary>
        /// <param name="resourceSet">The ResourceSet which is being changed</param>
        /// <param name="action">The ResourceSetAction change which is being made</param>
        /// <param name="resourceKey">The new key of the resource entry</param>
        /// <param name="resourceValue">The new value of the resource entry</param>
        /// <param name="resourceComment">The new comment of the resource entry</param>
        /// <param name="oldResourceKey">The old key of the resource entry</param>
        /// <param name="oldResourceValue">The old value of the resource entry</param>
        /// <param name="oldResourceComment">The old comment of the resource entry</param>
        public ResourceSetChangeEventArgs(ResourceSet resourceSet, ResourceSetAction action,
            string resourceKey, object resourceValue, string resourceComment,
            string oldResourceKey, object oldResourceValue, string oldResourceComment)
        {
            this.resourceSet = resourceSet;
            this.action = action;
            this.resourceKey = resourceKey;
            this.resourceValue = resourceValue;
            this.resourceComment = resourceComment;
            this.oldResourceKey = oldResourceKey;
            this.oldResourceValue = oldResourceValue;
            this.oldResourceComment = oldResourceComment;
        }
        /// <summary>
        /// The ResourceSet which is being changed
        /// </summary>
        public ResourceSet ResourceSet
        {
            get { return resourceSet; }
            set { resourceSet = value; }
        }
        /// <summary>
        /// The ResourceSetAction change which is being made
        /// </summary>
        public ResourceSetAction Action
        {
            get { return action; }
            set { action = value; }
        }
        /// <summary>
        /// The new key of the resource entry
        /// </summary>
        public string ResourceKey
        {
            get { return resourceKey; }
            set { resourceKey = value; }
        }
        /// <summary>
        /// The new comment of the resource entry
        /// </summary>
        public string ResourceComment
        {
            get { return resourceComment; }
            set { resourceComment = value; }
        }
        /// <summary>
        /// The new value of the resource entry
        /// </summary>
        public object ResourceValue
        {
            get { return resourceValue; }
            set { resourceValue = value; }
        }
        /// <summary>
        /// The old key of the resource entry
        /// </summary>
        public string OldResourceKey
        {
            get { return oldResourceKey; }
            set { oldResourceKey = value; }
        }
        /// <summary>
        /// The old value of the resource entry
        /// </summary>
        public object OldResourceValue
        {
            get { return oldResourceValue; }
            set { oldResourceValue = value; }
        }
        /// <summary>
        /// The old comment of the resource entry
        /// </summary>
        public string OldResourceComment
        {
            get { return oldResourceComment; }
            set { oldResourceComment = value; }
        }
    }
    /// <summary>
    /// ListViewItemDataNodeInterpreter is a ListViewItem which represents a ResourcesDataNodeInterpreter object
    /// </summary>
    public class ListViewItemDataNodeInterpreter : ListViewItem
    {
        private ResourcesDataNodeInterpreter dataNodeInterpreter;

        /// <summary>
        /// Constructs a ListViewItemDataNodeInterpreter object
        /// </summary>
        /// <param name="text">The text of the ListViewItem</param>
        /// <param name="imageIndex">The image index of the ListViewItem</param>
        /// <param name="dataNodeInterpreter">The ResourcesDataNodeInterpreter which the ListViewItem represents</param>
        public ListViewItemDataNodeInterpreter(
            string text, int imageIndex, ResourcesDataNodeInterpreter dataNodeInterpreter):
            base(text, imageIndex)
        {
            this.dataNodeInterpreter = dataNodeInterpreter;
        }
        /// <summary>
        /// The ResourcesDataNodeInterpreter which the ListViewItem represents
        /// </summary>
        public ResourcesDataNodeInterpreter DataNodeInterpreter
        {
            get { return dataNodeInterpreter; }
            set { dataNodeInterpreter = value; }
        }
    }
}