// File           : BaseGlobalizationRule.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Globalization;
using System.Collections.Specialized;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
    /// <summary>
    /// BaseGlobalizationRule is an abstract base class for introspection globalization rules
    /// </summary>
    public abstract class BaseGlobalizationRule: BaseIntrospectionRule
    {
        /// <summary>
        /// Constructs a BaseGlobalizationRule object
        /// </summary>
        /// <param name="name">The name of the rule</param>
        protected BaseGlobalizationRule(string name)
            : base(name, "GlobalizationRules.RuleData", typeof(BaseGlobalizationRule).Assembly)
        {
        }
        /// <summary>
        /// TypeIsSubClassOf returns true if the given type is a equal to or is a sub class of a given type name
        /// </summary>
        /// <param name="type">The TypeNode representing the type</param>
        /// <param name="typeName">The string of the type to check for</param>
        /// <returns>True if the given type is a equal to or is a sub class of a given type name</returns>
		protected virtual bool TypeIsSubClassOf(TypeNode type, string typeName)
		{
			if (type.FullName == typeName)
				return true;
			else if (type.BaseType == null)
				return false;
			else
				return TypeIsSubClassOf(type.BaseType, typeName);
		}
        /// <summary>
        /// GetResource returns true if it can get the resource for the given TypeNode
        /// </summary>
        /// <param name="type">The TypeNode to get the resource for</param>
        /// <param name="resource">The out Resource parameter into which the resource is put</param>
        /// <returns>True if the resource can be retrieved for the given TypeNode</returns>
		protected virtual bool GetResource(TypeNode type, out Resource resource)
		{
			return GetResource(type, out resource, String.Empty);
		}
        /// <summary>
        /// GetResource returns true if it can get the resource for the given TypeNode
        /// </summary>
        /// <param name="type">The TypeNode to get the resource for</param>
        /// <param name="resource">The out Resource parameter into which the resource is put</param>
        /// <param name="resourceNameSuffix">The suffix of the resource name</param>
        /// <returns>True if the resource can be retrieved for the given TypeNode</returns>
        protected virtual bool GetResource(TypeNode type, out Resource resource, string resourceNameSuffix)
		{
			for(int resourceNumber = 0; resourceNumber < type.DeclaringModule.Resources.Length; resourceNumber++)
			{
				Resource testResource = type.DeclaringModule.Resources[resourceNumber];
				string testResourceName = testResource.Name.ToLower(CultureInfo.InvariantCulture);
				string typeResourceName = type.Name.Name.ToLower(CultureInfo.InvariantCulture) + 
					resourceNameSuffix.ToLower() + ".resources";
				string typeResourceFullName = type.FullName.ToLower(CultureInfo.InvariantCulture) + 
					resourceNameSuffix.ToLower() + ".resources";
				if ((testResourceName == typeResourceName) || (testResourceName == typeResourceFullName))
				{
					resource = testResource;
					return true;
				}
			}
			resource = new Resource();
			return false;
		}
        /// <summary>
        /// IsInstructionSpecificMethodCall returns true if the instruction is a call to a specific method
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="methodName">The method to test the instruction for</param>
        /// <returns>True if the instruction is a call to the given method</returns>
        protected bool IsInstructionSpecificMethodCall(Instruction instruction, string methodName)
        {
            return IsInstructionSpecificMethodCall(instruction, new string[] { methodName });
        }
        /// <summary>
        /// IsInstructionSpecificMethodCall returns true if the instruction is a call to a specific method
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="methodNames">An array of methods to test the instruction for</param>
        /// <returns>True if the instruction is a call to one of the given methods</returns>
        protected bool IsInstructionSpecificMethodCall(Instruction instruction, string[] methodNames)
        {
            if ((instruction.OpCode == OpCode.Callvirt || instruction.OpCode == OpCode.Call)
                && instruction.Value is Method)
            {
                Method instructionMethod = (Method)instruction.Value;
                foreach (string methodName in methodNames)
                {
                    if (instructionMethod.FullName.StartsWith(methodName + "("))
                        return true;
                }
                return false;
            }
            return false;
        }
        /// <summary>
        /// IsInstructionNewObject returns true if the instruction creates a new object of the given class name
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="className">The name of the class to compare the instruction with</param>
        /// <returns>True if the instruction creates a new object of the given class name</returns>
        protected bool IsInstructionNewObject(Instruction instruction, string className)
        {
            return IsInstructionNewObject(instruction, new string[] { className });
        }
        /// <summary>
        /// IsInstructionNewObject returns true if the instruction creates a new object of one of the given class names
        /// </summary>
        /// <param name="instruction">The instruction to test</param>
        /// <param name="classNames">An array of the names of the classes to compare the instruction with</param>
        /// <returns>True if the instruction creates a new object of one of the given class names</returns>
        protected bool IsInstructionNewObject(Instruction instruction, string[] classNames)
        {
            if (instruction.OpCode == OpCode.Newobj && instruction.Value is InstanceInitializer)
            {
                InstanceInitializer instructionInstanceInitializer = (InstanceInitializer)instruction.Value;
                foreach (string className in classNames)
                {
                    if (instructionInstanceInitializer.FullName.StartsWith(className + ".#ctor"))
                        return true;
                }
                return false;
            }
            return false;
        }
    }
}
