// File           : BaseTypeResourceRule.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections;
using System.Resources;
using System.IO;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
	/// <summary>
    /// BaseTypeResourceRule is an abstract base class for Type globalization introspection rules
	/// </summary>
	public abstract class BaseTypeResourceRule: BaseGlobalizationRule
	{
		private string typeName;

        /// <summary>
        /// Constructs a BaseTypeResourceRule object
        /// </summary>
        /// <param name="name">The name of the rule</param>
        /// <param name="typeName">The string of the Type to check for</param>
		public BaseTypeResourceRule(string name, string typeName): base(name)
		{
			this.typeName = typeName;
		}
        /// <summary>
        /// Check checks resources for adherence to the rule
        /// </summary>
        /// <param name="type">The TypeNode to check the resource for</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
		public override ProblemCollection Check(TypeNode type)
		{
			if (TypeIsSubClassOf(type, typeName))
			{
				Resource resource;
				if (GetResource(type, out resource))
					return CheckResource(type, resource);
			}
			return base.Check(type);
		}
        /// <summary>
        /// CheckResource checks a resource for adherence to the rule
        /// </summary>
        /// <param name="type">The TypeNode to check the resource for</param>
        /// <param name="resource">The resource to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public abstract ProblemCollection CheckResource(TypeNode type, Resource resource);

        /// <summary>
        /// ResourceToList reads the resource entries in a resource into an IList
        /// </summary>
        /// <param name="resource">The Resource containing the resource entries</param>
        /// <param name="list">The IList to be filled with DictionaryEntry object for the resource entries</param>
		protected virtual void ResourceToList(Resource resource, IList list)
		{
			ResourceReader reader = new ResourceReader(new System.IO.MemoryStream(resource.Data));
			try
			{
				IDictionaryEnumerator enumerator = reader.GetEnumerator();
				while (enumerator.MoveNext())
				{
					list.Add(new DictionaryEntry(enumerator.Key, enumerator.Value));
				}
			}
			finally
			{
				reader.Close();
			}
		}
	}
}
