// File           : CultureInfoMustNotBeConstructedFromLCID.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
	/// <summary>
    /// CultureInfoMustNotBeConstructedFromLCID ensures that RegionInfo objects are not constructed from an LCID
	/// </summary>
	public class CultureInfoMustNotBeConstructedFromLCID: BaseGlobalizationRule
	{
        /// <summary>
        /// Constructs a CultureInfoMustNotBeConstructedFromLCID object
        /// </summary>
		public CultureInfoMustNotBeConstructedFromLCID(): base("CultureInfoMustNotBeConstructedFromLCID")
		{
		}
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
		{
			Method method = member as Method;
			if (method == null)
				return null;

			// start at the second instruction because we need to check the previous instruction and
			// if there is no previous instruction then the instruction is not what we are looking for
			for(int instructionNumber = 1; instructionNumber < method.Instructions.Length; instructionNumber++)
			{
				Microsoft.Cci.Instruction instruction = method.Instructions[instructionNumber];
				// check to see if the instruction is either a CultureInfo/CultureInfoEx constructor
				// or a call to CultureInfoProvider.GetCultureInfo
				if (instruction.OpCode == OpCode.Newobj && instruction.Value is Microsoft.Cci.InstanceInitializer)
				{
					Microsoft.Cci.InstanceInitializer instanceInitializer = 
						(Microsoft.Cci.InstanceInitializer) instruction.Value;

					if (TypeIsSubClassOf(instanceInitializer.DeclaringType, "System.Globalization.CultureInfo")
						|| TypeIsSubClassOf(instanceInitializer.DeclaringType, "Globalization.CultureInfoEx"))
					{
						// check to see if the constructor's first parameter is an integer (i.e. an LCID)
						TypeNode[] typeNodes = instanceInitializer.GetParameterTypes();
						if (typeNodes.GetLength(0) > 0 && typeNodes[0].Name.Name == "Int32")
						{
							// check the previous instruction to see if the LCID parameter value is a literal integer value
							// (e.g. new CultureInfo(0x0000040A), for the Spanish alternative sort order)
							Microsoft.Cci.Instruction previousInstruction = method.Instructions[instructionNumber - 1];
							if (previousInstruction.OpCode != OpCode.Ldc_I4)
							{
								Resolution resolution = GetResolution(new string[] {method.Name.Name});
								Problems.Add(new Problem(resolution));
								return Problems;
							}							
						}
					}
				}
				else if (instruction.OpCode == OpCode.Call && instruction.Value is Microsoft.Cci.Method)
				{
					Microsoft.Cci.Method calledMethod = (Microsoft.Cci.Method) instruction.Value;

                    // if the method is CultureInfoProvider.GetCultureInfo or CultureInfo.GetCultureInfo
					if ((TypeIsSubClassOf(calledMethod.DeclaringType, "Globalization.CultureInfoProvider") 
						&& calledMethod.Name.Name == "GetCultureInfo") ||
                        (TypeIsSubClassOf(calledMethod.DeclaringType, "System.Globalization.CultureInfo") 
						&& calledMethod.Name.Name == "GetCultureInfo"))
					{
						// check to see if the constructor's first parameter is an integer (i.e. an LCID)
						TypeNode[] typeNodes = calledMethod.GetParameterTypes();
						if (typeNodes.GetLength(0) > 0 && typeNodes[0].Name.Name == "Int32")
						{
							// check the previous instruction to see if the LCID parameter value is a literal integer value
							// (e.g. new CultureInfo(0x0000040A), for the Spanish alternative sort order)
							Microsoft.Cci.Instruction previousInstruction = method.Instructions[instructionNumber - 1];
							if (previousInstruction.OpCode != OpCode.Ldc_I4)
							{
								Resolution resolution = GetResolution(new string[] {method.Name.Name});
								Problems.Add(new Problem(resolution));
								return Problems;
							}							
						}
					}
				}
			}
			return base.Check(member);
		}
	}
}
