// File           : LabelAutoSizeMustBeTrue.cs 
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.IO;
using System.Resources;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using Microsoft.Cci;
using Microsoft.FxCop.Sdk;
using Microsoft.FxCop.Sdk.Introspection;

namespace GlobalizationRules
{
	/// <summary>
	/// LabelAutoSizeMustBeTrue checks that Label.AutoSize is true
	/// </summary>
	public class LabelAutoSizeMustBeTrue: BaseTypeResourceRule
	{
        /// <summary>
        /// Constructs a LabelAutoSizeMustBeTrue object
        /// </summary>
		public LabelAutoSizeMustBeTrue(): base("LabelAutoSizeMustBeTrue", "System.Windows.Forms.Form")
		{
		}
        /// <summary>
        /// CheckResource checks a resource for adherence to the rule
        /// </summary>
        /// <param name="type">The TypeNode to check the resource for</param>
        /// <param name="resource">The resource to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection CheckResource(TypeNode type, Resource resource)
		{
			StringCollection badControls = new StringCollection();
			ArrayList list = new ArrayList();
			ResourceToList(resource, list);
			foreach(DictionaryEntry dictionaryEntry in list)
			{
				string key = (string) dictionaryEntry.Key;
				if (key.StartsWith(">>") && key.EndsWith(".Type"))
				{
					string entryType = (String) dictionaryEntry.Value;
					if (entryType.StartsWith("System.Windows.Forms.Label, "))
					{
						// This is a label. Now check its AutoSize property.
						// If the key is ">>label1.Type" then the control name is "label1".
						string controlName = key.Substring(2, key.Length - 7);
						int labelAutoSizeIndex = IndexOfDictionaryEntry(list, controlName + ".AutoSize");
						if (labelAutoSizeIndex == -1 || 
							! (bool) ((DictionaryEntry) list[labelAutoSizeIndex]).Value)
							badControls.Add(controlName);
					}
				}
			}
			if (badControls.Count == 0)
				return null;
			else
			{
				StringBuilder badControlNames = new StringBuilder(badControls[0]);
				for(int badControlNumber = 1; badControlNumber < badControls.Count; badControlNumber++)
				{
					badControlNames.Append(", " + badControls[badControlNumber]);
				}
				Resolution resolution = GetResolution(new string[] {type.Name.Name, badControlNames.ToString()});
				Problems.Add(new Problem(resolution));
				return Problems;
			}
		}
        /// <summary>
        /// IndexOfDictionaryEntry returns the index of a given key name in an IList of DictionaryEntry objects
        /// </summary>
        /// <param name="list">The IList of DictionaryEntry objects</param>
        /// <param name="keyName">The resource entry name to return the index for</param>
        /// <returns>The index of a given key name in an IList of DictionaryEntry objects</returns>
		protected virtual int IndexOfDictionaryEntry(IList list, string keyName)
		{
			int entryNumber = 0;
			foreach(DictionaryEntry dictionaryEntry in list)
			{
				if (dictionaryEntry.Key.ToString() == keyName)
					return entryNumber;
				entryNumber++;
			}
			return -1;
		}
	}
}
