// File           : DoNotUseLiteralControlsForText.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : March 2008
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Specialized;
using Microsoft.FxCop.Sdk;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace GlobalizationRulesForASPGeneratedAssemblies
{
    /// <summary>
    /// DoNotUseLiteralControlsForText checks that literal text (other than HTML and white space) is not used on a page
    /// </summary>
    public class DoNotUseLiteralControlsForText: BaseGlobalizationRuleForASPGeneratedAssemblies
    {
        private StringCollection literalStrings;

        /// <summary>
        /// Constructs a DoNotUseLiteralControlsForText object
        /// </summary>
        public DoNotUseLiteralControlsForText(): base("DoNotUseLiteralControlsForText")
        {
        }
        /// <summary>
        /// Check checks types in an assembly
        /// </summary>
        /// <param name="type">The TypeNode being checked</param>
        /// <returns>A ProblemCollection of Problems</returns>
        public override ProblemCollection Check(TypeNode type)
        {
            List<Method> methods = GetBuildMethods(type);
            if (methods.Count > 0)
            {
                foreach (Method method in methods)
                {
                    // (don't count any method that returns an HtmlTitle object because
                    // the localization process for this is different and don't count any method
                    // that returns a localizable WebControl such as a Label)
                    if (method.ReturnType.FullName != "System.Web.UI.HtmlControls.HtmlTitle" &&
                        ! IsLocalizableWebControl(method))
                    {
                        literalStrings = new StringCollection();
                        VisitMethod(method);
                        if (literalStrings.Count > 0)
                        {
                            foreach (string literalString in literalStrings)
                            {
                                Resolution resolution = GetResolution(new string[] { GetWebFormName(type), literalString });
                                Problems.Add(new Problem(resolution));
                            }
                        }
                    }
                }
                return Problems;
            }
            return base.Check(type);
        }
        private bool IsLocalizableWebControl(Method method)
        {
            if (method.ReturnType.FullName.StartsWith("System.Web.UI.WebControls."))
            {
                List<string> localizableWebControls = new List<string>(new string[] {
                    "Label", "HyperLink", "CompareValidator", "CustomValidator", "RangeValidator",
                    "RequiredFieldValidator", "RegularExpressionValidator", "LinkButton" });

                if (localizableWebControls.IndexOf(method.ReturnType.Name.Name) != -1)
                    return true;
            }
            return false;
        }
        /// <summary>
        /// VisitConstruct is called for each construction of a new object
        /// </summary>
        /// <param name="cons">A Construct object</param>
        public override void VisitConstruct(Construct cons)
        {
            if (ProcessingRule)
            {
                if (cons != null)
                {
                    MemberBinding memberBinding = cons.Constructor as MemberBinding;
                    if (memberBinding != null)
                    {
                        InstanceInitializer instanceInitializer =
                            memberBinding.BoundMember as InstanceInitializer;
                        if (instanceInitializer != null &&
                            instanceInitializer.DeclaringType.FullName == "System.Web.UI.LiteralControl" &&
                            cons.Operands.Count > 0 &&
                            cons.Operands[0].Type.FullName == "System.String")
                        {
                            Literal literal = cons.Operands[0] as Literal;
                            if (literal != null)
                            {
                                // the LiteralControl constructor has been called with a string parameter
                                string literalText = ExtractLiteralText(literal.Value.ToString());
                                if (literalText.Length > 0)
                                    literalStrings.Add(literalText);
                            }
                        }
                    }
                }
            }
        }
        /// <summary>
        /// ExtractLiteralText extracts the literal text from a stream of text containing literal text and markup
        /// </summary>
        /// <param name="text">The string containing literal text and markup</param>
        /// <returns>A string that contains literal text only with no markup</returns>
        protected virtual string ExtractLiteralText(string text)
        {
            // remove the JavaScript first (if any)
            text = StripScript(text);
            // remove the HTML comments (if any)
            text = StripComments(text);
            return StripHtml(text.Replace("\t", String.Empty).Replace("\n", String.Empty).
                Replace("\r", String.Empty).Replace("&nbsp;", String.Empty)).Trim();
        }
        /// <summary>
        /// StripScript removes JavaScript from a string
        /// </summary>
        /// <param name="script">A string containing JavaScript</param>
        /// <returns>The string without the JavaScript</returns>
        protected virtual string StripScript(string script)
        {
            string scriptStartControlPattern = @"[" + Regex.Escape("<") + @"][s][c][r][i][p][t]";
            string scriptInnerControlPattern = @"[\w\s: ();" + Regex.Escape("<") + Regex.Escape(">")
                + Regex.Escape("/") + Regex.Escape("=") + Regex.Escape(@"""") + @"]*";
            string scriptEndControlPattern = "[s][c][r][i][p][t]" + @"[" + Regex.Escape(">") + @"]";
            string scriptControlPattern = scriptStartControlPattern + scriptInnerControlPattern + scriptEndControlPattern;
            Regex regex = new Regex(scriptControlPattern, RegexOptions.IgnoreCase);
            return regex.Replace(script, String.Empty);
        }
        /// <summary>
        /// StripComments removes HTML comments from a string
        /// </summary>
        /// <param name="script">A string containing HTML</param>
        /// <returns>The string without the HTML comments</returns>
        protected virtual string StripComments(string script)
        {
            string commentStartControlPattern = @"[" + Regex.Escape("<") + @"][!][-][-]";
            string commentInnerControlPattern = @"[\w\s: ();" + Regex.Escape("<") + Regex.Escape(">")
                + Regex.Escape(".") + Regex.Escape("/") + Regex.Escape("=") + Regex.Escape(@"""") + @"]*";
            string commentEndControlPattern = "[-][-]" + @"[" + Regex.Escape(">") + @"]";
            string scriptControlPattern = commentStartControlPattern + commentInnerControlPattern + commentEndControlPattern;
            Regex regex = new Regex(scriptControlPattern, RegexOptions.IgnoreCase);
            return regex.Replace(script, String.Empty);
        }
        /// <summary>
        /// StripHtml strips the HTML from a string
        /// </summary>
        /// <param name="text">A string containing HTML</param>
        /// <returns>A string that does not contain HTML</returns>
        protected virtual string StripHtml(string text)
        {
            // TODO: Rewrite this method to be more efficient and more intelligent
            int indexOpeningTag = text.IndexOf("<");
            while (indexOpeningTag > -1)
            {
                int indexClosingTag = text.IndexOf(">", indexOpeningTag);
                if (indexClosingTag == -1)
                    break;
                if (indexOpeningTag > 0)
                    text = text.Substring(0, indexOpeningTag) + text.Substring(indexClosingTag + 1);
                else
                    text = text.Substring(indexClosingTag + 1);

                indexOpeningTag = text.IndexOf("<");
            }
            return text;
        }
    }
}
