// File           : DoNotUseLiteralStrings.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Data;
using System.Xml;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using Microsoft.FxCop.Sdk;
using Internationalization.Common;
using System.Reflection;

namespace GlobalizationRules
{
	/// <summary>
	/// DoNotUseLiteralStrings checks that literal strings are not used in code
	/// </summary>
	public class DoNotUseLiteralStrings: BaseGlobalizationRule
	{
		private string ignoredLiteralStringsFilename;
		private DataSet ignoredLiteralStringsDataSet;
		private ArrayList literalStrings;
		private StringCollection assemblyPaths;

        /// <summary>
        /// Constructs a DoNotUseLiteralStrings object
        /// </summary>
		public DoNotUseLiteralStrings(): base("DoNotUseLiteralStrings")
		{
		}
        /// <summary>
        /// BeforeAnalysis is called before analysis of an assembly begins
        /// </summary>
		public override void BeforeAnalysis()
		{
			assemblyPaths = new StringCollection();
			literalStrings = new ArrayList();
			InitializeIgnoredLiteralStringsDataSet();
			base.BeforeAnalysis();
		}
        /// <summary>
        /// AfterAnalysis is called after analysis of an assembly completes
        /// </summary>
		public override void AfterAnalysis()
		{
			base.AfterAnalysis();
			WriteLiteralStrings();
		}
        /// <summary>
        /// InitializeIgnoredLiteralStringsDataSet initializes the internal list of ignored literal strings
        /// </summary>
		protected virtual void InitializeIgnoredLiteralStringsDataSet()
		{
			ignoredLiteralStringsFilename = FileFinder.Find("IgnoredLiteralStrings.xml");
			if (ignoredLiteralStringsFilename == String.Empty)
				ignoredLiteralStringsDataSet = null;
			else
			{
				ignoredLiteralStringsDataSet = new DataSet();
				ignoredLiteralStringsDataSet.ReadXml(ignoredLiteralStringsFilename);
			}
		}
        /// <summary>
        /// WriteLiteralStrings writes the list of literal strings which are not ignored back to LiteralStrings.xml
        /// </summary>
		protected virtual void WriteLiteralStrings()
		{
			string literalStringsFilename;
			if (ignoredLiteralStringsFilename == String.Empty)
				// put the literal strings file in a directory which is common to all of the assemblies analyzed
				literalStringsFilename = 
					GetCommonAssemblyPath() + Path.DirectorySeparatorChar + "LiteralStrings.xml";
			else
				// put the literal strings file back in the same place that we found the ignored literal strings
				// file
				literalStringsFilename = Path.GetDirectoryName(ignoredLiteralStringsFilename) +
					Path.DirectorySeparatorChar + "LiteralStrings.xml";

			XmlTextWriter xmlTextWriter = new XmlTextWriter(literalStringsFilename, System.Text.Encoding.UTF8);
			try
			{
				xmlTextWriter.WriteStartElement("LiteralStrings");
				foreach(LiteralStringInformation literalString in literalStrings)
				{
					xmlTextWriter.WriteStartElement("LiteralString");
					xmlTextWriter.WriteAttributeString("Method", literalString.MethodName);
					xmlTextWriter.WriteAttributeString("String", literalString.LiteralString);
					xmlTextWriter.WriteEndElement();
				}
				xmlTextWriter.WriteEndElement();
			}
			finally
			{
				xmlTextWriter.Close();
			}
		}
        /// <summary>
        /// Check checks modules for adherence to the rule
        /// </summary>
        /// <param name="module">The Module to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(ModuleNode module)
		{
			// collect all of the unique assembly paths
			string path = module.ContainingAssembly.Directory.ToString().ToUpper();
			if (assemblyPaths.IndexOf(path) == -1)
				// we haven't encountered this path before
				assemblyPaths.Add(path);
			return base.Check(module);
		}
        /// <summary>
        /// GetCommonAssemblyPath gets the lowest path which is common to all assemblies which were analyzed
        /// </summary>
        /// <returns>The lowest path which is common to all assemblies which were analyzed</returns>
		protected virtual string GetCommonAssemblyPath()
		{
			if (assemblyPaths.Count == 0)
				return String.Empty;

			string commonPath = assemblyPaths[0];
			for(int assemblyPathNumber = 1; assemblyPathNumber < assemblyPaths.Count; assemblyPathNumber++)
			{
				string assemblyPath = assemblyPaths[assemblyPathNumber];
				commonPath = GetCommonAncestor(commonPath, assemblyPath);
			}
			return commonPath;
		}
        /// <summary>
        /// GetCommonAncestor gets the common ancestor of two paths
        /// </summary>
        /// <param name="path1">The first path</param>
        /// <param name="path2">The second path</param>
        /// <returns>The common ancestor of two paths</returns>
        /// <example>
        /// path1                               path2                        Common Ancestor
        /// "c:\myproject1\mainapplication"     "c:\myproject1\commonlib"    "c:\myproject1"
        /// "c:\myproject1\mainapplication"     "c:\myproject1"              "c:\myproject1"
        /// "c:\myproject1"                     "c:\myproject1\commonlib"    "c:\myproject1"
        /// </example>
		protected virtual string GetCommonAncestor(string path1, string path2)
		{
			while (path1 != String.Empty && path1 != path2.Substring(0, Math.Min(path1.Length, path2.Length)))
			{
				int directorySeparatorIndex = path1.LastIndexOf(Path.DirectorySeparatorChar);
				if (directorySeparatorIndex == -1)
					return path1;

				path1 = path1.Substring(0, directorySeparatorIndex);
			}
			return path1;
		}
        /// <summary>
        /// Check checks members for adherence to the rule
        /// </summary>
        /// <param name="member">The Member to check</param>
        /// <returns>The ProblemCollection of problems found by this method</returns>
        public override ProblemCollection Check(Member member)
		{
			// Don't bother checking Form.InitializeComponent because this code is automatically generated
			// and if the form is localized then it will already have been dealt with and if it isn't
			// localized then that is a different problem.
			Method method = member as Method;
			if (method == null)
				return null;

			if ( ! (method.Name.Name == "InitializeComponent" && 
				TypeIsSubClassOf(method.DeclaringType, "System.Windows.Forms.Form")))
			{
				for(int instructionNumber = 0; instructionNumber < method.Instructions.Count; instructionNumber++)
				{
					Instruction instruction = method.Instructions[instructionNumber];
					if (instruction.OpCode == OpCode.Ldstr && instructionNumber + 1 < method.Instructions.Count)
					{
						// This instruction is a "load string" instruction and it isn't the last instruction 
						// in the method.
						string loadString = instruction.Value.ToString();
						if (IsSuspectString(method, loadString))
						{
							Instruction nextInstruction = method.Instructions[instructionNumber + 1];
							if (! (
								IsInstructionNewObject(nextInstruction, "System.Exception") ||
								IsInstructionNewObject(nextInstruction, "System.Resources.ResourceManager") ||
								IsInstructionNewObject(nextInstruction, "Internationalization.Resources.ResourceManagerProvider") ||
								IsInstructionSpecificMethodCall(nextInstruction, "System.DateTime.ToString") ||
								IsInstructionSpecificMethodCall(nextInstruction, "System.Diagnostics.Debug.WriteLine") ||
								IsInstructionSpecificMethodCall(nextInstruction, "System.Diagnostics.Trace.WriteLine")))
							{
								// The string is a literal string and it is suspiciously like a localizable
								// string and it is not already in the list of ignored strings and it is
								// not passed to one of the exempt constructors or methods.
								literalStrings.Add(new LiteralStringInformation(
									method.DeclaringType.FullName + "." + method.Name.Name, loadString));

								Resolution resolution = GetResolution(new string[] {loadString});
								Problems.Add(new Problem(resolution));
								return Problems;
							}
						}
					}
				}
			}
			return base.Check(member);
		}
		private new bool IsInstructionNewObject(Instruction instruction, string className)
		{
            // consider replacing this instance of IsInstructionNewObject with the base class's version
            if (instruction.OpCode == OpCode.Newobj && instruction.Value is InstanceInitializer)
			{
				InstanceInitializer instructionInitializer = (InstanceInitializer) instruction.Value;
				return TypeIsSubClassOf(instructionInitializer.DeclaringType, className);
			}
			return false;
		}
		private static string StripControlChars(string stringFound)
		{
			stringFound = stringFound.Replace(((char) 13).ToString(), "");
			stringFound = stringFound.Replace(((char) 10).ToString(), "");
			return stringFound;
		}
        /// <summary>
        /// IsSuspectString returns true if the string contains more than one character and is not in the
        /// list of ignored strings
        /// </summary>
        /// <param name="method">The Method in which the string was found</param>
        /// <param name="stringFound">The potentially suspect string</param>
        /// <returns>True if the string contains more than one character and is not in the list of ignored strings
        /// </returns>
		protected virtual bool IsSuspectString(Method method, string stringFound)
		{
			if (! StringContainsMoreThanOneCharacter(stringFound))
				// the string doesn't contain more than one character so it is not suspect
				return false;

			if (ignoredLiteralStringsDataSet != null)
				return ! IsStringIgnored(method, stringFound);

			return true;
		}
        /// <summary>
        /// IsStringIgnored returns true if the string is in the list of ignored strings
        /// </summary>
        /// <param name="method">The Method in which the string was found</param>
        /// <param name="stringFound">The potentially suspect string</param>
        /// <returns>True if the string is in the list of ignored strings</returns>
        protected virtual bool IsStringIgnored(Method method, string stringFound)
		{
			string methodName = method.DeclaringType.FullName + "." + method.Name.Name;
			try
			{
				DataRow[] dataRows = ignoredLiteralStringsDataSet.Tables["IgnoredLiteralStrings"].Select(
					"MethodName='" + methodName + "' and LiteralString='" + stringFound.Replace("'", "''") + "'");
				return dataRows.GetLength(0) > 0;
			}
			catch
			{
				// this is a complete bodge just as a temporary measure
				return false;
			}
		}
		private static bool StringContainsMoreThanOneCharacter(string testString)
		{
			testString = testString.Replace(" ", "");
			if (testString.Length == 0)
				return false;

			int numberOfCharacters = 0;
			foreach(char chr in testString)
			{
				if (Char.IsLetter(chr))
				{
					numberOfCharacters++;
					if (numberOfCharacters > 1)
						return true;
				}
			}
			return false;
		}
	}
    /// <summary>
    /// LiteralStringInformation maintains information about a literal string
    /// </summary>
	public class LiteralStringInformation
	{
		private string methodName;
		private string literalString;
		/// <summary>
        /// Constructs a LiteralStringInformation object
		/// </summary>
		/// <param name="methodName">The name of the method in which the string was found</param>
		/// <param name="literalString">The literal string which was found</param>
		public LiteralStringInformation(string methodName, string literalString)
		{
			this.methodName = methodName;
			this.literalString = literalString;
		}
		/// <summary>
		/// MethodName is the name of the method which contains the literal string
		/// </summary>
		public string MethodName
		{
			get {return methodName;}
			set {methodName = value;}
		}
		/// <summary>
		/// LiteralString is the literal string
		/// </summary>
		public string LiteralString
		{
			get {return literalString;}
			set {literalString = value;}
		}
	}
}
