// File           : InvokeWinRes.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Globalization;
using System.Collections;
using System.Collections.Specialized;
using System.Resources;
using System.Text;
using Microsoft.Win32;
using Internationalization.ResourceAdministration;

namespace Internationalization.WindowsForms
{
	/// <summary>
	/// InvokeWinRes is a class ofr invoking WinRes (The Windows Resource Localization Editor)
	/// </summary>
	public class InvokeWinRes
	{
        private static bool enabled = true;
        private static string resourceExtension = ".resx";
        private static string resourcePath = @"resources\";
        private static Keys invokeKey = Keys.F10;
        private static string winResPath;
        private static IResourcesGovernor resourcesGovernor;

        /// <summary>
        /// Enabled specifies whether InvokeWinRes is enabled
        /// </summary>
        /// <remarks>By default Enabled is true. Set Enabled to false to prevent Invoke from starting WinRes
        /// </remarks>
        public static bool Enabled
        {
            get {return enabled;}
            set {enabled = value;}
        }
        /// <summary>
        /// ResourceExtension is the extension of the resource files (with the leading period)
        /// </summary>
        /// <remarks>The default is ".resx". Set ResourceExtension to ".resources" to use WinRes with
        /// .resources files.</remarks>
        public static string ResourceExtension
        {
            get {return resourceExtension;}
            set {resourceExtension = value;}
        }
        /// <summary>
        /// ResourcePath is the path from the executable assembly to the source of the resources
        /// </summary>
        public static string ResourcePath
        {
            get {return resourcePath;}
            set {resourcePath = value;}
        }
        /// <summary>
        /// InvokeKey is the key which is designated as the key to invoke WinRes
        /// </summary>
        /// <remarks>The default is F10</remarks>
        public static Keys InvokeKey
        {
            get {return invokeKey;}
            set {invokeKey = value;}
        }
        /// <summary>
        /// ResourcesGovernor is the IResourcesGovernor used to read resources from their original source
        /// </summary>
        /// <remarks>It is only necessary to set ResourcesGovernor if the source of the resources is not .resx
        /// or WinRes 1.1 is being used</remarks>
        public static IResourcesGovernor ResourcesGovernor
        {
            get {return resourcesGovernor;}
            set {resourcesGovernor = value;}
        }

        /// <summary>
        /// GetFrameworkSdkPath gets the file path to the .NET Framework SDK
        /// </summary>
        /// <returns>The file path to the .NET Framework SDK (or null if the SDK is not installed)</returns>
        protected static string GetFrameworkSdkPath()
        {
            string sourcesKeyName = @"Software\Microsoft\.NETFramework";
            RegistryKey frameworkKey = Registry.LocalMachine.OpenSubKey(sourcesKeyName);

            if (frameworkKey == null)
                return null;

            string frameworkVersion = System.Environment.Version.Major.ToString() + "." + System.Environment.Version.Minor.ToString();
            object sdkValue = frameworkKey.GetValue(@"sdkInstallRootv" + frameworkVersion);
            if (sdkValue == null)
                return null;

            return sdkValue.ToString();
        }
        /// <summary>
        /// WinResPath is the full file path to WinRes.exe (including the file name)
        /// </summary>
        public static string WinResPath
        {
            get
            {
                if (winResPath == null)
                {
                    // look first for WinRes in the application's path
                    // (WinRes must be in the same location as the application's
                    // assemblies if the resx files use Form Inheritance)
                    string applicationPath = Path.GetDirectoryName(Application.ExecutablePath);
                    if (File.Exists(applicationPath + Path.DirectorySeparatorChar + "WinRes.exe"))
                        winResPath = applicationPath;
                    else
                    {
                        string frameworkSdkPath = GetFrameworkSdkPath();
                        if (frameworkSdkPath != null && frameworkSdkPath != String.Empty)
                            winResPath = frameworkSdkPath + "bin";
                    }
                }
                return winResPath;
            }
            set {winResPath = value;}
        }
        /// <summary>
        /// CheckInvoke checks whether the keystroke matches the InvokeKeystroke and whether
        /// InvokeWinRes is enabled and if it is invokes WinRes
        /// </summary>
        /// <param name="key">The key pressed</param>
        /// <param name="form">The form on which the key was pressed</param>
        public static void CheckInvoke(Keys key, Form form)
        {
            CheckInvoke(key, form.GetType().Name);
        }
        /// <summary>
        /// CheckInvoke checks whether the keystroke matches the InvokeKeystroke and whether
        /// InvokeWinRes is enabled and if it is invokes WinRes
        /// </summary>
        /// <param name="key">The key pressed</param>
        /// <param name="baseName">The base name of the form resource (e.g. "Form1")</param>
        public static void CheckInvoke(Keys key, string baseName)
        {
            if (key == invokeKey)
            {
                Invoke(baseName);
            }
        }
        /// <summary>
        /// Invoke invokes WinRes for the given base name if InvokeWinRes.Enabled is true
        /// </summary>
        /// <param name="baseName">The base name of the form resource (e.g. "Form1")</param>
        public static void Invoke(string baseName)
        {
            if (enabled)
            {
                string winResPath = WinResPath;
                if (winResPath == null || winResPath == String.Empty)
                    // do not localize this text because it is intended to be shown to the translator
                    MessageBox.Show(InvokeWinResResources.WinResPathNotFound);
                else
                {
                    string winResProgram = winResPath + @"\WinRes.exe";

                    if (! File.Exists(winResProgram))
                        // do not localize this text because it is intended to be shown to the translator
                        MessageBox.Show(String.Format(InvokeWinResResources.WinResNotFound, winResProgram));
                    else
                    {
                        if (resourcesGovernor != null)
                            ExportResources(baseName);

                        bool isFileFound = false;
                        CultureInfo cultureInfo = Thread.CurrentThread.CurrentUICulture;
                        string resxFileName = resourcePath + baseName + "." + cultureInfo.Name + resourceExtension;
                        StringBuilder resxFileNamesBuilder = new StringBuilder(resxFileName);
                        while (true)
                        {
                            if (File.Exists(resxFileName))
                            {
                                Process.Start(winResProgram, resxFileName);
                                isFileFound = true;
                                break;
                            }

                            if (cultureInfo == cultureInfo.Parent)
                                break;

                            cultureInfo = cultureInfo.Parent;
                            if (cultureInfo.Equals(CultureInfo.InvariantCulture))
                                resxFileName = resourcePath + baseName + resourceExtension;
                            else
                                resxFileName = resourcePath + baseName + "." + cultureInfo.Name + resourceExtension;

                            resxFileNamesBuilder.Append(", " + resxFileName);
                        }
                        if (! isFileFound)
                        {
                            // do not localize this text because it is intended to be shown to the translator
                            MessageBox.Show(String.Format(InvokeWinResResources.NoFilesToTranslate, resxFileNamesBuilder.ToString()));
                        }                
                    }
                }
            }
        }
        /// <summary>
        /// ExportResources exports resources to .resx files which are comsumable by WinRes
        /// </summary>
        /// <param name="baseName">The base name of the form resource (e.g. "Form1")</param>
        /// <remarks>It is necessary to export to .resx files if either the original source is not .resx
        /// or WinRes 1.1 is being used and the .resx files need to be flattened (i.e. converted from
        /// Visual Studio File Mode to Single File Mode)</remarks>
        protected static void ExportResources(string baseName)
        {
            if (resourcesGovernor != null)
            {
                IResourcesGovernor outputResourcesGovernor = new ResXResourcesGovernor(resourcePath);
                CultureInfo cultureInfo = Thread.CurrentThread.CurrentUICulture;
                ArrayList resourceSets = new ArrayList();
                ArrayList cultureInfos = new ArrayList();
                while (true)
                {
                    IResourceGovernor inputResourceGovernor  = resourcesGovernor.GetResourceGovernor(baseName, cultureInfo);
                    ResourceSet resourceSet = inputResourceGovernor.ReadResourceSet();
                    resourceSets.Add(resourceSet);
                    cultureInfos.Add(cultureInfo);

                    if (cultureInfo == cultureInfo.Parent)
                        break;

                    cultureInfo = cultureInfo.Parent;
                }
                // walk the lists starting with the invariant culture and getting more specific
                // (e.g. Form1.resx then Form1.fr.resx and then Form1.fr-FR.resx)
                for(int cultureInfoNumber = cultureInfos.Count - 1; cultureInfoNumber >= 0; cultureInfoNumber--)
                {
                    ResourceSet resourceSet = (ResourceSet) resourceSets[cultureInfoNumber];
                    cultureInfo = (CultureInfo) cultureInfos[cultureInfoNumber];
                    if (resourceSet != null)
                    {
                        IResourceGovernor outputResourceGovernor = outputResourcesGovernor.GetResourceGovernor(baseName, cultureInfo);
                        if (System.Environment.Version.Major == 1 && cultureInfoNumber != cultureInfos.Count - 1)
                        {
                            // WinRes 1.1 is being used. WinRes 1.1 only supports "Single File Mode".
                            // The resx file has to be "flattened" before WinRes can use it.
                            ResourceSet parentResourceSet = (ResourceSet) resourceSets[cultureInfoNumber + 1];
                            outputResourceGovernor.AddResourceSet(resourceSet, parentResourceSet);
                        }
                        outputResourceGovernor.WriteResourceSet(resourceSet);
                    }
                }
            }
        }
	}
}
