// File           : Resources.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Data;
using System.IO;
using System.Resources;
using System.Reflection;
using Internationalization.ResourceAdministration;

namespace ResourceTranslationManager
{
    /// <summary>
    /// ResourcesModifiedEventHandler is an event handler which is called when resources are modified
    /// </summary>
    /// <param name="sender">The source of the event</param>
    /// <param name="e">An EventArgs that contains no event data</param>
    public delegate void ResourcesModifiedEventHandler(object sender, EventArgs e);

    /// <summary>
    /// Resources represents the invariant resource and the corresponding target language resource for a single
    /// language
    /// </summary>
    public class Resources
    {
        private IResourceGovernor cultureResourceGovernor;
        private IResourceGovernor invariantResourceGovernor;
        private ResourceSet cultureResourceSet;
        private Hashtable cultureResourceTable;
        private DataTable cultureDataTable;
        private string markedAsTranslatedPath;
        private bool isLoaded = false;

        /// <summary>
        /// ResourcesModified is an event handler which is called when resources are modified
        /// </summary>
        public event ResourcesModifiedEventHandler ResourcesModified;

        /// <summary>
        /// Constructs a Resources object
        /// </summary>
        /// <param name="cultureResourceGovernor">The IResourceGovernor of the target language resource</param>
        /// <param name="invariantResourceGovernor">The IResourceGovernor of the invariant resource</param>
        /// <param name="markedAsTranslatedPath">The path to the files which specify whether resources have been
        /// marked as translated</param>
        public Resources(IResourceGovernor cultureResourceGovernor, IResourceGovernor invariantResourceGovernor,
            string markedAsTranslatedPath)
        {
            this.cultureResourceGovernor = cultureResourceGovernor;
            this.invariantResourceGovernor = invariantResourceGovernor;
            if (markedAsTranslatedPath != null)
            {
                if (markedAsTranslatedPath.EndsWith(Path.DirectorySeparatorChar.ToString()))
                    this.markedAsTranslatedPath = markedAsTranslatedPath;
                else
                    this.markedAsTranslatedPath = markedAsTranslatedPath + Path.DirectorySeparatorChar;
            }
        }
        /// <summary>
        /// CultureResourceGovernor is the IResourceGovernor of the target language resource
        /// </summary>
        public IResourceGovernor CultureResourceGovernor
        {
            get { return cultureResourceGovernor; }
        }

        /// <summary>
        /// InvariantResourceGovernor is the IResourceGovernor of the invariant resource
        /// </summary>
        public IResourceGovernor InvariantResourceGovernor
        {
            get { return invariantResourceGovernor; }
        }
        /// <summary>
        /// CultureDataTable is the DataTable of combined invariant and target language resource entries
        /// </summary>
        public DataTable CultureDataTable
        {
            get { return cultureDataTable; }
        }
        /// <summary>
        /// HasChanges is true if changes have been made to the resource entries
        /// </summary>
        public bool HasChanges
        {
            get { return cultureDataTable.DataSet.HasChanges(); }
        }
        /// <summary>
        /// IsLoaded is true if Load has been called
        /// </summary>
        public bool IsLoaded
        {
            get { return isLoaded; }
        }
        /// <summary>
        /// GetTranslatedDataSetFilename gets the filename of the corresponding "marked as translated" file
        /// </summary>
        /// <returns>The filename of the corresponding "marked as translated" file</returns>
        protected virtual string GetTranslatedDataSetFilename()
        {
            if (markedAsTranslatedPath == null)
                return null;
            else
                return markedAsTranslatedPath + cultureResourceGovernor.BaseName +
                    "." + cultureResourceGovernor.CultureInfo.Name + ".xml";
        }
        /// <summary>
        /// Loads loads the invariant and target language resource into CultureDataTable
        /// </summary>
        public void Load()
        {
            DataSet oldDataSet = null;
            string oldDataSetFilename = GetTranslatedDataSetFilename();

            if (oldDataSetFilename != null)
            {
                if (File.Exists(oldDataSetFilename))
                {
                    // there is an old copy of the DataSet
                    // use this to load the "MarkedAsTranslated" settings (only)
                    oldDataSet = new DataSet();
                    oldDataSet.ReadXml(oldDataSetFilename);
                }
            }
            SortedList invariantResources = ResourceSetToSortedList(invariantResourceGovernor.ReadResourceSet());
            cultureResourceSet = cultureResourceGovernor.ReadResourceSet();
            cultureResourceTable = GetResourceSetTable(cultureResourceSet);
            SortedList cultureResources = ResourceSetToSortedList(cultureResourceSet);

            cultureDataTable = CreateDataTable();
            foreach (DictionaryEntry entry in invariantResources)
            {
                ResourcesDataNodeInterpreter invariantDataNodeInterpreter =
                    new ResourcesDataNodeInterpreter(entry.Key.ToString(), entry.Value);
                if (Internationalization.Common.ResourceUtilities.IsLocalizable(
                    invariantDataNodeInterpreter.Name, invariantDataNodeInterpreter.Value))
                {
                    // the entry is internationalizable
                    int index = cultureResources.IndexOfKey(entry.Key);
                    if (index != -1)
                    {
                        object cultureValue = cultureResources.GetByIndex(index);
                        ResourcesDataNodeInterpreter cultureDataNodeInterpreter =
                            new ResourcesDataNodeInterpreter(invariantDataNodeInterpreter.Name, cultureValue);
                        if (cultureDataNodeInterpreter.Value != null &&
                            cultureDataNodeInterpreter.Value.ToString() != String.Empty)
                        {
                            // the entry in the invariant resources has a matching entry in the culture resources
                            // and the culture value is not null or empty (this is normal)

                            bool markedAsTranslated =
                                GetMarkedAsTranslated(invariantDataNodeInterpreter.Name, oldDataSet);

                            AddCultureRow(entry.Key.ToString(), invariantDataNodeInterpreter.Value.ToString(),
                                cultureDataNodeInterpreter.Value.ToString(), markedAsTranslated,
                                invariantDataNodeInterpreter.Comment);
                        }
                    }
                }
            }
            cultureDataTable.RowChanged += new DataRowChangeEventHandler(dataTable_RowChanged);

            cultureDataTable.ColumnChanged += new DataColumnChangeEventHandler(dataTable_ColumnChanged);

            cultureDataTable.AcceptChanges();
            cultureDataTable.DefaultView.AllowDelete = false;
            cultureDataTable.DefaultView.AllowNew = false;
            isLoaded = true;
        }
        /// <summary>
        /// GetResourceSetTable gets the Hashtable of resource entries for a given ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet of resource entries</param>
        /// <returns>The Hashtable of resource entries for a given ResourceSet</returns>
        protected virtual Hashtable GetResourceSetTable(ResourceSet resourceSet)
        {
            FieldInfo fieldInfo =
                resourceSet.GetType().GetField("Table", BindingFlags.Instance | BindingFlags.NonPublic);
            if (fieldInfo != null)
            {
                object tableObject = fieldInfo.GetValue(resourceSet);
                if (tableObject != null)
                    return (Hashtable)tableObject;
            }
            return null;
        }
        /// <summary>
        /// ResourceSetToSortedList gets the SortedList of resource entries for a given ResourceSet
        /// </summary>
        /// <param name="resourceSet">The ResourceSet of resource entries</param>
        /// <returns>The SortedList of resource entries for a given ResourceSet</returns>
        protected virtual SortedList ResourceSetToSortedList(ResourceSet resourceSet)
        {
            SortedList sortedList = new SortedList();
            foreach (DictionaryEntry entry in resourceSet)
            {
                sortedList.Add(entry.Key, entry.Value);
            }
            return sortedList;
        }
        private bool GetMarkedAsTranslated(string key, DataSet oldDataSet)
        {
            if (oldDataSet == null)
                return false;

            DataRow[] dataRows = oldDataSet.Tables["Resources"].Select("ResourceKey='" + key + "'");
            if (dataRows.GetLength(0) == 0)
                // this key doesn't exist in the old DataSet
                return false;

            return dataRows[0]["MarkedAsTranslated"].ToString() == "true";
        }
        /// <summary>
        /// Save saves the changes to the resource entries back to their source
        /// </summary>
        /// <remarks>Save also saves the "marked as translated" file</remarks>
        public void Save()
        {
            cultureResourceGovernor.WriteResourceSet(cultureResourceSet);
            cultureDataTable.AcceptChanges();
            string oldDataSetFilename = GetTranslatedDataSetFilename();
            if (oldDataSetFilename != null)
                cultureDataTable.DataSet.WriteXml(oldDataSetFilename);
        }
        private void AddCultureRow(string resourceKey, string original, string translated,
            bool markedAsTranslated, string originalComment)
        {
            DataRow dataRow = cultureDataTable.NewRow();
            dataRow["ResourceKey"] = resourceKey;
            dataRow["Original"] = original;
            dataRow["Translated"] = translated;
            dataRow["MarkedAsTranslated"] = markedAsTranslated;
            dataRow["OriginalComment"] = originalComment;
            cultureDataTable.Rows.Add(dataRow);
        }
        private DataTable CreateDataTable()
        {
            DataColumn dataColumn;

            DataTable dataTable = new DataTable("Resources");

            dataColumn = dataTable.Columns.Add("ResourceKey");
            dataColumn.ReadOnly = true;

            dataColumn = dataTable.Columns.Add("Original");
            dataColumn.ReadOnly = true;

            dataTable.Columns.Add("Translated");

            dataTable.Columns.Add("MarkedAsTranslated", typeof(bool));

            dataColumn = dataTable.Columns.Add("OriginalComment");
            dataColumn.ReadOnly = true;

            DataSet dataSet = new DataSet("ResourcesCollection");
            dataSet.Tables.Add(dataTable);

            return dataTable;
        }
        private void dataTable_RowChanged(object sender, DataRowChangeEventArgs e)
        {
            if (ResourcesModified != null)
                ResourcesModified(this, EventArgs.Empty);
        }
        private void dataTable_ColumnChanged(object sender, DataColumnChangeEventArgs e)
        {
            if (e.Column.ColumnName == "Translated")
            {
                // find the corresponding dictionary entry and update its value
                if (cultureResourceTable.ContainsKey(e.Row["ResourceKey"]))
                {
                    ResourcesDataNodeInterpreter dataNodeInterpreter =
                        new ResourcesDataNodeInterpreter(e.Row["ResourceKey"].ToString(),
                        cultureResourceTable[e.Row["ResourceKey"]]);

                    dataNodeInterpreter.Value = e.ProposedValue;
                }
            }
        }
        /// <summary>
        /// Find finds a value in a given column name
        /// </summary>
        /// <param name="columnName">The name of the column to find the value in</param>
        /// <param name="value">The value to find</param>
        /// <returns>The index of a value in a given column name or -1 if the value is not found</returns>
        public virtual int Find(string columnName, string value)
        {
            int index = 0;
            string upperValue = value.ToUpper();
            foreach (DataRow dataRow in cultureDataTable.Rows)
            {
                if (dataRow[columnName].ToString().ToUpper().IndexOf(upperValue) != -1)
                    return index;
                index++;
            }
            return -1;
        }
        /// <summary>
        /// FindAgain finds a value in a given column name from a given index position
        /// </summary>
        /// <param name="columnName">The name of the column to find the value in</param>
        /// <param name="value">The value to find</param>
        /// <param name="index">The index position to start at</param>
        /// <returns>The index of a value in a given column name or -1 if the value is not found</returns>
        public virtual int FindAgain(string columnName, string value, int index)
        {
            string upperValue = value.ToUpper();
            for (index = index + 1; index < cultureDataTable.Rows.Count; index++)
            {
                DataRow dataRow = cultureDataTable.Rows[index];
                if (dataRow[columnName].ToString().ToUpper().IndexOf(upperValue) != -1)
                    return index;
            }
            return -1;
        }
        /// <summary>
        /// IsUntranslated returns true if the given DataRow has not been translated
        /// </summary>
        /// <param name="dataRow">The DataRow to check if it has been translated</param>
        /// <returns>True if the given DataRow has not been translated</returns>
        public static bool IsUntranslated(DataRow dataRow)
        {
            if ((bool)dataRow["MarkedAsTranslated"])
                return false;
            else
                return dataRow["Original"].ToString() == dataRow["Translated"].ToString();
        }
        /// <summary>
        /// FindUntranslated finds the first untranslated row
        /// </summary>
        /// <returns>The first untranslated row</returns>
        public virtual int FindUntranslated()
        {
            int index = 0;
            foreach (DataRow dataRow in cultureDataTable.Rows)
            {
                if (IsUntranslated(dataRow))
                    return index;
                index++;
            }
            return -1;
        }
        /// <summary>
        /// FindUntranslatedAgain finds the next untranslated row from a given row index
        /// </summary>
        /// <param name="index">The index to start the find from</param>
        /// <returns>The index of the next untranslated row from a given row index</returns>
        public virtual int FindUntranslatedAgain(int index)
        {
            for (index = index + 1; index < cultureDataTable.Rows.Count; index++)
            {
                DataRow dataRow = cultureDataTable.Rows[index];
                if (IsUntranslated(dataRow))
                    return index;
            }
            return -1;
        }
        /// <summary>
        /// MarkAsTranslated marks a given row as translated
        /// </summary>
        /// <param name="dataRow">The DataRow to mark</param>
        public static void MarkAsTranslated(DataRow dataRow)
        {
            dataRow["MarkedAsTranslated"] = true;
        }
        /// <summary>
        /// UnmarkAsTranslated unmarks a given row as translated
        /// </summary>
        /// <param name="dataRow">The DataRow to unmark</param>
        public static void UnmarkAsTranslated(DataRow dataRow)
        {
            dataRow["MarkedAsTranslated"] = false;
        }
        /// <summary>
        /// IsMarkedAsTranslated returns true if the given DataRow is marked as translated
        /// </summary>
        /// <param name="dataRow">The DataRow to check</param>
        /// <returns>True if the given DataRow is marked as translated</returns>
        public static bool IsMarkedAsTranslated(DataRow dataRow)
        {
            return (bool)dataRow["MarkedAsTranslated"];
        }
    }
}
