// File           : MainForm.cs
// Original Author: Guy Smith-Ferrier
// Date Created   : September 2005
// Notes          : This file is part of the downloadable source code for .NET Internationalization, by
//                  Guy Smith-Ferrier, published by Addison-Wesley. See http://www.dotneti18n.com for details.
// Disclaimer     : No warranty is provided. Use at your own risk.
// Modification   : You are free to modify this code providing that this block of comments is not altered
//                  and is placed before any code.
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Configuration;
using System.Xml;
using Internationalization.ResourceAdministration;
using System.IO;

namespace ResourceTranslationManager
{
    /// <summary>
    /// MainForm is the main form of the Resource Translation Manager
    /// </summary>
    public partial class MainForm : Form
    {
        private CultureInfo targetCultureInfo;
        private string resourcesGovernorClassName;
        private string source;
        private string markedAsTranslatedPath;
        private IResourcesGovernor resourcesGovernor;
        private TreeNode previousNode;
        private string findOriginalValue = null;
        private bool foldersVisible;
        private string rootFolder;

        /// <summary>
        /// Constructs a MainForm object
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
            FoldersVisible = false;
            if (! LoadConfiguration())
            {
                if (! SetSourceAndTargetLanguage())
                {
                    MessageBox.Show(MainFormResources.SourceOrTargetMissing);
                    System.Environment.Exit(-1);
                }
            }
            Initialize(source);
            if (resourcesGovernor.IsFileBased)
                LoadFolders();
            treeViewFolders.AfterSelect += new TreeViewEventHandler(treeViewFolders_AfterSelect);
        }
        private void Initialize(string currentSource)
        {
            InitializeResourcesGovernor(currentSource);
            LoadResources();
            UpdateStatusBar();
        }
        private bool LoadConfiguration()
        {
            bool configurationLoaded = true;
            AppSettingsReader appSettingsReader = new AppSettingsReader();

            string targetCulture = GetAppSettingsStringValue(appSettingsReader, "TargetCulture");
            if (targetCulture == String.Empty)
                configurationLoaded = false;
            else
                targetCultureInfo = new CultureInfo(targetCulture);

            resourcesGovernorClassName = GetAppSettingsStringValue(appSettingsReader, "ResourcesGovernor");
            if (resourcesGovernorClassName == String.Empty)
                resourcesGovernorClassName = "ResXResourcesGovernor";

            string markedAsTranslatedPathValue = GetAppSettingsStringValue(appSettingsReader, "MarkedAsTranslatedPath");
            if (markedAsTranslatedPathValue != String.Empty)
                markedAsTranslatedPath = markedAsTranslatedPathValue;
            else
                markedAsTranslatedPath = null;

            string sourceValue = GetAppSettingsStringValue(appSettingsReader, "Source");
            if (sourceValue == String.Empty)
                configurationLoaded = false;
            else
                source = sourceValue;

            return configurationLoaded;
        }
        private string GetAppSettingsStringValue(AppSettingsReader appSettingsReader, string key)
        {
            try
            {
                return appSettingsReader.GetValue(key, typeof(System.String)).ToString();
            }
            catch (InvalidOperationException)
            {
                return String.Empty;
            }
        }
        /// <summary>
        /// FoldersVisible determines whether the Folders treeview is visible
        /// </summary>
        public bool FoldersVisible
        {
            get { return foldersVisible; }
            set
            {
                foldersVisible = value;
                splitContainerFoldersResources.Panel1Collapsed = !foldersVisible;
            }
        }
        private void InitializeResourcesGovernor(string currentSource)
        {
            resourcesGovernor = ResourceAdministrationHelper.CreateResourcesGovernor(
                resourcesGovernorClassName, currentSource);

            if (String.Compare(markedAsTranslatedPath, "null", true, CultureInfo.InvariantCulture) == 0)
                markedAsTranslatedPath = null;
            else if (markedAsTranslatedPath == null)
            {
                // initialize it from the resource governor if possible
                if (resourcesGovernor.IsFileBased)
                    markedAsTranslatedPath = resourcesGovernor.ResourceFileAbsolutePath;
            }

            rootFolder = null;
            FoldersVisible = resourcesGovernor.IsFileBased;
        }
        private void LoadFolders()
        {
            treeViewFolders.Nodes.Clear();
            rootFolder = source;
            LoadFolder(rootFolder, treeViewFolders.Nodes);
            treeViewFolders.ExpandAll();
        }
        private void LoadFolder(string folder, TreeNodeCollection nodes)
        {
            TreeNode rootNode = nodes.Add(folder.Substring(Path.GetDirectoryName(folder).Length + 1));
            string[] directories = Directory.GetDirectories(folder);
            foreach (string directory in directories)
            {
                LoadFolder(directory, rootNode.Nodes);
            }
        }
        private void LoadResources()
        {
            treeViewResources.Nodes.Clear();
            string[] baseNames = resourcesGovernor.GetInvariantCultureBaseNames();
            foreach (string baseName in baseNames)
            {
                IResourceGovernor invariantResourceGovernor =
                    resourcesGovernor.GetResourceGovernor(baseName, CultureInfo.InvariantCulture);

                IResourceGovernor cultureResourceGovernor =
                    resourcesGovernor.GetResourceGovernor(baseName, targetCultureInfo);

                Resources resources =
                    new Resources(cultureResourceGovernor, invariantResourceGovernor, markedAsTranslatedPath);

                resources.ResourcesModified +=
                    new ResourcesModifiedEventHandler(resources_ResourcesModified);

                treeViewResources.Nodes.Add(new ResourcesTreeNode(baseName, resources));
            }
            if (treeViewResources.Nodes.Count > 0)
                treeViewResources.SelectedNode = treeViewResources.Nodes[0];
        }
        private bool SetSourceAndTargetLanguage()
        {
            ResourcesSourceForm form = new ResourcesSourceForm();
            form.ResourcesGovernorClassName = resourcesGovernorClassName;
            if (targetCultureInfo != null)
                form.TargetLanguage = targetCultureInfo.Name;
            form.Source = source;
            form.Initialize();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (targetCultureInfo == null || String.Compare(
                    targetCultureInfo.Name, form.TargetLanguage, true, CultureInfo.InvariantCulture) != 0)
                    targetCultureInfo = new CultureInfo(form.TargetLanguage);

                resourcesGovernorClassName = form.ResourcesGovernorClassName;
                source = form.Source;
                SaveConfiguration();
                return true;
            }
            return false;
        }
        private void UpdateStatusBar()
        {
            targetLanguagePanel.Text = String.Format(MainFormResources.TargetLanguage, targetCultureInfo.Name);
            sourcePanel.Text = String.Format(MainFormResources.Source, source);
        }
        private void AddSetting(XmlTextWriter writer, string elementName, string elementValue)
        {
            writer.WriteStartElement("add");
            writer.WriteAttributeString("key", elementName);
            writer.WriteAttributeString("value", elementValue);
            writer.WriteEndElement();
        }
        private void SaveConfiguration()
        {
            string configFileName = Application.ExecutablePath + ".config";
            XmlTextWriter writer = new XmlTextWriter(configFileName, System.Text.Encoding.Unicode);
            writer.Formatting = Formatting.Indented;
            writer.WriteStartElement("configuration");
            writer.WriteStartElement("appSettings");
            AddSetting(writer, "TargetLanguage", targetCultureInfo.Name);
            AddSetting(writer, "Source", source);
            AddSetting(writer, "ResourcesGovernor", resourcesGovernorClassName);
            writer.WriteEndElement();
            writer.WriteEndElement();
            writer.Close();
        }
        private void resources_ResourcesModified(object sender, EventArgs e)
        {
            saveToolStripMenuItem.Enabled = true;
        }
        private void treeViewResources_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node != null)
            {
                ResourcesTreeNode resourcesTreeNode = (ResourcesTreeNode) e.Node;
                if (!resourcesTreeNode.Resources.IsLoaded)
                    resourcesTreeNode.Resources.Load();

                DataTable dataTable = resourcesTreeNode.Resources.CultureDataTable;
                dataGridViewResourceEntries.DataSource = dataTable.DataSet;
                dataGridViewResourceEntries.DataMember = dataTable.TableName;

                dataGridViewResourceEntries.Columns["ResourceKey"].Width = 160;
                dataGridViewResourceEntries.Columns["ResourceKey"].HeaderText = MainFormResources.ResourceKey;
                dataGridViewResourceEntries.Columns["Original"].Width = 240;
                dataGridViewResourceEntries.Columns["Original"].HeaderText = MainFormResources.Original;
                dataGridViewResourceEntries.Columns["Translated"].Width = 240;
                dataGridViewResourceEntries.Columns["Translated"].HeaderText = MainFormResources.Translated;
                dataGridViewResourceEntries.Columns["MarkedAsTranslated"].Visible = false;
                if (resourcesGovernor.SupportsComments)
                {
                    dataGridViewResourceEntries.Columns["OriginalComment"].Width = 240;
                    dataGridViewResourceEntries.Columns["OriginalComment"].HeaderText = MainFormResources.Comment;
                }
                else
                    dataGridViewResourceEntries.Columns["OriginalComment"].Visible = false;

                groupBoxResourceEntries.Text = String.Format(MainFormResources.ResourceEntries, resourcesTreeNode.Text);
            }
            previousNode = e.Node;
        }
        private void SaveNode(TreeNode treeNode)
        {
            if (treeNode != null)
            {
                ResourcesTreeNode resourcesTreeNode = (ResourcesTreeNode)treeNode;
                if (resourcesTreeNode.Resources.HasChanges)
                    resourcesTreeNode.Resources.Save();

                saveToolStripMenuItem.Enabled = false;
            }
        }

        private void treeViewResources_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            SaveNode(previousNode);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveNode(treeViewResources.SelectedNode);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void dataGridViewResourceEntries_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            DataTable dataTable = ((DataSet)dataGridViewResourceEntries.DataSource).Tables[
                dataGridViewResourceEntries.DataMember];

            DataRow dataRow = dataTable.Rows[e.RowIndex];

            if (Resources.IsUntranslated(dataRow))
                // change the backBrush to the DataGrid's BackBrush
                e.CellStyle.BackColor = Color.Red;
        }

        private void resourcesSourceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (SetSourceAndTargetLanguage())
            {
                Initialize(source);
                if (resourcesGovernor.IsFileBased)
                    LoadFolders();
            }
        }
        private void Find(string findCategoryCaption, string findCategoryColumnName, ref string findValue)
        {
            ResourcesTreeNode resourceTreeNode = (ResourcesTreeNode)treeViewResources.SelectedNode;
            if (resourceTreeNode == null)
                MessageBox.Show(MainFormResources.MustSelectAResource);
            else
            {
                FindDialog findDialog;
                if (findValue == null)
                    findDialog = new FindDialog(findCategoryCaption, String.Empty);
                else
                    findDialog = new FindDialog(findCategoryCaption, findValue);

                if (findDialog.ShowDialog() == DialogResult.OK)
                {
                    findValue = findDialog.FindValue;
                    int index = resourceTreeNode.Resources.Find(findCategoryColumnName, findValue);
                    if (index == -1)
                        MessageBox.Show(String.Format(MainFormResources.FindValueNotFound, findCategoryCaption, findValue));
                    else
                        dataGridViewResourceEntries.CurrentCell = dataGridViewResourceEntries.Rows[index].Cells[0];
                }
            }
        }
        private void FindAgain(string findCategoryCaption, string findCategoryColumnName, string findValue)
        {
            ResourcesTreeNode resourceTreeNode = (ResourcesTreeNode)treeViewResources.SelectedNode;
            if (resourceTreeNode == null)
                MessageBox.Show(MainFormResources.MustSelectAResource);
            else if (findValue == null)
            {
                // there is nothing to do - there was no Find before this Find Again
            }
            else
            {
                int index =
                    resourceTreeNode.Resources.FindAgain(
                    findCategoryColumnName, findValue, dataGridViewResourceEntries.CurrentCell.RowIndex);
                if (index == -1)
                    MessageBox.Show(String.Format(MainFormResources.FindValueNotFound, findCategoryCaption, findValue));
                else
                    dataGridViewResourceEntries.CurrentCell = dataGridViewResourceEntries.Rows[index].Cells[0];
            }
        }

        private void findOriginalInThisResourceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Find(MainFormResources.Original, "Original", ref findOriginalValue);
        }

        private void findOriginalInThisResourceAgainToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FindAgain(MainFormResources.Original, "Original", findOriginalValue);
        }

        private void markAsTranslatedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (dataGridViewResourceEntries.CurrentCell.RowIndex > -1)
            {
                DataGridViewRow dataGridViewRow =
                    dataGridViewResourceEntries.Rows[dataGridViewResourceEntries.CurrentCell.RowIndex];
                DataRowView dataRowView = (DataRowView) dataGridViewRow.DataBoundItem;
                DataRow dataRow = (DataRow) dataRowView.Row;
                Resources.MarkAsTranslated(dataRow);
            }
        }

        private void unmarkAsTranslatedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (dataGridViewResourceEntries.CurrentCell.RowIndex > -1)
            {
                DataGridViewRow dataGridViewRow =
                    dataGridViewResourceEntries.Rows[dataGridViewResourceEntries.CurrentCell.RowIndex];
                DataRowView dataRowView = (DataRowView)dataGridViewRow.DataBoundItem;
                DataRow dataRow = (DataRow)dataRowView.Row;
                Resources.UnmarkAsTranslated(dataRow);
            }
        }
        /// <summary>
        /// CursorPositionToRow gets the row index for the current cursor position in a DataGridView
        /// </summary>
        /// <param name="dataGridView">The DataGridView to get the row index for</param>
        /// <returns>The row index for the current cursor position in a DataGridView</returns>
        public static int CursorPositionToRow(DataGridView dataGridView)
        {
            Point point = Cursor.Position;
            Point clientPoint = dataGridView.PointToClient(point);
            DataGridView.HitTestInfo hitTestInfo = dataGridView.HitTest(clientPoint.X, clientPoint.Y);
            return hitTestInfo.RowIndex;
        }
        private void dataGridViewResourceEntries_MouseDown(object sender, MouseEventArgs e)
        {
            int index = CursorPositionToRow(dataGridViewResourceEntries);
            if (index > -1)
            {
                dataGridViewResourceEntries.CurrentCell = dataGridViewResourceEntries.Rows[index].Cells[0];

                DataGridViewRow dataGridViewRow =
                    dataGridViewResourceEntries.Rows[dataGridViewResourceEntries.CurrentCell.RowIndex];
                DataRowView dataRowView = (DataRowView)dataGridViewRow.DataBoundItem;
                DataRow dataRow = (DataRow)dataRowView.Row;

                if (Resources.IsUntranslated(dataRow))
                {
                    markAsTranslatedToolStripMenuItem.Enabled = true;
                    unmarkAsTranslatedToolStripMenuItem.Enabled = false;
                }
                else if (Resources.IsMarkedAsTranslated(dataRow))
                {
                    markAsTranslatedToolStripMenuItem.Enabled = false;
                    unmarkAsTranslatedToolStripMenuItem.Enabled = true;
                }
                else
                {
                    markAsTranslatedToolStripMenuItem.Enabled = false;
                    unmarkAsTranslatedToolStripMenuItem.Enabled = false;
                }
            }
        }

        private void treeViewFolders_AfterSelect(object sender, TreeViewEventArgs e)
        {
            string path = String.Empty;
            TreeNode node = e.Node;
            while (node.Parent != null)
            {
                if (path == String.Empty)
                    path = node.Text;
                else
                    path = node.Text + Path.DirectorySeparatorChar + path;

                node = node.Parent;
            }
            Initialize(Path.Combine(source, path));
        }
    }
    /// <summary>
    /// ResourcesTreeNode is a TreeNode for a Resources object
    /// </summary>
    public class ResourcesTreeNode : TreeNode
    {
        private Resources resources;

        /// <summary>
        /// Constructs a ResourcesTreeNode object
        /// </summary>
        /// <param name="text">The text of the tree node</param>
        /// <param name="resources">The Resources object which is associated with the tree node</param>
        public ResourcesTreeNode(string text, Resources resources): base(text)
        {
            this.resources = resources;
        }
        /// <summary>
        /// Constructs a ResourcesTreeNode object
        /// </summary>
        /// <param name="text">The text of the tree node</param>
        public ResourcesTreeNode(string text)
            : base(text)
        {
        }
        /// <summary>
        /// Constructs a ResourcesTreeNode object
        /// </summary>
        /// <param name="text">The text of the tree node</param>
        /// <param name="children">A TreeNode array of child tree nodes</param>
        public ResourcesTreeNode(string text, TreeNode[] children): base(text, children)
        {
        }
        /// <summary>
        /// Constructs a ResourcesTreeNode object
        /// </summary>
        /// <param name="text">The text of the tree node</param>
        /// <param name="imageIndex">The index into an ImageList for the image of the TreeNode</param>
        /// <param name="selectedImageIndex">The index into an ImageList for the image of the TreeNode when
        /// it is selected</param>
        public ResourcesTreeNode(string text, int imageIndex, int selectedImageIndex)
            :base(text, imageIndex, selectedImageIndex)
        {
        }
        /// <summary>
        /// Constructs a ResourcesTreeNode object
        /// </summary>
        /// <param name="text">The text of the tree node</param>
        /// <param name="imageIndex">The index into an ImageList for the image of the TreeNode</param>
        /// <param name="selectedImageIndex">The index into an ImageList for the image of the TreeNode when
        /// it is selected</param>
        /// <param name="children">A TreeNode array of child tree nodes</param>
        public ResourcesTreeNode(string text, int imageIndex, int selectedImageIndex, TreeNode[] children)
            :
            base(text, imageIndex, selectedImageIndex, children)
        {
        }
        /// <summary>
        /// Resources is the Resources object which is associated with the tree node
        /// </summary>
        public Resources Resources
        {
            get { return resources; }
            set { resources = value; }
        }
    }
}